@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUILoadThread.cpp 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: GUILoadThread.cpp 18498 2015-06-22 05:59:25Z behrisch $
 ///
 // Class describing the thread that performs the loading of a simulation
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -80,7 +80,7 @@
 // ===========================================================================
 // member method definitions
 // ===========================================================================
-GUILoadThread::GUILoadThread(FXApp* app, MFXInterThreadEventClient* mw,
+GUILoadThread::GUILoadThread(FXApp* app, GUIApplicationWindow* mw,
                              MFXEventQue<GUIEvent*>& eq, FXEX::FXThreadEvent& ev)
     : FXSingleEventThread(app, mw), myParent(mw), myEventQue(eq),
       myEventThrow(ev) {
@@ -100,34 +100,35 @@ GUILoadThread::~GUILoadThread() {
 
 FXint
 GUILoadThread::run() {
-    GUINet* net = 0;
-    int simStartTime = 0;
-    int simEndTime = 0;
-    std::vector<std::string> guiSettingsFiles;
-    bool osgView = false;
-    OptionsCont& oc = OptionsCont::getOptions();
-
     // register message callbacks
     MsgHandler::getMessageInstance()->addRetriever(myMessageRetriever);
     MsgHandler::getErrorInstance()->addRetriever(myErrorRetriever);
-    if (!OptionsCont::getOptions().getBool("no-warnings")) {
-        MsgHandler::getWarningInstance()->addRetriever(myWarningRetriever);
-    }
+    MsgHandler::getWarningInstance()->addRetriever(myWarningRetriever);
 
     // try to load the given configuration
+    OptionsCont& oc = OptionsCont::getOptions();
     try {
         oc.clear();
         MSFrame::fillOptions();
         if (myFile != "") {
+            // triggered by menu option or reload
             if (myLoadNet) {
                 oc.set("net-file", myFile);
             } else {
                 oc.set("configuration-file", myFile);
             }
-            OptionsIO::getOptions(true, 1, 0);
+            oc.resetWritable(); // there may be command line options
+            OptionsIO::getOptions();
         } else {
-            OptionsIO::getOptions(true);
+            // triggered at application start
+            OptionsIO::getOptions();
+            if (oc.isSet("configuration-file")) {
+                myFile = oc.getString("configuration-file");
+            } else if (oc.isSet("net-file")) {
+                myFile = oc.getString("net-file");
+            }
         }
+        myTitle = myFile;
         // within gui-based applications, nothing is reported to the console
         MsgHandler::getMessageInstance()->removeRetriever(&OutputDevice::getDevice("stdout"));
         MsgHandler::getWarningInstance()->removeRetriever(&OutputDevice::getDevice("stderr"));
@@ -147,7 +148,7 @@ GUILoadThread::run() {
         // the options are not valid but maybe we want to quit
         GUIGlobals::gQuitOnEnd = oc.getBool("quit-on-end");
         MsgHandler::getErrorInstance()->inform("Quitting (on error).", false);
-        submitEndAndCleanup(net, simStartTime, simEndTime);
+        submitEndAndCleanup(0, 0, 0);
         return 0;
     }
 
@@ -156,6 +157,9 @@ GUILoadThread::run() {
     RandHelper::initRandGlobal(MSRouteHandler::getParsingRNG());
     MSFrame::setMSGlobals(oc);
     GUITexturesHelper::allowTextures(!oc.getBool("disable-textures"));
+    if (oc.getBool("game")) {
+        myParent->onCmdGaming(0, 0, 0);
+    }
     MSVehicleControl* vehControl = 0;
 #ifdef HAVE_INTERNAL
     GUIVisualizationSettings::UseMesoSim = MSGlobals::gUseMesoSim;
@@ -165,6 +169,11 @@ GUILoadThread::run() {
 #endif
         vehControl = new GUIVehicleControl();
 
+    GUINet* net = 0;
+    int simStartTime = 0;
+    int simEndTime = 0;
+    std::vector<std::string> guiSettingsFiles;
+    bool osgView = false;
     GUIEdgeControlBuilder* eb = 0;
     try {
         net = new GUINet(
@@ -224,7 +233,6 @@ GUILoadThread::run() {
 }
 
 
-
 void
 GUILoadThread::submitEndAndCleanup(GUINet* net,
                                    const SUMOTime simStartTime,
@@ -236,7 +244,7 @@ GUILoadThread::submitEndAndCleanup(GUINet* net,
     MsgHandler::getWarningInstance()->removeRetriever(myWarningRetriever);
     MsgHandler::getMessageInstance()->removeRetriever(myMessageRetriever);
     // inform parent about the process
-    GUIEvent* e = new GUIEvent_SimulationLoaded(net, simStartTime, simEndTime, myFile, guiSettingsFiles, osgView);
+    GUIEvent* e = new GUIEvent_SimulationLoaded(net, simStartTime, simEndTime, myTitle, guiSettingsFiles, osgView);
     myEventQue.add(e);
     myEventThrow.signal();
 }
@@ -246,6 +254,9 @@ void
 GUILoadThread::loadConfigOrNet(const std::string& file, bool isNet) {
     myFile = file;
     myLoadNet = isNet;
+    if (myFile != "") {
+        OptionsIO::setArgs(0, 0);
+    }
     start();
 }
 
@@ -264,6 +275,4 @@ GUILoadThread::getFileName() const {
 }
 
 
-
 /****************************************************************************/
-
