@@ -1,10 +1,10 @@
-/****************************************************************************/
+﻿/****************************************************************************/
 /// @file    GUITriggerBuilder.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 26.04.2004
-/// @version $Id: GUITriggerBuilder.cpp 18096 2015-03-17 09:50:59Z behrisch $
+/// @version $Id: GUITriggerBuilder.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // Builds trigger objects for guisim
 /****************************************************************************/
@@ -38,8 +38,10 @@
 #include <guisim/GUIBusStop.h>
 #include <guisim/GUIContainerStop.h>
 #include <guisim/GUICalibrator.h>
+#include <guisim/GUIChrgStn.h>
 #include "GUITriggerBuilder.h"
 
+
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
 #endif // CHECK_MEMORY_LEAKS
@@ -73,12 +75,12 @@ GUITriggerBuilder::buildRerouter(MSNet& net, const std::string& id,
     return rr;
 }
 
-
 void
 GUITriggerBuilder::buildBusStop(MSNet& net, const std::string& id,
                                 const std::vector<std::string>& lines,
                                 MSLane* lane,
                                 SUMOReal frompos, SUMOReal topos) {
+
     GUIBusStop* stop = new GUIBusStop(id, lines, *lane, frompos, topos);
     if (!net.addBusStop(stop)) {
         delete stop;
@@ -101,6 +103,21 @@ GUITriggerBuilder::buildContainerStop(MSNet& net, const std::string& id,
 }
 
 
+void
+GUITriggerBuilder::buildChrgStn(MSNet& net, const std::string& id,
+                                const std::vector<std::string>& lines,
+                                MSLane* lane,
+                                SUMOReal frompos, SUMOReal topos, SUMOReal chrgpower, SUMOReal efficiency, SUMOReal chargeInTransit, SUMOReal chargeDelay) {
+    GUIChrgStn* chrg = new GUIChrgStn(id, lines, *lane, frompos, topos, chrgpower, efficiency, chargeInTransit, chargeDelay);
+
+    if (!net.addChrgStn(chrg)) {
+        delete chrg;
+        throw InvalidArgument("Could not build charging station '" + id + "'; probably declared twice.");
+    }
+
+    static_cast<GUINet&>(net).getVisualisationSpeedUp().addAdditionalGLObject(chrg);
+}
+
 MSCalibrator*
 GUITriggerBuilder::buildCalibrator(MSNet& net, const std::string& id,
                                    MSEdge* edge, SUMOReal pos,
