@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Sept 2002
-/// @version $Id: GUIEdge.cpp 17271 2014-11-05 11:12:48Z namdre $
+/// @version $Id: GUIEdge.cpp 18467 2015-05-29 03:50:41Z behrisch $
 ///
 // A road/street connecting two junctions (gui-version)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -57,6 +57,7 @@
 #include "GUINet.h"
 #include "GUILane.h"
 #include "GUIPerson.h"
+#include "GUIContainer.h"
 
 #ifdef HAVE_INTERNAL
 #include <mesogui/GUIMEVehicleControl.h>
@@ -99,8 +100,8 @@ std::vector<GUIGlID>
 GUIEdge::getIDs(bool includeInternal) {
     std::vector<GUIGlID> ret;
     ret.reserve(MSEdge::myDict.size());
-    for (MSEdge::DictType::iterator i = MSEdge::myDict.begin(); i != MSEdge::myDict.end(); ++i) {
-        GUIEdge* edge = dynamic_cast<GUIEdge*>(i->second);
+    for (MSEdge::DictType::const_iterator i = MSEdge::myDict.begin(); i != MSEdge::myDict.end(); ++i) {
+        const GUIEdge* edge = dynamic_cast<const GUIEdge*>(i->second);
         assert(edge);
         if (edge->getPurpose() != EDGEFUNCTION_INTERNAL || includeInternal) {
             ret.push_back(edge->getGlID());
@@ -110,6 +111,20 @@ GUIEdge::getIDs(bool includeInternal) {
 }
 
 
+SUMOReal
+GUIEdge::getTotalLength(bool includeInternal, bool eachLane) {
+    SUMOReal result = 0;
+    for (MSEdge::DictType::const_iterator i = MSEdge::myDict.begin(); i != MSEdge::myDict.end(); ++i) {
+        const MSEdge* edge = i->second;
+        if (edge->getPurpose() != EDGEFUNCTION_INTERNAL || includeInternal) {
+            // @note needs to be change once lanes may have different length
+            result += edge->getLength() * (eachLane ? edge->getLanes().size() : 1);
+        }
+    }
+    return result;
+}
+
+
 Boundary
 GUIEdge::getBoundary() const {
     Boundary ret;
@@ -253,13 +268,22 @@ GUIEdge::drawGL(const GUIVisualizationSettings& s) const {
     }
     if (s.scale * s.personSize.getExaggeration(s) > s.personSize.minSize) {
         myLock.lock();
-        for (std::set<MSPerson*>::const_iterator i = myPersons.begin(); i != myPersons.end(); ++i) {
+        for (std::set<MSTransportable*>::const_iterator i = myPersons.begin(); i != myPersons.end(); ++i) {
             GUIPerson* person = dynamic_cast<GUIPerson*>(*i);
             assert(person != 0);
             person->drawGL(s);
         }
         myLock.unlock();
     }
+    if (s.scale * s.containerSize.getExaggeration(s) > s.containerSize.minSize) {
+        myLock.lock();
+        for (std::set<MSTransportable*>::const_iterator i = myContainers.begin(); i != myContainers.end(); ++i) {
+            GUIContainer* container = dynamic_cast<GUIContainer*>(*i);
+            assert(container != 0);
+            container->drawGL(s);
+        }
+        myLock.unlock();
+    }
 }
 
 
@@ -267,6 +291,7 @@ GUIEdge::drawGL(const GUIVisualizationSettings& s) const {
 void
 GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
     const GUIVisualizationTextSettings& nameSettings = s.vehicleName;
+    const SUMOReal exaggeration = s.vehicleSize.getExaggeration(s);
     GUIMEVehicleControl* vehicleControl = GUINet::getGUIInstance()->getGUIMEVehicleControl();
     if (vehicleControl != 0) {
         // draw the meso vehicles
@@ -275,16 +300,7 @@ GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
         MESegment::Queue queue;
         for (std::vector<MSLane*>::const_iterator msl = myLanes->begin(); msl != myLanes->end(); ++msl, ++laneIndex) {
             GUILane* l = static_cast<GUILane*>(*msl);
-            const PositionVector& shape = l->getShape();
-            const std::vector<SUMOReal>& shapeRotations = l->getShapeRotations();
-            const std::vector<SUMOReal>& shapeLengths = l->getShapeLengths();
-            const Position& laneBeg = shape[0];
-            glPushMatrix();
-            glTranslated(laneBeg.x(), laneBeg.y(), 0);
-            glRotated(shapeRotations[0], 0, 0, 1);
             // go through the vehicles
-            int shapeIndex = 0;
-            SUMOReal shapeOffset = 0; // ofset at start of current shape
             SUMOReal segmentOffset = 0; // offset at start of current segment
             for (MESegment* segment = MSGlobals::gMesoNet->getSegmentForEdge(*this);
                     segment != 0; segment = segment->getNextSegment()) {
@@ -293,43 +309,46 @@ GUIEdge::drawMesoVehicles(const GUIVisualizationSettings& s) const {
                     // make a copy so we don't have to worry about synchronization
                     queue = segment->getQueue(laneIndex);
                     const SUMOReal avgCarSize = segment->getBruttoOccupancy() / segment->getCarNumber();
+                    const SUMOReal avgCarHalfSize = 0.5 * avgCarSize;
                     const size_t queueSize = queue.size();
+                    SUMOReal vehiclePosition = segmentOffset + length;
+                    // draw vehicles beginning with the leader at the end of the segment
+                    SUMOReal xOff = 0;
                     for (size_t i = 0; i < queueSize; ++i) {
                         MSBaseVehicle* veh = queue[queueSize - i - 1];
+                        const SUMOReal vehLength = veh->getVehicleType().getLengthWithGap();
                         setVehicleColor(s, veh);
-                        SUMOReal vehiclePosition = segmentOffset + length - i * avgCarSize;
-                        SUMOReal xOff = 0.f;
                         while (vehiclePosition < segmentOffset) {
                             // if there is only a single queue for a
                             // multi-lane edge shift vehicles and start
                             // drawing again from the end of the segment
                             vehiclePosition += length;
-                            xOff += 0.5f;
-                        }
-                        while (shapeIndex < (int)shapeRotations.size() - 1 && vehiclePosition > shapeOffset + shapeLengths[shapeIndex]) {
-                            glPopMatrix();
-                            shapeOffset += shapeLengths[shapeIndex];
-                            shapeIndex++;
-                            glPushMatrix();
-                            glTranslated(shape[shapeIndex].x(), shape[shapeIndex].y(), 0);
-                            glRotated(shapeRotations[shapeIndex], 0, 0, 1);
+                            xOff += 2;
                         }
+                        const Position p = l->geometryPositionAtOffset(vehiclePosition);
+                        const SUMOReal angle = -l->getShape().rotationDegreeAtOffset(l->interpolateLanePosToGeometryPos(vehiclePosition));
                         glPushMatrix();
-                        glTranslated(xOff, -(vehiclePosition - shapeOffset), GLO_VEHICLE);
-                        glPushMatrix();
-                        glScaled(1, avgCarSize, 1);
+                        glTranslated(p.x(), p.y(), 0);
+                        glRotated(angle, 0, 0, 1);
+                        glTranslated(xOff, 0, GLO_VEHICLE);
+                        glScaled(exaggeration, vehLength * exaggeration, 1);
                         glBegin(GL_TRIANGLES);
                         glVertex2d(0, 0);
                         glVertex2d(0 - 1.25, 1);
                         glVertex2d(0 + 1.25, 1);
                         glEnd();
                         glPopMatrix();
-                        glPopMatrix();
                         if (nameSettings.show) {
+                            glPushMatrix();
+                            glRotated(angle, 0, 0, 1);
+                            glTranslated(xOff, 0, 0);
+                            glRotated(-angle, 0, 0, 1);
                             GLHelper::drawText(veh->getID(),
-                                               Position(xOff, -(vehiclePosition - shapeOffset)),
-                                               GLO_MAX, nameSettings.size / s.scale, nameSettings.color, 0);
+                                               l->geometryPositionAtOffset(vehiclePosition - 0.5 * vehLength),
+                                               GLO_MAX, nameSettings.size / s.scale, nameSettings.color);
+                            glPopMatrix();
                         }
+                        vehiclePosition -= vehLength;
                     }
                 }
                 segmentOffset += length;
@@ -490,7 +509,7 @@ GUIEdge::getVehicleColorValue(size_t activeScheme, MSBaseVehicle* veh) const {
         case 10:
             return 0; // invalid getLastLaneChangeOffset();
         case 11:
-            return MIN2(veh->getMaxSpeed(), getVehicleMaxSpeed(veh));
+            return getVehicleMaxSpeed(veh);
         case 12:
             return 0; // invalid getCO2Emissions();
         case 13:
