@@ -5,7 +5,7 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GUIEdge.h 18096 2015-03-17 09:50:59Z behrisch $
+/// @version $Id: GUIEdge.h 18274 2015-04-22 15:00:44Z behrisch $
 ///
 // A road/street connecting two junctions (gui-version)
 /****************************************************************************/
@@ -144,23 +144,23 @@ public:
     //@}
 
 
-    void addPerson(MSPerson* p) const {
+    void addPerson(MSTransportable* p) const {
         AbstractMutex::ScopedLocker locker(myLock);
         MSEdge::addPerson(p);
     }
 
-    void removePerson(MSPerson* p) const {
+    void removePerson(MSTransportable* p) const {
         AbstractMutex::ScopedLocker locker(myLock);
         MSEdge::removePerson(p);
     }
 
 
-    void addContainer(MSContainer* c) const {
+    void addContainer(MSTransportable* c) const {
         AbstractMutex::ScopedLocker locker(myLock);
         MSEdge::addContainer(c);
     }
 
-    void removeContainer(MSContainer* c) const {
+    void removeContainer(MSTransportable* c) const {
         AbstractMutex::ScopedLocker locker(myLock);
         MSEdge::removeContainer(c);
     }
