@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 25.07.2005
-/// @version $Id: GUITriggeredRerouter.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUITriggeredRerouter.cpp 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Reroutes vehicles passing an edge (gui version)
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -228,27 +228,16 @@ GUITriggeredRerouter::GUITriggeredRerouterPopupMenu::onCmdOpenManip(FXObject*,
  * ----------------------------------------------------------------------- */
 GUITriggeredRerouter::GUITriggeredRerouter(
     const std::string& id,
-    const std::vector<MSEdge*>& edges,
+    const MSEdgeVector& edges,
     SUMOReal prob, const std::string& aXMLFilename, bool off,
     SUMORTree& rtree) :
     MSTriggeredRerouter(id, edges, prob, aXMLFilename, off),
     GUIGlObject_AbstractAdd("rerouter", GLO_TRIGGER, id) {
     // add visualisation objects for edges which trigger the rerouter
-    for (std::vector<MSEdge*>::const_iterator it = edges.begin(); it != edges.end(); ++it) {
+    for (MSEdgeVector::const_iterator it = edges.begin(); it != edges.end(); ++it) {
         myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(dynamic_cast<GUIEdge*>(*it), this, false));
-    }
-    // add visualisation objects for closed edges
-    for (std::vector<RerouteInterval>::const_iterator it_interval = myIntervals.begin();
-            it_interval != myIntervals.end(); ++it_interval) {
-        const std::vector<MSEdge*>& closed = it_interval->closed;
-        for (std::vector<MSEdge*>::const_iterator it = closed.begin(); it != closed.end(); ++it) {
-            myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(dynamic_cast<GUIEdge*>(*it), this, true));
-        }
-    }
-    // register with rtree and rerouter boundary
-    for (std::vector<GUITriggeredRerouterEdge*>::iterator it = myEdgeVisualizations.begin(); it != myEdgeVisualizations.end(); ++it) {
-        rtree.addAdditionalGLObject(*it);
-        myBoundary.add((*it)->getCenteringBoundary());
+        rtree.addAdditionalGLObject(myEdgeVisualizations.back());
+        myBoundary.add(myEdgeVisualizations.back()->getCenteringBoundary());
     }
 }
 
@@ -261,6 +250,21 @@ GUITriggeredRerouter::~GUITriggeredRerouter() {
 }
 
 
+void
+GUITriggeredRerouter::myEndElement(int element) {
+    MSTriggeredRerouter::myEndElement(element);
+    if (element == SUMO_TAG_INTERVAL) {
+        // add visualisation objects for closed edges
+        const RerouteInterval& ri = myIntervals.back();
+        for (MSEdgeVector::const_iterator it = ri.closed.begin(); it != ri.closed.end(); ++it) {
+            myEdgeVisualizations.push_back(new GUITriggeredRerouterEdge(dynamic_cast<GUIEdge*>(*it), this, true));
+            dynamic_cast<GUINet*>(GUINet::getInstance())->getVisualisationSpeedUp().addAdditionalGLObject(myEdgeVisualizations.back());
+            myBoundary.add(myEdgeVisualizations.back()->getCenteringBoundary());
+        }
+    }
+}
+
+
 GUIGLObjectPopupMenu*
 GUITriggeredRerouter::getPopUpMenu(GUIMainWindow& app,
                                    GUISUMOAbstractView& parent) {
@@ -354,11 +358,11 @@ GUITriggeredRerouter::GUITriggeredRerouterEdge::drawGL(const GUIVisualizationSet
         const SUMOReal prob = myParent->getProbability();
         if (myAmClosedEdge) {
             // draw closing symbol onto all lanes
-            if (myParent->hasCurrentReroute(MSNet::getInstance()->getCurrentTimeStep()) && prob > 0) {
-                const RerouteInterval& ri =
-                    myParent->getCurrentReroute(MSNet::getInstance()->getCurrentTimeStep());
+            const RerouteInterval* const ri =
+                myParent->getCurrentReroute(MSNet::getInstance()->getCurrentTimeStep());
+            if (ri != 0 && prob > 0) {
                 // draw only if the edge is closed at this time
-                if (std::find(ri.closed.begin(), ri.closed.end(), myEdge) != ri.closed.end()) {
+                if (std::find(ri->closed.begin(), ri->closed.end(), myEdge) != ri->closed.end()) {
                     const size_t noLanes = myFGPositions.size();
                     for (size_t j = 0; j < noLanes; ++j) {
                         Position pos = myFGPositions[j];
