@@ -4,7 +4,7 @@
 /// @author  Laura Bieker
 /// @author  Michael Behrisch
 /// @date    Feb 2013
-/// @version $Id: ROMAAssignments.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: ROMAAssignments.h 18675 2015-08-20 20:00:26Z behrisch $
 ///
 // Assignment methods
 /****************************************************************************/
@@ -58,14 +58,22 @@ class ROVehicle;
 class ROMAAssignments {
 public:
     /// Constructor
-    ROMAAssignments(const SUMOTime begin, const SUMOTime end, RONet& net, ODMatrix& matrix, SUMOAbstractRouter<ROEdge, ROVehicle>& router);
+    ROMAAssignments(const SUMOTime begin, const SUMOTime end, const bool timeSplit,
+                    RONet& net, ODMatrix& matrix, SUMOAbstractRouter<ROEdge, ROVehicle>& router);
 
     /// Destructor
     ~ROMAAssignments();
 
+    ROVehicle* getDefaultVehicle() {
+        return myDefaultVehicle;
+    }
+
     // @brief calculate edge travel time with the given road class and max link speed
     SUMOReal capacityConstraintFunction(const ROEdge* edge, const SUMOReal flow) const;
 
+    // @brief clear effort storage
+    void resetFlows();
+
     // @brief incremental method
     void incremental(const int numIter);
 
@@ -124,11 +132,12 @@ private:
 private:
     const SUMOTime myBegin;
     const SUMOTime myEnd;
+    const bool myTimeSplit;
     RONet& myNet;
     ODMatrix& myMatrix;
     SUMOAbstractRouter<ROEdge, ROVehicle>& myRouter;
     static std::map<const ROEdge* const, SUMOReal> myPenalties;
-    static ROVehicle* myDefaultVehicle;
+    ROVehicle* myDefaultVehicle;
 
 private:
     /// @brief Invalidated assignment operator
