@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 8 Nov 2010
-/// @version $Id: MSBaseVehicle.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSBaseVehicle.h 18663 2015-08-19 13:02:24Z namdre $
 ///
 // A base class for vehicle implementations
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2010-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2010-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -139,6 +139,14 @@ public:
     }
 
 
+    /** @brief Returns an iterator pointing to the current edge in this vehicles route
+     * @return The current route pointer
+     */
+    const MSRouteIterator& getCurrentRouteEdge() const {
+        return myCurrEdge;
+    }
+
+
     /** @brief Performs a rerouting using the given router
      *
      * Tries to find a new route between the current edge and the destination edge, first.
@@ -148,7 +156,7 @@ public:
      * @param[in] router The router to use
      * @see replaceRoute
      */
-    void reroute(SUMOTime t, SUMOAbstractRouter<MSEdge, SUMOVehicle>& router, bool withTaz = false);
+    void reroute(SUMOTime t, SUMOAbstractRouter<MSEdge, SUMOVehicle>& router, const bool onInit = false, const bool withTaz = false);
 
 
     /** @brief Replaces the current route by the given edges
@@ -161,7 +169,7 @@ public:
      * @param[in] simTime The time at which the route was replaced
      * @return Whether the new route was accepted
      */
-    bool replaceRouteEdges(MSEdgeVector& edges, bool onInit = false);
+    bool replaceRouteEdges(ConstMSEdgeVector& edges, bool onInit = false);
 
 
     /** @brief Returns the vehicle's acceleration
@@ -200,6 +208,12 @@ public:
         return myArrivalPos;
     }
 
+    /** @brief Sets this vehicle's desired arrivalPos for its current route
+     */
+    virtual void setArrivalPos(SUMOReal arrivalPos) {
+        myArrivalPos = arrivalPos;
+    }
+
     /** @brief Returns whether this vehicle has already departed
      */
     bool hasDeparted() const;
@@ -233,7 +247,16 @@ public:
      *
      * @param[in] person The person to add
      */
-    virtual void addPerson(MSPerson* person);
+    virtual void addPerson(MSTransportable* person);
+
+
+    /** @brief Adds a container to this vehicle
+     *
+     * The default implementation does nothing since containers are not supported by default
+     *
+     * @param[in] container The container to add
+     */
+    virtual void addContainer(MSTransportable* container);
 
     /** @brief Validates the current route
      * @param[out] msg Description why the route is not valid (if it is the case)
@@ -308,6 +331,10 @@ protected:
      */
     void calculateArrivalPos();
 
+    /** @brief Returns the list of still pending stop edges
+     */
+    virtual const ConstMSEdgeVector getStopEdges() const = 0;
+
 protected:
     /// @brief This Vehicle's parameter.
     const SUMOVehicleParameter* myParameter;
