@@ -8,12 +8,12 @@
 /// @author  Michael Behrisch
 /// @author  Sascha Krieg
 /// @date    Tue, 06 Mar 2001
-/// @version $Id: MSEdge.cpp 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: MSEdge.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // A road/street connecting two junctions
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -45,8 +45,9 @@
 #include "MSLaneChanger.h"
 #include "MSGlobals.h"
 #include "MSVehicle.h"
-#include "MSPerson.h"
+#include "MSContainer.h"
 #include "MSEdgeWeightsStorage.h"
+#include <microsim/devices/MSDevice_Routing.h>
 
 #ifdef HAVE_INTERNAL
 #include <mesosim/MELoop.h>
@@ -63,7 +64,7 @@
 // static member definitions
 // ===========================================================================
 MSEdge::DictType MSEdge::myDict;
-std::vector<MSEdge*> MSEdge::myEdges;
+MSEdgeVector MSEdge::myEdges;
 
 
 // ===========================================================================
@@ -173,6 +174,7 @@ MSEdge::rebuildAllowedLanes() {
         }
     }
     myClassedAllowed.clear();
+    myClassesSuccessorMap.clear();
     // rebuild myMinimumPermissions and myCombinedPermissions
     myMinimumPermissions = SVCAll;
     myCombinedPermissions = 0;
@@ -235,7 +237,7 @@ MSEdge::getAllowedLanesWithDefault(const AllowedLanesCont& c, const MSEdge* dest
 
 const std::vector<MSLane*>*
 MSEdge::allowedLanes(const MSEdge* destination, SUMOVehicleClass vclass) const {
-    if ((myMinimumPermissions & vclass) == vclass) {
+    if (destination == 0 && (myMinimumPermissions & vclass) == vclass) {
         // all lanes allow vclass
         return getAllowedLanesWithDefault(myAllowed, destination);
     }
@@ -248,16 +250,33 @@ MSEdge::allowedLanes(const MSEdge* destination, SUMOVehicleClass vclass) const {
     } else {
         // this vclass is requested for the first time. rebuild all destinations
         // go through connected edges
+#ifdef HAVE_FOX
+        if (MSDevice_Routing::isParallel()) {
+            MSDevice_Routing::lock();
+        }
+#endif
         for (AllowedLanesCont::const_iterator i1 = myAllowed.begin(); i1 != myAllowed.end(); ++i1) {
             const MSEdge* edge = i1->first;
             const std::vector<MSLane*>* lanes = i1->second;
             myClassedAllowed[vclass][edge] = new std::vector<MSLane*>();
             // go through lanes approaching current edge
             for (std::vector<MSLane*>::const_iterator i2 = lanes->begin(); i2 != lanes->end(); ++i2) {
-                // allows the current vehicle class?
+                // origin lane allows the current vehicle class?
                 if ((*i2)->allowsVehicleClass(vclass)) {
-                    // -> may be used
-                    myClassedAllowed[vclass][edge]->push_back(*i2);
+                    if (edge == 0) {
+                        myClassedAllowed[vclass][edge]->push_back(*i2);
+                    } else {
+                        // target lane allows the current vehicle class?
+                        const MSLinkCont& lc = (*i2)->getLinkCont();
+                        for (MSLinkCont::const_iterator it_link = lc.begin(); it_link != lc.end(); ++it_link) {
+                            const MSLane* targetLane = (*it_link)->getLane();
+                            if ((&(targetLane->getEdge()) == edge) && targetLane->allowsVehicleClass(vclass)) {
+                                // -> may be used
+                                myClassedAllowed[vclass][edge]->push_back(*i2);
+                                break;
+                            }
+                        }
+                    }
                 }
             }
             // assert that 0 is returned if no connection is allowed for a class
@@ -266,6 +285,11 @@ MSEdge::allowedLanes(const MSEdge* destination, SUMOVehicleClass vclass) const {
                 myClassedAllowed[vclass][edge] = 0;
             }
         }
+#ifdef HAVE_FOX
+        if (MSDevice_Routing::isParallel()) {
+            MSDevice_Routing::unlock();
+        }
+#endif
         return myClassedAllowed[vclass][destination];
     }
 }
@@ -426,14 +450,20 @@ MSEdge::insertVehicle(SUMOVehicle& v, SUMOTime time, const bool checkOnly) const
     UNUSED_PARAMETER(time);
 #endif
     if (checkOnly) {
+        if (v.getEdge()->getPurpose() == MSEdge::EDGEFUNCTION_DISTRICT) {
+            return true;
+        }
         switch (v.getParameter().departLaneProcedure) {
             case DEPART_LANE_GIVEN:
             case DEPART_LANE_DEFAULT:
-            case DEPART_LANE_FIRST_ALLOWED:
-                return getDepartLane(static_cast<MSVehicle&>(v))->getBruttoOccupancy() * myLength + v.getVehicleType().getLengthWithGap() <= myLength;
+            case DEPART_LANE_FIRST_ALLOWED: {
+                const SUMOReal occupancy = getDepartLane(static_cast<MSVehicle&>(v))->getBruttoOccupancy();
+                return occupancy == (SUMOReal)0 || occupancy * myLength + v.getVehicleType().getLengthWithGap() <= myLength;
+            }
             default:
                 for (std::vector<MSLane*>::const_iterator i = myLanes->begin(); i != myLanes->end(); ++i) {
-                    if ((*i)->getBruttoOccupancy() * myLength + v.getVehicleType().getLengthWithGap() <= myLength) {
+                    const SUMOReal occupancy = (*i)->getBruttoOccupancy();
+                    if (occupancy == (SUMOReal)0 || occupancy * myLength + v.getVehicleType().getLengthWithGap() <= myLength) {
                         return true;
                     }
                 }
@@ -583,7 +613,7 @@ MSEdge::insertIDs(std::vector<std::string>& into) {
 
 
 void
-MSEdge::parseEdgesList(const std::string& desc, std::vector<const MSEdge*>& into,
+MSEdge::parseEdgesList(const std::string& desc, ConstMSEdgeVector& into,
                        const std::string& rid) {
     if (desc[0] == BinaryFormatter::BF_ROUTE) {
         std::istringstream in(desc, std::ios::binary);
@@ -598,7 +628,7 @@ MSEdge::parseEdgesList(const std::string& desc, std::vector<const MSEdge*>& into
 
 
 void
-MSEdge::parseEdgesList(const std::vector<std::string>& desc, std::vector<const MSEdge*>& into,
+MSEdge::parseEdgesList(const std::vector<std::string>& desc, ConstMSEdgeVector& into,
                        const std::string& rid) {
     for (std::vector<std::string>::const_iterator i = desc.begin(); i != desc.end(); ++i) {
         const MSEdge* edge = MSEdge::dictionary(*i);
@@ -636,23 +666,68 @@ MSEdge::getVehicleMaxSpeed(const SUMOVehicle* const veh) const {
 }
 
 
-std::vector<MSPerson*>
+std::vector<MSTransportable*>
 MSEdge::getSortedPersons(SUMOTime timestep) const {
-    std::vector<MSPerson*> result(myPersons.begin(), myPersons.end());
-    sort(result.begin(), result.end(), person_by_offset_sorter(timestep));
+    std::vector<MSTransportable*> result(myPersons.begin(), myPersons.end());
+    sort(result.begin(), result.end(), transportable_by_position_sorter(timestep));
+    return result;
+}
+
+
+std::vector<MSTransportable*>
+MSEdge::getSortedContainers(SUMOTime timestep) const {
+    std::vector<MSTransportable*> result(myContainers.begin(), myContainers.end());
+    sort(result.begin(), result.end(), transportable_by_position_sorter(timestep));
     return result;
 }
 
 
 int
-MSEdge::person_by_offset_sorter::operator()(const MSPerson* const p1, const MSPerson* const p2) const {
-    const SUMOReal pos1 = p1->getCurrentStage()->getEdgePos(myTime);
-    const SUMOReal pos2 = p2->getCurrentStage()->getEdgePos(myTime);
+MSEdge::transportable_by_position_sorter::operator()(const MSTransportable* const c1, const MSTransportable* const c2) const {
+    const SUMOReal pos1 = c1->getCurrentStage()->getEdgePos(myTime);
+    const SUMOReal pos2 = c2->getCurrentStage()->getEdgePos(myTime);
     if (pos1 != pos2) {
         return pos1 < pos2;
     }
-    return p1->getID() < p2->getID();
+    return c1->getID() < c2->getID();
 }
 
+const MSEdgeVector&
+MSEdge::getSuccessors(SUMOVehicleClass vClass) const {
+    if (vClass == SVC_IGNORING || !MSNet::getInstance()->hasPermissions()) {
+        return mySuccessors;
+    }
+#ifdef HAVE_FOX
+    if (MSDevice_Routing::isParallel()) {
+        MSDevice_Routing::lock();
+    }
+#endif
+    std::map<SUMOVehicleClass, MSEdgeVector>::const_iterator i = myClassesSuccessorMap.find(vClass);
+    if (i != myClassesSuccessorMap.end()) {
+        // can use cached value
+#ifdef HAVE_FOX
+        if (MSDevice_Routing::isParallel()) {
+            MSDevice_Routing::unlock();
+        }
+#endif
+        return i->second;
+    } else {
+        // this vClass is requested for the first time. rebuild all successors
+        for (MSEdgeVector::const_iterator it = mySuccessors.begin(); it != mySuccessors.end(); ++it) {
+            const std::vector<MSLane*>* allowed = allowedLanes(*it, vClass);
+            if (allowed != 0 && allowed->size() > 0) {
+                myClassesSuccessorMap[vClass].push_back(*it);
+            }
+        }
+#ifdef HAVE_FOX
+        if (MSDevice_Routing::isParallel()) {
+            MSDevice_Routing::unlock();
+        }
+#endif
+        return myClassesSuccessorMap[vClass];
+    }
+}
+
+
 /****************************************************************************/
 
