@@ -6,7 +6,7 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Mon, 12 Mar 2001
-/// @version $Id: MSInsertionControl.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSInsertionControl.cpp 18473 2015-06-03 21:35:22Z namdre $
 ///
 // Inserts vehicles into the network when their departure time is reached
 /****************************************************************************/
@@ -141,6 +141,11 @@ MSInsertionControl::tryInsert(SUMOTime time, SUMOVehicle* veh,
                               MSVehicleContainer::VehicleVector& refusedEmits) {
     assert(veh->getParameter().depart < time + DELTA_T);
     const MSEdge& edge = *veh->getEdge();
+    if (veh->isOnRoad()) {
+        // may have been inserted forcefully already
+        veh->onDepart();
+        return 1;
+    }
     if ((!myCheckEdgesOnce || edge.getLastFailedInsertionTime() != time) && edge.insertVehicle(*veh, time)) {
         // Successful insertion
         checkFlowWait(veh);
