@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: MSLink.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSLink.h 18319 2015-04-29 06:49:01Z namdre $
 ///
 // A connnection between lanes
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -141,8 +141,9 @@ public:
      * @param[in] dir The direction of this link
      * @param[in] state The state of this link
      * @param[in] length The length of this link
+     * @param[in] keepClear Whether the junction after this link must be kept clear
      */
-    MSLink(MSLane* succLane, LinkDirection dir, LinkState state, SUMOReal length);
+    MSLink(MSLane* succLane, LinkDirection dir, LinkState state, SUMOReal length, bool keepClear);
 #else
     /** @brief Constructor for simulation which uses internal lanes
      *
@@ -152,7 +153,7 @@ public:
      * @param[in] state The state of this link
      * @param[in] length The length of this link
      */
-    MSLink(MSLane* succLane, MSLane* via, LinkDirection dir, LinkState state, SUMOReal length);
+    MSLink(MSLane* succLane, MSLane* via, LinkDirection dir, LinkState state, SUMOReal length, bool keepClear);
 #endif
 
 
@@ -250,6 +251,12 @@ public:
     }
 
 
+    //@brief Returns the time of the last state change
+    inline SUMOTime getLastStateChange() const {
+        return myLastStateChange;
+    }
+
+
     /** @brief Returns the direction the vehicle passing this link take
      *
      * @return The direction of this link
@@ -260,6 +267,7 @@ public:
     /** @brief Sets the current tl-state
      *
      * @param[in] state The current state of the link
+     * @param[in] t The time of the state change
      */
     void setTLState(LinkState state, SUMOTime t);
 
@@ -271,6 +279,13 @@ public:
     MSLane* getLane() const;
 
 
+    /** @brief Returns the lane leading to this link
+     *
+     * @return The lane leading to this link
+     */
+    MSLane* getApproachingLane() const;
+
+
     /** @brief Returns the respond index (for visualization)
      *
      * @return The respond index for this link
@@ -317,6 +332,11 @@ public:
     }
 
 
+    /// @brief whether the junction after this link must be kept clear
+    bool keepClear() const {
+        return myKeepClear;
+    }
+
     /// @brief whether this is a link past an internal junction which currently has priority
     bool lastWasContMajor() const;
 
@@ -365,7 +385,7 @@ private:
     static bool maybeOccupied(MSLane* lane);
 
 private:
-    /// @brief The lane approached by this link
+    /// @brief The lane (but the internal one) approached by this link
     MSLane* myLane;
 
     std::map<const SUMOVehicle*, ApproachingVehicleInformation> myApproachingVehicles;
@@ -377,6 +397,9 @@ private:
     /// @brief The state of the link
     LinkState myState;
 
+    /// @brief The time of the last state change
+    SUMOTime myLastStateChange;
+
     /// @brief An abstract (hopefully human readable) definition of the link's direction
     LinkDirection myDirection;
 
@@ -388,10 +411,15 @@ private:
 
     bool myAmCont;
 
+    bool myKeepClear;
+
 #ifdef HAVE_INTERNAL_LANES
     /// @brief The following junction-internal lane if used
     MSLane* const myJunctionInlane;
 
+    /// @brief The preceding junction-internal lane if used
+    const MSLane* myInternalLaneBefore;
+
     /* @brief lengths after the crossing point with foeLane
      * (lengthOnThis, lengthOnFoe)
      * (index corresponds to myFoeLanes)
@@ -404,7 +432,7 @@ private:
     const MSJunction* myJunction;
 
     std::vector<MSLink*> myFoeLinks;
-    std::vector<MSLane*> myFoeLanes;
+    std::vector<const MSLane*> myFoeLanes;
     static SUMOTime myLookaheadTime;
 
 
