@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Wed, 12 Dez 2001
-/// @version $Id: MSLogicJunction.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: MSLogicJunction.h 18635 2015-08-14 13:33:44Z namdre $
 ///
 // with one ore more logics.
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -61,13 +61,8 @@ public:
     /// Destructor.
     virtual ~MSLogicJunction();
 
-    /** @brief Container for link foes */
-    typedef std::bitset<64> LinkFoes;
-
-    /** @brief Container for junction-internal lane occupation
-        Each element of this container represents one particular
-        junction-internal lane */
-    typedef std::bitset<64> InnerState;
+    /** @brief Container for link response and foes */
+    typedef std::bitset<64> LinkBits;
 
     /// initialises the junction after the whole net has been loaded
     virtual void postloadInit();
