@@ -10,12 +10,12 @@
 /// @author  Mario Krumnow
 /// @author  Christoph Sommer
 /// @date    Tue, 06 Mar 2001
-/// @version $Id: MSNet.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSNet.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // The simulated network and simulation perfomer
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -47,6 +47,7 @@
 #include <utils/common/UtilExceptions.h>
 #include "MSNet.h"
 #include "MSPersonControl.h"
+#include "MSContainerControl.h"
 #include "MSEdgeControl.h"
 #include "MSJunctionControl.h"
 #include "MSInsertionControl.h"
@@ -76,6 +77,9 @@
 #include <utils/iodevices/OutputDevice_File.h>
 #include "output/MSFCDExport.h"
 #include "output/MSEmissionExport.h"
+
+#include "output/MSBatteryExport.h"
+
 #include "output/MSFullExport.h"
 #include "output/MSQueueExport.h"
 #include "output/MSVTKExport.h"
@@ -87,9 +91,11 @@
 #include <utils/options/OptionsCont.h>
 #include <utils/vehicle/PedestrianRouter.h>
 #include "MSGlobals.h"
-#include "MSPModel.h"
+#include <microsim/pedestrians/MSPModel.h>
+#include <microsim/MSCModel_NonInteracting.h>
 #include <utils/geom/GeoConvHelper.h>
-#include "MSPerson.h"
+#include <microsim/pedestrians/MSPerson.h>
+#include "MSContainer.h"
 #include "MSEdgeWeightsStorage.h"
 #include "MSStateHandler.h"
 
@@ -165,7 +171,9 @@ MSNet::MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
              MSEventControl* endOfTimestepEvents, MSEventControl* insertionEvents,
              ShapeContainer* shapeCont):
     myVehiclesMoved(0),
-    myHaveRestrictions(false),
+    myHavePermissions(false),
+    myHasInternalLinks(false),
+    myHasElevation(false),
     myRouterTTInitialized(false),
     myRouterTTDijkstra(0),
     myRouterTTAStar(0),
@@ -187,6 +195,7 @@ MSNet::MSNet(MSVehicleControl* vc, MSEventControl* beginOfTimestepEvents,
     myRouteLoaders = 0;
     myLogics = 0;
     myPersonControl = 0;
+    myContainerControl = 0;
     myEdgeWeights = 0;
     myShapeContainer = shapeCont == 0 ? new ShapeContainer() : shapeCont;
 
@@ -211,7 +220,8 @@ MSNet::closeBuilding(MSEdgeControl* edges, MSJunctionControl* junctions,
                      SUMORouteLoaderControl* routeLoaders,
                      MSTLLogicControl* tlc,
                      std::vector<SUMOTime> stateDumpTimes,
-                     std::vector<std::string> stateDumpFiles) {
+                     std::vector<std::string> stateDumpFiles,
+                     bool hasInternalLinks) {
     myEdges = edges;
     myJunctions = junctions;
     myRouteLoaders = routeLoaders;
@@ -227,6 +237,8 @@ MSNet::closeBuilding(MSEdgeControl* edges, MSJunctionControl* junctions,
     if (myLogExecutionTime) {
         mySimBeginMillis = SysUtils::getCurrentMillis();
     }
+    myHasInternalLinks = hasInternalLinks;
+    myHasElevation = checkElevation();
 }
 
 
@@ -250,6 +262,9 @@ MSNet::~MSNet() {
     if (myPersonControl != 0) {
         delete myPersonControl;
     }
+    if (myContainerControl != 0) {
+        delete myContainerControl;
+    }
     delete myShapeContainer;
     delete myEdgeWeights;
     delete myRouterTTDijkstra;
@@ -266,6 +281,22 @@ MSNet::~MSNet() {
 }
 
 
+void
+MSNet::addRestriction(const std::string& id, const SUMOVehicleClass svc, const SUMOReal speed) {
+    myRestrictions[id][svc] = speed;
+}
+
+
+const std::map<SUMOVehicleClass, SUMOReal>*
+MSNet::getRestrictions(const std::string& id) const {
+    std::map<std::string, std::map<SUMOVehicleClass, SUMOReal> >::const_iterator i = myRestrictions.find(id);
+    if (i == myRestrictions.end()) {
+        return 0;
+    }
+    return &i->second;
+}
+
+
 int
 MSNet::simulate(SUMOTime start, SUMOTime stop) {
     // report the begin when wished
@@ -437,7 +468,10 @@ MSNet::simulationStep() {
         myPersonControl->checkWaitingPersons(this, myStep);
     }
     // insert vehicles
-    myInserter->determineCandidates(myStep);
+    myInserter->determineCandidates(myStep);    // containers
+    if (myContainerControl != 0) {
+        myContainerControl->checkWaitingContainers(this, myStep);
+    }
     myInsertionEvents->execute(myStep);
 #ifdef HAVE_FOX
     MSDevice_Routing::waitForAll();
@@ -484,10 +518,14 @@ MSNet::simulationState(SUMOTime stopTime) const {
         if (myInsertionEvents->isEmpty()
                 && (myVehicleControl->getActiveVehicleCount() == 0)
                 && (myInserter->getPendingFlowCount() == 0)
-                && (myPersonControl == 0 || !myPersonControl->hasNonWaiting())) {
+                && (myPersonControl == 0 || !myPersonControl->hasNonWaiting())
+                && (myContainerControl == 0 || !myContainerControl->hasNonWaiting())) {
             if (myPersonControl) {
                 myPersonControl->abortWaiting();
             }
+            if (myContainerControl) {
+                myContainerControl->abortWaiting();
+            }
             myVehicleControl->abortWaiting();
             return SIMSTATE_NO_FURTHER_VEHICLES;
         }
@@ -531,6 +569,7 @@ MSNet::clearAll() {
     MSTrigger::cleanup();
     MSCalibrator::cleanup();
     MSPModel::cleanup();
+    MSCModel_NonInteracting::cleanup();
     PedestrianEdge<MSEdge, MSLane, MSJunction>::cleanup();
 }
 
@@ -539,15 +578,17 @@ void
 MSNet::writeOutput() {
     // update detector values
     myDetectorControl->updateDetectors(myStep);
+    const OptionsCont& oc = OptionsCont::getOptions();
 
     // check state dumps
-    if (OptionsCont::getOptions().isSet("netstate-dump")) {
-        MSXMLRawOut::write(OutputDevice::getDeviceByOption("netstate-dump"), *myEdges, myStep);
+    if (oc.isSet("netstate-dump")) {
+        MSXMLRawOut::write(OutputDevice::getDeviceByOption("netstate-dump"), *myEdges, myStep,
+                           oc.getInt("netstate-dump.precision"));
     }
 
     // check fcd dumps
     if (OptionsCont::getOptions().isSet("fcd-output")) {
-        MSFCDExport::write(OutputDevice::getDeviceByOption("fcd-output"), myStep);
+        MSFCDExport::write(OutputDevice::getDeviceByOption("fcd-output"), myStep, myHasElevation);
     }
 
     // check emission dumps
@@ -555,6 +596,12 @@ MSNet::writeOutput() {
         MSEmissionExport::write(OutputDevice::getDeviceByOption("emission-output"), myStep);
     }
 
+    // battery dumps
+    if (OptionsCont::getOptions().isSet("battery-output")) {
+        MSBatteryExport::write(OutputDevice::getDeviceByOption("battery-output"), myStep,
+                               oc.getInt("battery-output.precision"));
+    }
+
     // check full dumps
     if (OptionsCont::getOptions().isSet("full-output")) {
         MSFullExport::write(OutputDevice::getDeviceByOption("full-output"), myStep);
@@ -613,8 +660,8 @@ MSNet::writeOutput() {
         OutputDevice& od = OutputDevice::getDeviceByOption("link-output");
         od.openTag("timestep");
         od.writeAttr(SUMO_ATTR_ID, STEPS2TIME(myStep));
-        const std::vector<MSEdge*>& edges = myEdges->getEdges();
-        for (std::vector<MSEdge*>::const_iterator i = edges.begin(); i != edges.end(); ++i) {
+        const MSEdgeVector& edges = myEdges->getEdges();
+        for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
             const std::vector<MSLane*>& lanes = (*i)->getLanes();
             for (std::vector<MSLane*>::const_iterator j = lanes.begin(); j != lanes.end(); ++j) {
                 const std::vector<MSLink*>& links = (*j)->getLinkCont();
@@ -642,6 +689,14 @@ MSNet::getPersonControl() {
     return *myPersonControl;
 }
 
+MSContainerControl&
+MSNet::getContainerControl() {
+    if (myContainerControl == 0) {
+        myContainerControl = new MSContainerControl();
+    }
+    return *myContainerControl;
+}
+
 
 MSEdgeWeightsStorage&
 MSNet::getWeightsStorage() {
@@ -711,12 +766,12 @@ MSNet::informVehicleStateListener(const SUMOVehicle* const vehicle, VehicleState
 
 // ------ Insertion and retrieval of bus stops ------
 bool
-MSNet::addBusStop(MSBusStop* busStop) {
+MSNet::addBusStop(MSStoppingPlace* busStop) {
     return myBusStopDict.add(busStop->getID(), busStop);
 }
 
 
-MSBusStop*
+MSStoppingPlace*
 MSNet::getBusStop(const std::string& id) const {
     return myBusStopDict.get(id);
 }
@@ -724,9 +779,32 @@ MSNet::getBusStop(const std::string& id) const {
 
 std::string
 MSNet::getBusStopID(const MSLane* lane, const SUMOReal pos) const {
-    const std::map<std::string, MSBusStop*>& vals = myBusStopDict.getMyMap();
-    for (std::map<std::string, MSBusStop*>::const_iterator it = vals.begin(); it != vals.end(); ++it) {
-        MSBusStop* stop = it->second;
+    const std::map<std::string, MSStoppingPlace*>& vals = myBusStopDict.getMyMap();
+    for (std::map<std::string, MSStoppingPlace*>::const_iterator it = vals.begin(); it != vals.end(); ++it) {
+        MSStoppingPlace* stop = it->second;
+        if (&stop->getLane() == lane && fabs(stop->getEndLanePosition() - pos) < POSITION_EPS) {
+            return stop->getID();
+        }
+    }
+    return "";
+}
+
+// ------ Insertion and retrieval of container stops ------
+bool
+MSNet::addContainerStop(MSStoppingPlace* containerStop) {
+    return myContainerStopDict.add(containerStop->getID(), containerStop);
+}
+
+MSStoppingPlace*
+MSNet::getContainerStop(const std::string& id) const {
+    return myContainerStopDict.get(id);
+}
+
+std::string
+MSNet::getContainerStopID(const MSLane* lane, const SUMOReal pos) const {
+    const std::map<std::string, MSStoppingPlace*>& vals = myContainerStopDict.getMyMap();
+    for (std::map<std::string, MSStoppingPlace*>::const_iterator it = vals.begin(); it != vals.end(); ++it) {
+        MSStoppingPlace* stop = it->second;
         if (&stop->getLane() == lane && fabs(stop->getEndLanePosition() - pos) < POSITION_EPS) {
             return stop->getID();
         }
@@ -735,19 +813,44 @@ MSNet::getBusStopID(const MSLane* lane, const SUMOReal pos) const {
 }
 
 
+bool
+MSNet::addChrgStn(MSChrgStn* chrgStn) {
+    return myChrgStnDict.add(chrgStn->getID(), chrgStn);
+}
+
+
+MSChrgStn*
+MSNet::getChrgStn(const std::string& id) const {
+    return myChrgStnDict.get(id);
+}
+
+
+std::string
+MSNet::getChrgStnID(const MSLane* lane, const SUMOReal pos) const {
+    const std::map<std::string, MSChrgStn*>& vals = myChrgStnDict.getMyMap();
+    for (std::map<std::string, MSChrgStn*>::const_iterator it = vals.begin(); it != vals.end(); ++it) {
+        MSChrgStn* chrgStn = it->second;
+        if (&chrgStn->getLane() == lane && chrgStn->getBeginLanePosition() <= pos && chrgStn->getEndLanePosition() >= pos) {
+            return chrgStn->getID();
+        }
+    }
+    return "";
+}
+
+
 SUMOAbstractRouter<MSEdge, SUMOVehicle>&
-MSNet::getRouterTT(const std::vector<MSEdge*>& prohibited) const {
+MSNet::getRouterTT(const MSEdgeVector& prohibited) const {
     if (!myRouterTTInitialized) {
         myRouterTTInitialized = true;
         const std::string routingAlgorithm = OptionsCont::getOptions().getString("routing-algorithm");
         if (routingAlgorithm == "dijkstra") {
-            myRouterTTDijkstra = new DijkstraRouterTT<MSEdge, SUMOVehicle, prohibited_withRestrictions<MSEdge, SUMOVehicle> >(
+            myRouterTTDijkstra = new DijkstraRouterTT<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >(
                 MSEdge::numericalDictSize(), true, &MSNet::getTravelTime);
         } else {
             if (routingAlgorithm != "astar") {
                 WRITE_WARNING("TraCI and Triggers cannot use routing algorithm '" + routingAlgorithm + "'. using 'astar' instead.");
             }
-            myRouterTTAStar = new AStarRouter<MSEdge, SUMOVehicle, prohibited_withRestrictions<MSEdge, SUMOVehicle> >(
+            myRouterTTAStar = new AStarRouter<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >(
                 MSEdge::numericalDictSize(), true, &MSNet::getTravelTime);
         }
     }
@@ -763,9 +866,9 @@ MSNet::getRouterTT(const std::vector<MSEdge*>& prohibited) const {
 
 
 SUMOAbstractRouter<MSEdge, SUMOVehicle>&
-MSNet::getRouterEffort(const std::vector<MSEdge*>& prohibited) const {
+MSNet::getRouterEffort(const MSEdgeVector& prohibited) const {
     if (myRouterEffort == 0) {
-        myRouterEffort = new DijkstraRouterEffort<MSEdge, SUMOVehicle, prohibited_withRestrictions<MSEdge, SUMOVehicle> >(
+        myRouterEffort = new DijkstraRouterEffort<MSEdge, SUMOVehicle, prohibited_withPermissions<MSEdge, SUMOVehicle> >(
             MSEdge::numericalDictSize(), true, &MSNet::getEffort, &MSNet::getTravelTime);
     }
     myRouterEffort->prohibit(prohibited);
@@ -774,7 +877,7 @@ MSNet::getRouterEffort(const std::vector<MSEdge*>& prohibited) const {
 
 
 MSNet::MSPedestrianRouterDijkstra&
-MSNet::getPedestrianRouter(const std::vector<MSEdge*>& prohibited) const {
+MSNet::getPedestrianRouter(const MSEdgeVector& prohibited) const {
     if (myPedestrianRouter == 0) {
         myPedestrianRouter = new MSPedestrianRouterDijkstra();
     }
@@ -793,4 +896,17 @@ MSNet::getLanesRTree() const {
 }
 
 
+bool
+MSNet::checkElevation() {
+    const MSEdgeVector& edges = myEdges->getEdges();
+    for (MSEdgeVector::const_iterator e = edges.begin(); e != edges.end(); ++e) {
+        for (std::vector<MSLane*>::const_iterator i = (*e)->getLanes().begin(); i != (*e)->getLanes().end(); ++i) {
+            if ((*i)->getShape().hasElevation()) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
 /****************************************************************************/
