@@ -4,7 +4,7 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Fri, 08.10.2013
-/// @version $Id: MSLCM_JE2013.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSLCM_JE2013.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // A lane change model developed by J. Erdmann
 // based on the model of D. Krajzewicz developed between 2004 and 2011 (MSLCM_DK2004)
@@ -43,13 +43,6 @@
 #endif // CHECK_MEMORY_LEAKS
 
 //#define DEBUG_VEHICLE_GUI_SELECTION 1
-#ifdef DEBUG_VEHICLE_GUI_SELECTION
-#include <utils/gui/div/GUIGlobalSelection.h>
-#include <guisim/GUIVehicle.h>
-#include <guisim/GUILane.h>
-#endif
-
-
 
 // ===========================================================================
 // variable definitions
@@ -96,7 +89,7 @@
 #define TURN_LANE_DIST (SUMOReal)200.0 // the distance at which a lane leading elsewhere is considered to be a turn-lane that must be avoided
 
 //#define DEBUG_COND (myVehicle.getID() == "1501_27271428" || myVehicle.getID() == "1502_27270000")
-//#define DEBUG_COND (myVehicle.getID() == "175129_26220000")
+//#define DEBUG_COND (myVehicle.getID() == "overtaking")
 //#define DEBUG_COND (myVehicle.getID() == "pkw150478" || myVehicle.getID() == "pkw150494" || myVehicle.getID() == "pkw150289")
 //#define DEBUG_COND (myVehicle.getID() == "A" || myVehicle.getID() == "B") // fail change to left
 //#define DEBUG_COND (myVehicle.getID() == "Costa_12_13") // test stops_overtaking
@@ -763,6 +756,14 @@ MSLCM_JE2013::_wantsChange(
     //              : laSpeed *  LOOK_FORWARD_NEAR;
     SUMOReal laDist = myLookAheadSpeed * (right ? LOOK_FORWARD_RIGHT : LOOK_FORWARD_LEFT);
     laDist += myVehicle.getVehicleType().getLengthWithGap() * (SUMOReal) 2.;
+
+    // react to a stopped leader on the current lane
+    if (bestLaneOffset == 0 && leader.first != 0 && leader.first->isStopped()) {
+        // value is doubled for the check since we change back and forth
+        laDist = 0.5 * (myVehicle.getVehicleType().getLengthWithGap()
+                        + leader.first->getVehicleType().getLengthWithGap());
+    }
+
     // free space that is available for changing
     //const SUMOReal neighSpeed = (neighLead.first != 0 ? neighLead.first->getSpeed() :
     //        neighFollow.first != 0 ? neighFollow.first->getSpeed() :
