@@ -5,12 +5,12 @@
 /// @author  Mario Krumnow
 /// @author  Michael Behrisch
 /// @date    2012-04-26
-/// @version $Id: MSFCDExport.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSFCDExport.cpp 18287 2015-04-23 13:20:21Z namdre $
 ///
 // Realises dumping Floating Car Data (FCD) Data
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2012-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2012-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -41,14 +41,10 @@
 #include "MSFCDExport.h"
 #include <microsim/MSNet.h>
 #include <microsim/MSVehicle.h>
-#include <microsim/MSPerson.h>
+#include <microsim/pedestrians/MSPerson.h>
 #include <microsim/MSPersonControl.h>
-
-#ifdef HAVE_MESOSIM
-#include <mesosim/MELoop.h>
-#include <mesosim/MESegment.h>
-#endif
-
+#include <microsim/MSContainer.h>
+#include <microsim/MSContainerControl.h>
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
 #endif // CHECK_MEMORY_LEAKS
@@ -58,19 +54,16 @@
 // method definitions
 // ===========================================================================
 void
-MSFCDExport::write(OutputDevice& of, SUMOTime timestep) {
+MSFCDExport::write(OutputDevice& of, SUMOTime timestep, bool elevation) {
     const bool useGeo = OptionsCont::getOptions().getBool("fcd-output.geo");
     const bool signals = OptionsCont::getOptions().getBool("fcd-output.signals");
-    MSVehicleControl& vc = MSNet::getInstance()->getVehicleControl();
-    MSVehicleControl::constVehIt it = vc.loadedVehBegin();
-    MSVehicleControl::constVehIt end = vc.loadedVehEnd();
-
     of.openTag("timestep").writeAttr(SUMO_ATTR_TIME, time2string(timestep));
-    for (; it != end; ++it) {
-        const MSVehicle* veh = static_cast<const MSVehicle*>((*it).second);
+    MSVehicleControl& vc = MSNet::getInstance()->getVehicleControl();
+    for (MSVehicleControl::constVehIt it = vc.loadedVehBegin(); it != vc.loadedVehEnd(); ++it) {
+        const SUMOVehicle* veh = it->second;
+        const MSVehicle* microVeh = dynamic_cast<const MSVehicle*>(veh);
         if (veh->isOnRoad()) {
             Position pos = veh->getPosition();
-            MSLane* lane = veh->getLane();
             if (useGeo) {
                 of.setPrecision(GEO_OUTPUT_ACCURACY);
                 GeoConvHelper::getFinal().cartesian2geo(pos);
@@ -79,14 +72,19 @@ MSFCDExport::write(OutputDevice& of, SUMOTime timestep) {
             of.writeAttr(SUMO_ATTR_ID, veh->getID());
             of.writeAttr(SUMO_ATTR_X, pos.x());
             of.writeAttr(SUMO_ATTR_Y, pos.y());
+            if (elevation) {
+                of.writeAttr(SUMO_ATTR_Z, pos.z());
+            }
             of.writeAttr(SUMO_ATTR_ANGLE, veh->getAngle());
             of.writeAttr(SUMO_ATTR_TYPE, veh->getVehicleType().getID());
             of.writeAttr(SUMO_ATTR_SPEED, veh->getSpeed());
             of.writeAttr(SUMO_ATTR_POSITION, veh->getPositionOnLane());
-            of.writeAttr(SUMO_ATTR_LANE, lane->getID());
-            of.writeAttr(SUMO_ATTR_SLOPE, lane->getShape().slopeDegreeAtOffset(veh->getPositionOnLane()));
-            if (signals) {
-                of.writeAttr("signals", toString(veh->getSignals()));
+            if (microVeh != 0) {
+                of.writeAttr(SUMO_ATTR_LANE, microVeh->getLane()->getID());
+            }
+            of.writeAttr(SUMO_ATTR_SLOPE, veh->getSlope());
+            if (microVeh != 0 && signals) {
+                of.writeAttr("signals", toString(microVeh->getSignals()));
             }
             of.closeTag();
         }
@@ -94,11 +92,11 @@ MSFCDExport::write(OutputDevice& of, SUMOTime timestep) {
     if (MSNet::getInstance()->getPersonControl().hasPersons()) {
         // write persons
         MSEdgeControl& ec = MSNet::getInstance()->getEdgeControl();
-        const std::vector<MSEdge*>& edges = ec.getEdges();
-        for (std::vector<MSEdge*>::const_iterator e = edges.begin(); e != edges.end(); ++e) {
-            const std::vector<MSPerson*>& persons = (*e)->getSortedPersons(timestep);
-            for (std::vector<MSPerson*>::const_iterator it_p = persons.begin(); it_p != persons.end(); ++it_p) {
-                MSPerson* p = *it_p;
+        const MSEdgeVector& edges = ec.getEdges();
+        for (MSEdgeVector::const_iterator e = edges.begin(); e != edges.end(); ++e) {
+            const std::vector<MSTransportable*>& persons = (*e)->getSortedPersons(timestep);
+            for (std::vector<MSTransportable*>::const_iterator it_p = persons.begin(); it_p != persons.end(); ++it_p) {
+                MSTransportable* p = *it_p;
                 Position pos = p->getPosition();
                 if (useGeo) {
                     of.setPrecision(GEO_OUTPUT_ACCURACY);
@@ -108,6 +106,9 @@ MSFCDExport::write(OutputDevice& of, SUMOTime timestep) {
                 of.writeAttr(SUMO_ATTR_ID, p->getID());
                 of.writeAttr(SUMO_ATTR_X, pos.x());
                 of.writeAttr(SUMO_ATTR_Y, pos.y());
+                if (elevation) {
+                    of.writeAttr(SUMO_ATTR_Z, pos.z());
+                }
                 of.writeAttr(SUMO_ATTR_ANGLE, p->getAngle());
                 of.writeAttr(SUMO_ATTR_SPEED, p->getSpeed());
                 of.writeAttr(SUMO_ATTR_POSITION, p->getEdgePos());
@@ -117,6 +118,35 @@ MSFCDExport::write(OutputDevice& of, SUMOTime timestep) {
             }
         }
     }
+    if (MSNet::getInstance()->getContainerControl().hasContainers()) {
+        // write containers
+        MSEdgeControl& ec = MSNet::getInstance()->getEdgeControl();
+        const std::vector<MSEdge*>& edges = ec.getEdges();
+        for (std::vector<MSEdge*>::const_iterator e = edges.begin(); e != edges.end(); ++e) {
+            const std::vector<MSTransportable*>& containers = (*e)->getSortedContainers(timestep);
+            for (std::vector<MSTransportable*>::const_iterator it_c = containers.begin(); it_c != containers.end(); ++it_c) {
+                MSTransportable* c = *it_c;
+                Position pos = c->getPosition();
+                if (useGeo) {
+                    of.setPrecision(GEO_OUTPUT_ACCURACY);
+                    GeoConvHelper::getFinal().cartesian2geo(pos);
+                }
+                of.openTag(SUMO_TAG_CONTAINER);
+                of.writeAttr(SUMO_ATTR_ID, c->getID());
+                of.writeAttr(SUMO_ATTR_X, pos.x());
+                of.writeAttr(SUMO_ATTR_Y, pos.y());
+                if (elevation) {
+                    of.writeAttr(SUMO_ATTR_Z, pos.z());
+                }
+                of.writeAttr(SUMO_ATTR_ANGLE, c->getAngle());
+                of.writeAttr(SUMO_ATTR_SPEED, c->getSpeed());
+                of.writeAttr(SUMO_ATTR_POSITION, c->getEdgePos());
+                of.writeAttr(SUMO_ATTR_EDGE, (*e)->getID());
+                of.writeAttr(SUMO_ATTR_SLOPE, (*e)->getLanes()[0]->getShape().slopeDegreeAtOffset(c->getEdgePos()));
+                of.closeTag();
+            }
+        }
+    }
     of.closeTag();
 }
 
