@@ -4,12 +4,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: MSActuatedTrafficLightLogic.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSActuatedTrafficLightLogic.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // An actuated (adaptive) traffic light logic
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -93,11 +93,10 @@ public:
     /// @{
 
     /** @brief Switches to the next phase
-     * @param[in] isActive Whether this program is the currently used one
      * @return The time of the next switch
      * @see MSTrafficLightLogic::trySwitch
      */
-    SUMOTime trySwitch(bool isActive);
+    SUMOTime trySwitch();
     /// @}
 
 
@@ -131,6 +130,9 @@ protected:
     /// The detector distance in seconds
     SUMOReal myDetectorGap;
 
+    /// Whether the detectors shall be shown in the GUI
+    bool myShowDetectors;
+
 };
 
 
