@@ -6,7 +6,7 @@
 /// @author  Michael Behrisch
 /// @author  Friedemann Wesner
 /// @date    Sept 2002
-/// @version $Id: MSSimpleTrafficLightLogic.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: MSSimpleTrafficLightLogic.cpp 18213 2015-04-16 11:55:42Z behrisch $
 ///
 // A fixed traffic light logic
 /****************************************************************************/
@@ -160,7 +160,7 @@ MSSimpleTrafficLightLogic::getOffsetFromIndex(unsigned int index) const {
     if (index == 0) {
         return 0;
     }
-    unsigned int pos = 0;
+    SUMOTime pos = 0;
     for (unsigned int i = 0; i < index; i++) {
         pos += getPhase(i).duration;
     }
