@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, May 2005
-/// @version $Id: MSCalibrator.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: MSCalibrator.cpp 18213 2015-04-16 11:55:42Z behrisch $
 ///
 // Calibrates the flow on an edge by removing an inserting vehicles
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2005-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2005-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -140,7 +140,7 @@ MSCalibrator::myStartElement(int element,
                              const SUMOSAXAttributes& attrs) {
     if (element == SUMO_TAG_FLOW) {
         AspiredState state;
-        int lastEnd = -1;
+        SUMOTime lastEnd = -1;
         if (myIntervals.size() > 0) {
             lastEnd = myIntervals.back().end;
             if (lastEnd == -1) {
