@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBContHelper.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBContHelper.cpp 18566 2015-07-14 12:46:29Z behrisch $
 ///
 // Some methods for traversing lists of edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -258,8 +258,36 @@ int
 NBContHelper::edge_by_angle_to_nodeShapeCentroid_sorter::operator()(const NBEdge* e1, const NBEdge* e2) const {
     assert(e1->getFromNode() == myNode || e1->getToNode() == myNode);
     assert(e2->getFromNode() == myNode || e2->getToNode() == myNode);
-    const SUMOReal angle1 = e1->getFromNode() == myNode ? e1->getStartAngle() : e1->getEndAngle();
-    const SUMOReal angle2 = e2->getFromNode() == myNode ? e2->getStartAngle() : e2->getEndAngle();
+    const SUMOReal angle1 = e1->getAngleAtNodeToCenter(myNode);
+    const SUMOReal angle2 = e2->getAngleAtNodeToCenter(myNode);
+    const SUMOReal absDiff = abs(angle1 - angle2);
+
+    // cannot trust the angle difference hence a heuristic:
+    if (absDiff < 2 || absDiff > (360 - 2)) {
+        const bool sameDir = ((e1->getFromNode() == myNode && e2->getFromNode() == myNode)
+                              || (e1->getToNode() == myNode && e2->getToNode() == myNode));
+        if (sameDir) {
+            // put edges that allow pedestrians on the 'outside', but be aware if both allow / disallow
+            if (e1->getToNode() == myNode) {
+                if ((e1->getPermissions() & SVC_PEDESTRIAN) != 0) {
+                    if ((e2->getPermissions() & SVC_PEDESTRIAN) == 0) {
+                        return true;
+                    }
+                }
+            } else {
+                if ((e1->getPermissions() & SVC_PEDESTRIAN) == 0) {
+                    if ((e2->getPermissions() & SVC_PEDESTRIAN) != 0) {
+                        return true;
+                    }
+                }
+            }
+            // break ties to ensure strictly weak ordering
+            return e1->getID() < e2->getID();
+        } else {
+            // sort incoming before outgoing, no need to break ties here
+            return e1->getToNode() == myNode;
+        }
+    }
     return angle1 < angle2;
 }
 
