@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdge.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NBEdge.h 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // The representation of a single edge during network building
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -153,7 +153,7 @@ public:
          */
         Connection(int fromLane_, NBEdge* toEdge_, int toLane_)
             : fromLane(fromLane_), toEdge(toEdge_), toLane(toLane_),
-              mayDefinitelyPass(false), haveVia(false) { }
+              mayDefinitelyPass(false), keepClear(true), haveVia(false) { }
 
         ~Connection() { }
 
@@ -170,6 +170,8 @@ public:
         unsigned int tlLinkNo;
         /// @brief Information about being definitely free to drive (on-ramps)
         bool mayDefinitelyPass;
+        /// @brief whether the junction must be kept clear when using this connection
+        bool keepClear;
 
 
         std::string origID;
@@ -195,17 +197,20 @@ public:
 
     /// Computes the offset from the edge shape on the current segment
     static std::pair<SUMOReal, SUMOReal> laneOffset(const Position& from,
-            const Position& to, SUMOReal laneCenterOffset, bool leftHand);
+            const Position& to, SUMOReal laneCenterOffset);
 
     /// @brief unspecified lane width
     static const SUMOReal UNSPECIFIED_WIDTH;
     /// @brief unspecified lane offset
     static const SUMOReal UNSPECIFIED_OFFSET;
-    /// @brief unspecified signal offset
-    static const SUMOReal UNSPECIFIED_SIGNAL_OFFSET;
+    /// @brief unspecified lane speed
+    static const SUMOReal UNSPECIFIED_SPEED;
+
     /// @brief no length override given
     static const SUMOReal UNSPECIFIED_LOADED_LENGTH;
-    /// @brief the distance at which to take the default anglen
+    /// @brief unspecified signal offset
+    static const SUMOReal UNSPECIFIED_SIGNAL_OFFSET;
+    /// @brief the distance at which to take the default angle
     static const SUMOReal ANGLE_LOOKAHEAD;
 
 public:
@@ -324,17 +329,11 @@ public:
      * @param[in] yoff The y-offset to apply
      */
     void reshiftPosition(SUMOReal xoff, SUMOReal yoff);
+
+    /// @brief mirror coordinates along the x-axis
+    void mirrorX();
     /// @}
 
-
-
-    /** @brief Marks this edge to be left-handed
-     */
-    void setLeftHanded() {
-        myAmLeftHand = true;
-    }
-
-
     /// @name Atomar getter methods
     //@{
 
@@ -453,6 +452,10 @@ public:
     SUMOReal getLaneWidth(int lane) const;
 
 
+    /** @brief Returns the combined width of all lanes of this edge
+     */
+    SUMOReal getTotalWidth() const;
+
     /** @brief Returns the street name of this edge
      */
     const std::string& getStreetName() const {
@@ -489,14 +492,6 @@ public:
         mySignalOffset = offset;
     }
 
-    /** @brief Returns the type name
-     * @return The name of this edge's type
-     */
-    const std::string& getTypeName() const {
-        return myType;
-    }
-
-
     /** @brief Returns the lane definitions
      * @return The stored lane definitions
      */
@@ -505,8 +500,11 @@ public:
     }
     //@}
 
-    /// @brief return the first lane with permissions other than SVC_PEDESTRIAN
-    int getFirstNonPedestrianLaneIndex(int direction) const;
+    /** @brief return the first lane with permissions other than SVC_PEDESTRIAN and 0
+     * @param[in] direction The direction in which the lanes shall be checked
+     * @param[in] exclusive Whether lanes that allow pedestrians along with other classes shall be counted as non-pedestrian
+     */
+    int getFirstNonPedestrianLaneIndex(int direction, bool exclusive = false) const;
     NBEdge::Lane getFirstNonPedestrianLane(int direction) const;
 
     /// @brief return the angle for computing pedestrian crossings at the given node
@@ -564,6 +562,10 @@ public:
      */
     void addGeometryPoint(int index, const Position& p);
 
+    /** @brief shift geometry at the given node
+     * to avoid overlap
+     */
+    void shiftPositionAtNode(NBNode* node, NBEdge* opposite);
 
     /** @brief Recomputeds the lane shapes to terminate at the node shape
      * For every lane the intersection with the fromNode and toNode is
@@ -666,7 +668,8 @@ public:
     bool addLane2LaneConnection(unsigned int fromLane, NBEdge* dest,
                                 unsigned int toLane, Lane2LaneInfoType type,
                                 bool mayUseSameDestination = false,
-                                bool mayDefinitelyPass = false);
+                                bool mayDefinitelyPass = false,
+                                bool keepClear = true);
 
 
     /** @brief Builds no connections starting at the given lanes
@@ -706,7 +709,8 @@ public:
                        unsigned int destLane,
                        Lane2LaneInfoType type,
                        bool mayUseSameDestination = false,
-                       bool mayDefinitelyPass = false);
+                       bool mayDefinitelyPass = false,
+                       bool keepClear = true);
 
 
 
@@ -814,12 +818,18 @@ public:
 
 
     /** @brief Returns whether the given edge is the opposite direction to this edge
-     * @param[in] n The node at which this may be turnaround direction
      * @param[in] edge The edge which may be the turnaround direction
      * @return Whether the given edge is this edge's turnaround direction
+     * (regardless of whether a connection exists)
      */
-    bool isTurningDirectionAt(const NBNode* n, const NBEdge* const edge) const;
-    void setTurningDestination(NBEdge* e);
+    bool isTurningDirectionAt(const NBEdge* const edge) const;
+
+
+    /** @brief Sets the turing destination at the given edge
+     * @param[in] e The turn destination
+     * @param[in] onlyPossible If true, only sets myPossibleTurnDestination
+     */
+    void setTurningDestination(NBEdge* e, bool onlyPossible = false);
 
 
 
@@ -890,7 +900,7 @@ public:
     bool needsLaneSpecificOutput() const;
 
     /// @brief whether at least one lane has restrictions
-    bool hasRestrictions() const;
+    bool hasPermissions() const;
 
     /// @brief whether lanes differ in allowed vehicle classes
     bool hasLaneSpecificPermissions() const;
@@ -908,11 +918,11 @@ public:
     bool computeEdge2Edges(bool noLeftMovers);
 
     /// computes the edge, step2: computation of which lanes approach the edges)
-    bool computeLanes2Edges(const bool buildCrossingsAndWalkingAreas);
+    bool computeLanes2Edges();
 
     /** recheck whether all lanes within the edge are all right and
         optimises the connections once again */
-    bool recheckLanes(const bool buildCrossingsAndWalkingAreas);
+    bool recheckLanes();
 
     /** @brief Add a connection to the previously computed turnaround, if wished
      *
@@ -947,10 +957,11 @@ public:
     void addCrossingPointsAsIncomingWithGivenOutgoing(NBEdge* o,
             PositionVector& into);
 
-    SUMOReal width() const;
+    /// @brief get the outer boundary of this edge when going clock-wise around the given node
+    PositionVector getCWBoundaryLine(const NBNode& n) const;
 
-    PositionVector getCWBoundaryLine(const NBNode& n, SUMOReal offset) const;
-    PositionVector getCCWBoundaryLine(const NBNode& n, SUMOReal offset) const;
+    /// @brief get the outer boundary of this edge when going counter-clock-wise around the given node
+    PositionVector getCCWBoundaryLine(const NBNode& n) const;
 
     bool expandableBy(NBEdge* possContinuation) const;
     void append(NBEdge* continuation);
@@ -960,7 +971,11 @@ public:
 
     void moveOutgoingConnectionsFrom(NBEdge* e, unsigned int laneOff);
 
-    NBEdge* getTurnDestination() const;
+    /* @brief return the turn destination if it exists
+     * @param[in] possibleDestination Wether myPossibleTurnDestination should be returned if no turnaround connection
+     * exists
+     */
+    NBEdge* getTurnDestination(bool possibleDestination = false) const;
 
     std::string getLaneID(unsigned int lane) const;
 
@@ -981,6 +996,17 @@ public:
     SUMOReal getAngleAtNode(const NBNode* const node) const;
 
 
+    /** @brief Returns the angle of from the node shape center to where the edge meets
+     * the node shape
+     *
+     * The angle is signed, disregards direction, and starts at 12 o'clock
+     *  (north->south), proceeds positive clockwise.
+     * @param[in] node The node for which the edge's angle shall be returned
+     * @return This edge's angle at the given node shape
+     */
+    SUMOReal getAngleAtNodeToCenter(const NBNode* const node) const;
+
+
     void incLaneNo(unsigned int by);
 
     void decLaneNo(unsigned int by, int dir = 0);
@@ -990,6 +1016,9 @@ public:
     /// add a pedestrian sidewalk of the given width and shift existing connctions
     void addSidewalk(SUMOReal width);
 
+    /// add a bicycle lane of the given width and shift existing connctions
+    void addBikeLane(SUMOReal width);
+
     /// @brief set allowed/disallowed classes for the given lane or for all lanes if -1 is given
     void setPermissions(SVCPermissions permissions, int lane = -1);
 
@@ -1050,6 +1079,8 @@ public:
         mySigns.push_back(sign);
     }
 
+    /// @brief cut shape at the intersection shapes
+    PositionVector cutAtIntersection(const PositionVector& old) const;
 
 private:
     /**
@@ -1132,7 +1163,7 @@ private:
     };
 
     /// Computes the shape for the given lane
-    PositionVector computeLaneShape(unsigned int lane, SUMOReal offset);
+    PositionVector computeLaneShape(unsigned int lane, SUMOReal offset) const;
 
     /// Computes the offset from the edge shape on the current segment
     //std::pair<SUMOReal, SUMOReal> laneOffset(const Position& from, const Position& to, SUMOReal laneCenterOffset);
@@ -1161,7 +1192,9 @@ private:
 
 
     /** divides the lanes on the outgoing edges */
-    void divideOnEdges(const EdgeVector* outgoing, const bool buildCrossingsAndWalkingAreas);
+    void divideOnEdges(const EdgeVector* outgoing);
+    void divideSelectedLanesOnEdges(const EdgeVector* outgoing, const std::vector<int>& availableLanes,
+                                    const std::vector<unsigned int>* priorities);
 
     /** recomputes the edge priorities and manipulates them for a distribution
         of lanes on edges which is more like in real-life */
@@ -1169,7 +1202,7 @@ private:
         const EdgeVector* outgoing);
 
     /** computes the sum of the given list's entries (sic!) */
-    unsigned int computePrioritySum(std::vector<unsigned int>* priorities);
+    static unsigned int computePrioritySum(const std::vector<unsigned int>& priorities);
 
 
     /// @name Setting and getting connections
@@ -1177,14 +1210,14 @@ private:
 
     /** moves a connection one place to the left;
         Attention! no checking for field validity */
-    void moveConnectionToLeft(unsigned int lane, const bool buildCrossingsAndWalkingAreas);
+    void moveConnectionToLeft(unsigned int lane);
 
     /** moves a connection one place to the right;
         Attention! no checking for field validity */
-    void moveConnectionToRight(unsigned int lane, const bool buildCrossingsAndWalkingAreas);
+    void moveConnectionToRight(unsigned int lane);
 
     /// @brief whether the connection can originate on newFromLane
-    bool canMoveConnection(const Connection& con, unsigned int newFromLane, const bool buildCrossingsAndWalkingAreas) const;
+    bool canMoveConnection(const Connection& con, unsigned int newFromLane) const;
     /// @}
 
 
@@ -1197,6 +1230,15 @@ private:
     /// @brief computes the angle of this edge and stores it in myAngle
     void computeAngle();
 
+
+    /* @brief compute the first intersection point between the given lane
+     * geometries considering their rspective widths */
+    static SUMOReal firstIntersection(const PositionVector& v1, const PositionVector& v2, SUMOReal width2);
+
+
+    /// add a lane of the given width, restricted to the given class and shift existing connctions
+    void addRestrictedLane(SUMOReal width, SUMOVehicleClass vclass);
+
 private:
     /** @brief The building step
      * @see EdgeBuildingStep
@@ -1232,8 +1274,10 @@ private:
      */
     std::vector<Connection> myConnectionsToDelete;
 
-    /// @brief The turn destination edge
+    /// @brief The turn destination edge (if a connection exists)
     NBEdge* myTurnDestination;
+    /// @brief The edge that would be the turn destination if there was one
+    NBEdge* myPossibleTurnDestination;
 
     /// @brief The priority normalised for the node the edge is outgoing of
     int myFromJunctionPriority;
@@ -1261,10 +1305,6 @@ private:
     /// @brief An optional length to use (-1 if not valid)
     SUMOReal myLoadedLength;
 
-    /// @brief Whether this edge is a left-hand edge
-    bool myAmLeftHand;
-
-
     /// @brief Information whether this is a junction-inner edge
     bool myAmInnerEdge;
 
@@ -1329,13 +1369,16 @@ public:
     class connections_toedge_finder {
     public:
         /// constructor
-        connections_toedge_finder(NBEdge* const edge2find) : myEdge2Find(edge2find) { }
+        connections_toedge_finder(NBEdge* const edge2find, bool hasFromLane = false) :
+            myHasFromLane(hasFromLane),
+            myEdge2Find(edge2find) { }
 
         bool operator()(const Connection& c) const {
-            return c.toEdge == myEdge2Find;
+            return c.toEdge == myEdge2Find && (!myHasFromLane || c.fromLane != -1);
         }
 
     private:
+        const bool myHasFromLane;
         NBEdge* const myEdge2Find;
 
     private:
