@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBEdgeCont.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: NBEdgeCont.h 18647 2015-08-18 07:09:50Z namdre $
 ///
 // Storage for edges, including some functionality operating on multiple edges
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -237,13 +237,17 @@ public:
      * @param[in] secondEdgeName The id the second part of the split edge shall have
      * @param[in] noLanesFirstEdge The number of lanes the second part of the split edge shall have
      * @param[in] noLanesSecondEdge The number of lanes the second part of the split edge shall have
+     * @param[in] speed The speed for the edge after the split
+     * @param[in] changedLeft The number of lanes that is added or removed on the left side of the edge
+     *            (By default all added/removed lanes are assumed to be on the right when computing connections)
      * @return Whether the edge could be split
      * @exception ProcessError If connections between the edges can not be built
      * @see NBEdge::splitAt(NBDistrictCont &, NBEdge *, SUMOReal, NBNode *, const std::string &, const std::string &, unsigned int , unsigned int)
      */
     bool splitAt(NBDistrictCont& dc, NBEdge* edge, NBNode* node,
                  const std::string& firstEdgeName, const std::string& secondEdgeName,
-                 unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge, const SUMOReal speed = -1.);
+                 unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge,
+                 const SUMOReal speed = -1., const int changedLeft = 0);
 
 
     /** @brief Splits the edge at the position nearest to the given node using the given modifications
@@ -255,12 +259,16 @@ public:
      * @param[in] secondEdgeName The id the second part of the split edge shall have
      * @param[in] noLanesFirstEdge The number of lanes the second part of the split edge shall have
      * @param[in] noLanesSecondEdge The number of lanes the second part of the split edge shall have
+     * @param[in] speed The speed for the edge after the split
+     * @param[in] changedLeft The number of lanes that is added or removed on the left side of the edge
+     *            (By default all added/removed lanes are assumed to be on the right when computing connections)
      * @return Whether the edge could be split
      * @exception ProcessError If connections between the edges can not be built
      */
     bool splitAt(NBDistrictCont& dc, NBEdge* edge, SUMOReal edgepos, NBNode* node,
                  const std::string& firstEdgeName, const std::string& secondEdgeName,
-                 unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge, const SUMOReal speed = -1.);
+                 unsigned int noLanesFirstEdge, unsigned int noLanesSecondEdge,
+                 const SUMOReal speed = -1., const int changedLeft = 0);
     /// @}
 
 
@@ -363,7 +371,7 @@ public:
      * @todo Recheck whether a visitor-pattern should be used herefor
      * @see NBEdge::computeLanes2Edges
      */
-    void computeLanes2Edges(const bool buildCrossingsAndWalkingAreas);
+    void computeLanes2Edges();
 
 
     /** @brief Rechecks whether all lanes have a successor for each of the stored edges
@@ -373,7 +381,7 @@ public:
      * @todo Recheck whether a visitor-pattern should be used herefor
      * @see NBEdge::recheckLanes
      */
-    void recheckLanes(const bool buildCrossingsAndWalkingAreas);
+    void recheckLanes();
 
 
     /** @brief Appends turnarounds to all edges stored in the container
@@ -449,14 +457,6 @@ public:
     void guessRoundabouts();
 
 
-    /** @brief Returns whether the built edges are left-handed
-     * @return Whether this edge container is left-handed
-     */
-    bool isLeftHanded() const {
-        return myAmLeftHanded;
-    }
-
-
     /** @brief Returns whether the edge with the id was ignored during parsing
      * @return Whether the edge with the id was ignored during parsing
      */
@@ -486,8 +486,9 @@ public:
      * @param[in] to The id of the edge the connection ends at
      * @param[in] toLane The number of the lane the connection ends at
      * @param[in] mayDefinitelyPass Whether the connection may be passed without braking
+     * @param[in] keepClear Whether the connection must check to keep the junction clear
      */
-    void addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass);
+    void addPostProcessConnection(const std::string& from, int fromLane, const std::string& to, int toLane, bool mayDefinitelyPass, bool keepClear);
 
 
     /** @brief Try to set any stored connections
@@ -498,8 +499,8 @@ public:
     /// @brief assigns street signs to edges based on toNode types
     void generateStreetSigns();
 
-    /// @brief add sidwalks to edges within the given limits and return the number of edges affected
-    int guessSidewalks(SUMOReal width, SUMOReal minSpeed, SUMOReal maxSpeed);
+    /// @brief add sidwalks to edges within the given limits or permissions and return the number of edges affected
+    int guessSidewalks(SUMOReal width, SUMOReal minSpeed, SUMOReal maxSpeed, bool fromPermissions);
 
 
     /** @brief Returns the determined roundabouts
@@ -542,8 +543,8 @@ private:
          * @param[in] toLane The number of the lane the connection ends at
          * @param[in] mayDefinitelyPass Whether the connection may be passed without braking
          */
-        PostProcessConnection(const std::string& from_, int fromLane_, const std::string& to_, int toLane_, bool mayDefinitelyPass_)
-            : from(from_), fromLane(fromLane_), to(to_), toLane(toLane_), mayDefinitelyPass(mayDefinitelyPass_)
+        PostProcessConnection(const std::string& from_, int fromLane_, const std::string& to_, int toLane_, bool mayDefinitelyPass_, bool keepClear_)
+            : from(from_), fromLane(fromLane_), to(to_), toLane(toLane_), mayDefinitelyPass(mayDefinitelyPass_), keepClear(keepClear_)
         { }
         /// @brief The id of the edge the connection starts at
         std::string from;
@@ -555,6 +556,8 @@ private:
         int toLane;
         /// @brief Whether the connection may be passed without braking
         bool mayDefinitelyPass;
+        /// @brief Whether the connection may be passed without braking
+        bool keepClear;
     };
 
     /// @brief The list of connections to recheck
@@ -576,10 +579,6 @@ private:
     /// @brief the number of splits of edges during the building
     unsigned int myEdgesSplit;
 
-    /// @brief Whether the network is left-handed
-    bool myAmLeftHanded;
-
-
     /// @name Settings for accepting/dismissing edges
     /// @{
 
