@@ -5,7 +5,7 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NBRequest.cpp 18096 2015-03-17 09:50:59Z behrisch $
+/// @version $Id: NBRequest.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // This class computes the logic of a junction
 /****************************************************************************/
@@ -153,12 +153,12 @@ NBRequest::~NBRequest() {}
 
 
 void
-NBRequest::buildBitfieldLogic(bool leftHanded) {
+NBRequest::buildBitfieldLogic() {
     EdgeVector::const_iterator i, j;
     for (i = myIncoming.begin(); i != myIncoming.end(); i++) {
         for (j = myOutgoing.begin(); j != myOutgoing.end(); j++) {
-            computeRightOutgoingLinkCrossings(leftHanded, *i, *j);
-            computeLeftOutgoingLinkCrossings(leftHanded, *i, *j);
+            computeRightOutgoingLinkCrossings(*i, *j);
+            computeLeftOutgoingLinkCrossings(*i, *j);
         }
     }
     // reset signalised/non-signalised dependencies
@@ -169,7 +169,7 @@ NBRequest::buildBitfieldLogic(bool leftHanded) {
 
 
 void
-NBRequest::computeRightOutgoingLinkCrossings(bool leftHanded, NBEdge* from, NBEdge* to) {
+NBRequest::computeRightOutgoingLinkCrossings(NBEdge* from, NBEdge* to) {
     EdgeVector::const_iterator pfrom = find(myAll.begin(), myAll.end(), from);
     while (*pfrom != to) {
         NBContHelper::nextCCW(myAll, pfrom);
@@ -177,7 +177,7 @@ NBRequest::computeRightOutgoingLinkCrossings(bool leftHanded, NBEdge* from, NBEd
             EdgeVector::const_iterator pto = find(myAll.begin(), myAll.end(), to);
             while (*pto != from) {
                 if (!((*pto)->getToNode() == myJunction)) {
-                    setBlocking(leftHanded, from, to, *pfrom, *pto);
+                    setBlocking(from, to, *pfrom, *pto);
                 }
                 NBContHelper::nextCCW(myAll, pto);
             }
@@ -187,7 +187,7 @@ NBRequest::computeRightOutgoingLinkCrossings(bool leftHanded, NBEdge* from, NBEd
 
 
 void
-NBRequest::computeLeftOutgoingLinkCrossings(bool leftHanded, NBEdge* from, NBEdge* to) {
+NBRequest::computeLeftOutgoingLinkCrossings(NBEdge* from, NBEdge* to) {
     EdgeVector::const_iterator pfrom = find(myAll.begin(), myAll.end(), from);
     while (*pfrom != to) {
         NBContHelper::nextCW(myAll, pfrom);
@@ -195,7 +195,7 @@ NBRequest::computeLeftOutgoingLinkCrossings(bool leftHanded, NBEdge* from, NBEdg
             EdgeVector::const_iterator pto = find(myAll.begin(), myAll.end(), to);
             while (*pto != from) {
                 if (!((*pto)->getToNode() == myJunction)) {
-                    setBlocking(leftHanded, from, to, *pfrom, *pto);
+                    setBlocking(from, to, *pfrom, *pto);
                 }
                 NBContHelper::nextCW(myAll, pto);
             }
@@ -205,8 +205,7 @@ NBRequest::computeLeftOutgoingLinkCrossings(bool leftHanded, NBEdge* from, NBEdg
 
 
 void
-NBRequest::setBlocking(bool leftHanded,
-                       NBEdge* from1, NBEdge* to1,
+NBRequest::setBlocking(NBEdge* from1, NBEdge* to1,
                        NBEdge* from2, NBEdge* to2) {
     // check whether one of the links has a dead end
     if (to1 == 0 || to2 == 0) {
@@ -287,11 +286,7 @@ NBRequest::setBlocking(bool leftHanded,
     while (*c1 != from1 && *c1 != from2) {
         if (*c1 == to2) {
             // if we encounter to2 the second one prohibits the first
-            if (!leftHanded) {
-                myForbids[idx2][idx1] = true;
-            } else {
-                myForbids[idx1][idx2] = true;
-            }
+            myForbids[idx2][idx1] = true;
             return;
         }
         NBContHelper::nextCW(myAll, c1);
@@ -303,11 +298,7 @@ NBRequest::setBlocking(bool leftHanded,
     while (*c2 != from2 && *c2 != from1) {
         if (*c2 == to1) {
             // if we encounter to1 the second one prohibits the first
-            if (!leftHanded) {
-                myForbids[idx1][idx2] = true;
-            } else {
-                myForbids[idx2][idx1] = true;
-            }
+            myForbids[idx1][idx2] = true;
             return;
         }
         NBContHelper::nextCW(myAll, c2);
@@ -519,7 +510,7 @@ NBRequest::writeCrossingResponse(OutputDevice& od, const NBNode::Crossing& cross
                     }
                 }
                 foes += foe ? '1' : '0';
-                response += mustBrakeForCrossing(from, to, crossing) || !foe ? '0' : '1';
+                response += mustBrakeForCrossing(myJunction, from, to, crossing) || !foe ? '0' : '1';
             }
         }
     }
@@ -536,6 +527,7 @@ NBRequest::writeCrossingResponse(OutputDevice& od, const NBNode::Crossing& cross
 std::string
 NBRequest::getResponseString(const NBEdge* const from, const NBEdge* const to,
                              int fromLane, int toLane, bool mayDefinitelyPass, const bool checkLaneFoes) const {
+    const bool lefthand = OptionsCont::getOptions().getBool("lefthand");
     int idx = 0;
     if (to != 0) {
         idx = getIndex(from, to);
@@ -543,7 +535,7 @@ NBRequest::getResponseString(const NBEdge* const from, const NBEdge* const to,
     std::string result;
     // crossings
     for (std::vector<NBNode::Crossing>::const_reverse_iterator i = myCrossings.rbegin(); i != myCrossings.rend(); i++) {
-        result += mustBrakeForCrossing(from, to, *i) ? '1' : '0';
+        result += mustBrakeForCrossing(myJunction, from, to, *i) ? '1' : '0';
     }
     // normal connections
     for (EdgeVector::const_reverse_iterator i = myIncoming.rbegin(); i != myIncoming.rend(); i++) {
@@ -566,7 +558,7 @@ NBRequest::getResponseString(const NBEdge* const from, const NBEdge* const to,
                     // check whether the connection is prohibited by another one
                     if ((myForbids[getIndex(*i, connected[k].toEdge)][idx] &&
                             (!checkLaneFoes || laneConflict(from, to, toLane, *i, connected[k].toEdge, connected[k].toLane)))
-                            || rightTurnConflict(from, to, fromLane, *i, connected[k].toEdge, connected[k].fromLane)) {
+                            || NBNode::rightTurnConflict(from, to, fromLane, *i, connected[k].toEdge, connected[k].fromLane, lefthand)) {
                         result += '1';
                     } else {
                         result += '0';
@@ -607,7 +599,7 @@ NBRequest::getFoesString(NBEdge* from, NBEdge* to, int fromLane, int toLane, con
             for (int k = size; k-- > 0;) {
                 if ((foes(from, to, (*i), connected[k].toEdge) &&
                         (!checkLaneFoes || laneConflict(from, to, toLane, *i, connected[k].toEdge, connected[k].toLane)))
-                        || rightTurnConflict(from, to, fromLane, *i, connected[k].toEdge, connected[k].fromLane)) {
+                        || NBNode::rightTurnConflict(from, to, fromLane, *i, connected[k].toEdge, connected[k].fromLane)) {
                     result += '1';
                 } else {
                     result += '0';
@@ -641,34 +633,6 @@ NBRequest::laneConflict(const NBEdge* from, const NBEdge* to, int toLane,
 }
 
 
-bool
-NBRequest::rightTurnConflict(const NBEdge* from, const NBEdge* to, int fromLane,
-                             const NBEdge* prohibitorFrom, const NBEdge* prohibitorTo, int prohibitorFromLane) const {
-    if (from != prohibitorFrom) {
-        return false;
-    }
-    if (from->isTurningDirectionAt(to)
-            || prohibitorFrom->isTurningDirectionAt(prohibitorTo)) {
-        // XXX should warn if there are any non-turning connections left of this
-        return false;
-    }
-    const bool lefthand = OptionsCont::getOptions().getBool("lefthand");
-    if ((!lefthand && fromLane <= prohibitorFromLane) ||
-            (lefthand && fromLane >= prohibitorFromLane)) {
-        return false;
-    }
-    // conflict if to is between prohibitorTo and from when going clockwise
-    if (to->getStartAngle() == prohibitorTo->getStartAngle()) {
-        // reduce rounding errors
-        return false;
-    }
-    const SUMOReal toAngleAtNode = fmod(to->getStartAngle() + 180, (SUMOReal)360.0);
-    const SUMOReal prohibitorToAngleAtNode = fmod(prohibitorTo->getStartAngle() + 180, (SUMOReal)360.0);
-    return (lefthand != (GeomHelper::getCWAngleDiff(from->getEndAngle(), toAngleAtNode) <
-                         GeomHelper::getCWAngleDiff(from->getEndAngle(), prohibitorToAngleAtNode)));
-}
-
-
 int
 NBRequest::getIndex(const NBEdge* const from, const NBEdge* const to) const {
     EdgeVector::const_iterator fp = find(myIncoming.begin(), myIncoming.end(), from);
@@ -723,7 +687,7 @@ NBRequest::mustBrake(const NBEdge* const from, const NBEdge* const to, int fromL
     // maybe we need to brake for a pedestrian crossing
     if (includePedCrossings) {
         for (std::vector<NBNode::Crossing>::const_reverse_iterator i = myCrossings.rbegin(); i != myCrossings.rend(); i++) {
-            if (mustBrakeForCrossing(from, to, *i)) {
+            if (mustBrakeForCrossing(myJunction, from, to, *i)) {
                 return true;
             }
         }
@@ -734,8 +698,8 @@ NBRequest::mustBrake(const NBEdge* const from, const NBEdge* const to, int fromL
     if (dir == LINKDIR_RIGHT || dir == LINKDIR_PARTRIGHT) {
         const std::vector<NBEdge::Connection>& cons = from->getConnections();
         for (std::vector<NBEdge::Connection>::const_iterator i = cons.begin(); i != cons.end(); i++) {
-            if (rightTurnConflict(from, to, fromLane,
-                                  from, (*i).toEdge, (*i).fromLane)) {
+            if (NBNode::rightTurnConflict(from, to, fromLane,
+                                          from, (*i).toEdge, (*i).fromLane)) {
                 return true;
             }
         }
@@ -744,8 +708,8 @@ NBRequest::mustBrake(const NBEdge* const from, const NBEdge* const to, int fromL
 }
 
 bool
-NBRequest::mustBrakeForCrossing(const NBEdge* const from, const NBEdge* const to, const NBNode::Crossing& crossing) const {
-    const LinkDirection dir = myJunction->getDirection(from, to);
+NBRequest::mustBrakeForCrossing(const NBNode* node, const NBEdge* const from, const NBEdge* const to, const NBNode::Crossing& crossing) {
+    const LinkDirection dir = node->getDirection(from, to);
     const bool mustYield = dir == LINKDIR_LEFT || dir == LINKDIR_RIGHT;
     if (crossing.priority || mustYield) {
         for (EdgeVector::const_iterator it_e = crossing.edges.begin(); it_e != crossing.edges.end(); ++it_e) {
