@@ -6,7 +6,7 @@
 /// @author  Thimor Bohn
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: NIImporter_ArcView.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: NIImporter_ArcView.cpp 18752 2015-08-30 19:09:05Z namdre $
 ///
 // Importer for networks stored in ArcView-shape format
 /****************************************************************************/
@@ -125,8 +125,13 @@ void
 NIImporter_ArcView::load() {
 #ifdef HAVE_GDAL
     PROGRESS_BEGIN_MESSAGE("Loading data from '" + mySHPName + "'");
+#if GDAL_VERSION_MAJOR < 2
     OGRRegisterAll();
     OGRDataSource* poDS = OGRSFDriverRegistrar::Open(mySHPName.c_str(), FALSE);
+#else
+    GDALAllRegister();
+    GDALDataset* poDS = (GDALDataset*) GDALOpen(mySHPName.c_str(), GA_ReadOnly);
+#endif
     if (poDS == NULL) {
         WRITE_ERROR("Could not open shape description '" + mySHPName + "'.");
         return;
