@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_OpenDrive.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NIImporter_OpenDrive.cpp 18747 2015-08-28 06:38:38Z namdre $
 ///
 // Importer for networks stored in openDrive format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -90,8 +90,8 @@ StringBijection<int>::Entry NIImporter_OpenDrive::openDriveTags[] = {
     { "junction",         NIImporter_OpenDrive::OPENDRIVE_TAG_JUNCTION },
     { "connection",       NIImporter_OpenDrive::OPENDRIVE_TAG_CONNECTION },
     { "laneLink",         NIImporter_OpenDrive::OPENDRIVE_TAG_LANELINK },
-    { "width",			  NIImporter_OpenDrive::OPENDRIVE_TAG_WIDTH },
-    { "speed",			  NIImporter_OpenDrive::OPENDRIVE_TAG_SPEED },
+    { "width",            NIImporter_OpenDrive::OPENDRIVE_TAG_WIDTH },
+    { "speed",            NIImporter_OpenDrive::OPENDRIVE_TAG_SPEED },
 
     { "",                 NIImporter_OpenDrive::OPENDRIVE_TAG_NOTHING }
 };
@@ -123,8 +123,8 @@ StringBijection<int>::Entry NIImporter_OpenDrive::openDriveAttrs[] = {
     { "dynamic",        NIImporter_OpenDrive::OPENDRIVE_ATTR_DYNAMIC },
     { "incomingRoad",   NIImporter_OpenDrive::OPENDRIVE_ATTR_INCOMINGROAD },
     { "connectingRoad", NIImporter_OpenDrive::OPENDRIVE_ATTR_CONNECTINGROAD },
-    { "from",			NIImporter_OpenDrive::OPENDRIVE_ATTR_FROM },
-    { "to",  			NIImporter_OpenDrive::OPENDRIVE_ATTR_TO },
+    { "from",           NIImporter_OpenDrive::OPENDRIVE_ATTR_FROM },
+    { "to",             NIImporter_OpenDrive::OPENDRIVE_ATTR_TO },
     { "max",            NIImporter_OpenDrive::OPENDRIVE_ATTR_MAX },
     { "sOffset",        NIImporter_OpenDrive::OPENDRIVE_ATTR_SOFFSET },
     { "name",           NIImporter_OpenDrive::OPENDRIVE_ATTR_NAME },
@@ -152,17 +152,9 @@ NIImporter_OpenDrive::loadNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
     myImportAllTypes = oc.getBool("opendrive.import-all-lanes");
     myImportWidths = !oc.getBool("opendrive.ignore-widths");
     NBTypeCont& tc = nb.getTypeCont();
-    const SUMOReal WIDTH(3.65); // as wanted
-    const SVCPermissions defaultPermissions = SVCAll & ~SVC_PEDESTRIAN;
-    tc.insert("driving",  1, (SUMOReal)(80. / 3.6), 1, defaultPermissions, WIDTH, true, NBEdge::UNSPECIFIED_WIDTH);
-    tc.insert("mwyEntry", 1, (SUMOReal)(80. / 3.6), 1, defaultPermissions, WIDTH, true, NBEdge::UNSPECIFIED_WIDTH);
-    tc.insert("mwyExit",  1, (SUMOReal)(80. / 3.6), 1, defaultPermissions, WIDTH, true, NBEdge::UNSPECIFIED_WIDTH);
-    tc.insert("stop",     1, (SUMOReal)(80. / 3.6), 1, defaultPermissions, WIDTH, true, NBEdge::UNSPECIFIED_WIDTH);
-    tc.insert("special1", 1, (SUMOReal)(80. / 3.6), 1, defaultPermissions, WIDTH, true, NBEdge::UNSPECIFIED_WIDTH);
-    tc.insert("parking",  1, (SUMOReal)(5. / 3.6),  1, defaultPermissions, WIDTH, true, NBEdge::UNSPECIFIED_WIDTH);
     // build the handler
     std::map<std::string, OpenDriveEdge*> edges;
-    NIImporter_OpenDrive handler(tc, edges);
+    NIImporter_OpenDrive handler(nb.getTypeCont(), edges);
     // parse file(s)
     std::vector<std::string> files = oc.getStringVector("opendrive-files");
     for (std::vector<std::string>::const_iterator file = files.begin(); file != files.end(); ++file) {
@@ -332,7 +324,9 @@ NIImporter_OpenDrive::loadNetwork(const OptionsCont& oc, NBNetBuilder& nb) {
         int priorityL = e->getPriority(OPENDRIVE_TAG_LEFT);
         SUMOReal sB = 0;
         SUMOReal sE = e->length;
-        SUMOReal cF = e->length / e->geom.length2D();
+        // 0-length geometries are possible if only the inner points are represented
+        const SUMOReal length2D = e->geom.length2D();
+        SUMOReal cF = length2D == 0 ? 1 : e->length / length2D;
         NBEdge* prevRight = 0;
         NBEdge* prevLeft = 0;
 
@@ -1086,14 +1080,14 @@ NIImporter_OpenDrive::OpenDriveLaneSection::getInnerConnections(OpenDriveXMLTag
         }
         if (from != UNSET_CONNECTION && to != UNSET_CONNECTION) {
             if (ret.find(from) != ret.end()) {
-//				WRITE_WARNING("double connection");
+//        WRITE_WARNING("double connection");
             }
             if (dir == OPENDRIVE_TAG_LEFT) {
                 std::swap(from, to);
             }
             ret[from] = to;
         } else {
-//			WRITE_WARNING("missing connection");
+//      WRITE_WARNING("missing connection");
         }
     }
     return ret;
