@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Mon, 14.04.2008
-/// @version $Id: NIImporter_SUMO.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: NIImporter_SUMO.h 18319 2015-04-29 06:49:01Z namdre $
 ///
 // Importer for networks stored in SUMO format
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -189,6 +189,8 @@ private:
         unsigned int tlLinkNo;
         /// @brief Information about being definitely free to drive (on-ramps)
         bool mayDefinitelyPass;
+        /// @brief Whether the junction must be kept clear coming from this connection
+        bool keepClear;
     };
 
 
@@ -316,6 +318,12 @@ private:
     /// @brief loaded roundabout edges
     std::vector<std::vector<std::string> > myRoundabouts;
 
+    /// @brief customLaneShape (cannot be added to the NBNode when parsed since the node doesn't yet exist
+    std::map<std::string, NBNode::CustomShapeMap> myCustomShapeMaps;
+
+    /// @brief list of node id with rail signals (no NBTrafficLightDefinition exists)
+    std::set<std::string> myRailSignals;
+
     /** @brief Parses lane index from lane ID an retrieve lane from EdgeAttrs
      * @param[in] edge The EdgeAttrs* which should contain the lane
      * @param[in] lane_id The ID of the lane
