@@ -6,7 +6,7 @@
 /// @author  Walter Bamberger
 /// @author  Laura Bieker
 /// @date    Tue, 20 Nov 2001
-/// @version $Id: NIXMLEdgesHandler.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: NIXMLEdgesHandler.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // Importer for network edges stored in XML
 /****************************************************************************/
@@ -52,6 +52,7 @@
 #include <utils/common/ToString.h>
 #include <utils/options/OptionsCont.h>
 #include <utils/geom/GeoConvHelper.h>
+#include "NIXMLNodesHandler.h"
 #include "NIXMLEdgesHandler.h"
 
 #ifdef CHECK_MEMORY_LEAKS
@@ -81,6 +82,7 @@ NIXMLEdgesHandler::NIXMLEdgesHandler(NBNodeCont& nc,
       myDistrictCont(dc),
       myTLLogicCont(tlc),
       myCurrentEdge(0), myHaveReportedAboutOverwriting(false),
+      myHaveReportedAboutTypeOverride(false),
       myHaveWarnedAboutDeprecatedLaneId(false),
       myKeepEdgeShape(!options.getBool("plain.extend-edge-shape"))
 {}
@@ -134,6 +136,7 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
     myCurrentEndOffset = NBEdge::UNSPECIFIED_OFFSET;
     if (myCurrentEdge != 0) {
         // update existing edge. only update lane-specific settings when explicitly requested
+        myIsUpdate = true;
         myCurrentSpeed = NBEdge::UNSPECIFIED_SPEED;
         myPermissions = SVC_UNSPECIFIED;
         myCurrentWidth = NBEdge::UNSPECIFIED_WIDTH;
@@ -150,6 +153,7 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
     myCurrentStreetName = "";
     myReinitKeepEdgeShape = false;
     mySidewalkWidth = NBEdge::UNSPECIFIED_WIDTH;
+    myBikeLaneWidth = NBEdge::UNSPECIFIED_WIDTH;
     // check whether a type's values shall be used
     if (attrs.hasAttribute(SUMO_ATTR_TYPE)) {
         myCurrentType = attrs.get<std::string>(SUMO_ATTR_TYPE, myCurrentID.c_str(), ok);
@@ -166,14 +170,20 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
         myPermissions = myTypeCont.getPermissions(myCurrentType);
         myCurrentWidth = myTypeCont.getWidth(myCurrentType);
         mySidewalkWidth = myTypeCont.getSidewalkWidth(myCurrentType);
+        myBikeLaneWidth = myTypeCont.getBikeLaneWidth(myCurrentType);
     }
     // use values from the edge to overwrite if existing, then
-    if (myCurrentEdge != 0) {
-        myIsUpdate = true;
+    if (myIsUpdate) {
         if (!myHaveReportedAboutOverwriting) {
-            WRITE_MESSAGE("Duplicate edge id occured ('" + myCurrentID + "'); assuming overwriting is wished.");
+            WRITE_MESSAGE("Duplicate edge id occurred ('" + myCurrentID + "'); assuming overwriting is wished.");
             myHaveReportedAboutOverwriting = true;
         }
+        if (attrs.hasAttribute(SUMO_ATTR_TYPE) && myCurrentType != myCurrentEdge->getTypeID()) {
+            if (!myHaveReportedAboutTypeOverride) {
+                WRITE_MESSAGE("Edge '" + myCurrentID + "' changed it's type; assuming type override is wished.");
+                myHaveReportedAboutTypeOverride = true;
+            }
+        }
         if (attrs.getOpt<bool>(SUMO_ATTR_REMOVE, myCurrentID.c_str(), ok, false)) {
             myEdgeCont.erase(myDistrictCont, myCurrentEdge);
             myCurrentEdge = 0;
@@ -181,7 +191,6 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
         }
         myCurrentPriority = myCurrentEdge->getPriority();
         myCurrentLaneNo = myCurrentEdge->getNumLanes();
-        myCurrentType = myCurrentEdge->getTypeID();
         if (!myCurrentEdge->hasDefaultGeometry()) {
             myShape = myCurrentEdge->getGeometry();
             myReinitKeepEdgeShape = true;
@@ -242,8 +251,10 @@ NIXMLEdgesHandler::addEdge(const SUMOSAXAttributes& attrs) {
     myLanesSpread = tryGetLaneSpread(attrs);
     // try to get the length
     myLength = attrs.getOpt<SUMOReal>(SUMO_ATTR_LENGTH, myCurrentID.c_str(), ok, myLength);
-    // tro to get the sidewalkWidth
+    // try to get the sidewalkWidth
     mySidewalkWidth = attrs.getOpt<SUMOReal>(SUMO_ATTR_SIDEWALKWIDTH, myCurrentID.c_str(), ok, mySidewalkWidth);
+    // try to get the bikeLaneWidth
+    myBikeLaneWidth = attrs.getOpt<SUMOReal>(SUMO_ATTR_BIKELANEWIDTH, myCurrentID.c_str(), ok, myBikeLaneWidth);
     // insert the parsed edge into the edges map
     if (!ok) {
         return;
@@ -344,7 +355,7 @@ void NIXMLEdgesHandler::addSplit(const SUMOSAXAttributes& attrs) {
             WRITE_ERROR("Edge '" + myCurrentID + "' has already a split at position " + toString(e.pos) + ".");
             return;
         }
-        e.nameid = (int)e.pos;
+        const std::string nameid = toString((int)e.pos);
         if (e.pos < 0) {
             e.pos += myCurrentEdge->getGeometry().length();
         }
@@ -372,6 +383,10 @@ void NIXMLEdgesHandler::addSplit(const SUMOSAXAttributes& attrs) {
         if (!ok) {
             return;
         }
+        e.node = new NBNode(myCurrentID + "." + nameid,
+                            myCurrentEdge->getGeometry().positionAtOffset(e.pos));
+        NIXMLNodesHandler::processNodeType(attrs, e.node, e.node->getID(), e.node->getPosition(), false,
+                                           myNodeCont, myTLLogicCont);
         mySplits.push_back(e);
     }
 }
@@ -473,6 +488,10 @@ NIXMLEdgesHandler::deleteEdge(const SUMOSAXAttributes& attrs) {
 void
 NIXMLEdgesHandler::myEndElement(int element) {
     if (element == SUMO_TAG_EDGE && myCurrentEdge != 0) {
+        // add bike lane, wait until lanes are loaded to avoid building if it already exists
+        if (myBikeLaneWidth != NBEdge::UNSPECIFIED_WIDTH) {
+            myCurrentEdge->addBikeLane(myBikeLaneWidth);
+        }
         // add sidewalk, wait until lanes are loaded to avoid building if it already exists
         if (mySidewalkWidth != NBEdge::UNSPECIFIED_WIDTH) {
             myCurrentEdge->addSidewalk(mySidewalkWidth);
@@ -497,7 +516,6 @@ NIXMLEdgesHandler::myEndElement(int element) {
             unsigned int noLanesMax = e->getNumLanes();
             // compute the node positions and sort the lanes
             for (i = mySplits.begin(); i != mySplits.end(); ++i) {
-                (*i).gpos = e->getGeometry().positionAtOffset((*i).pos);
                 sort((*i).lanes.begin(), (*i).lanes.end());
                 noLanesMax = MAX2(noLanesMax, (unsigned int)(*i).lanes.size());
             }
@@ -521,18 +539,16 @@ NIXMLEdgesHandler::myEndElement(int element) {
                 const Split& exp = *i;
                 assert(exp.lanes.size() != 0);
                 if (exp.pos > 0 && e->getGeometry().length() + seen > exp.pos && exp.pos > seen) {
-                    std::string nid = edgeid + "." +  toString(exp.nameid);
-                    NBNode* rn = new NBNode(nid, exp.gpos);
-                    if (myNodeCont.insert(rn)) {
+                    if (myNodeCont.insert(exp.node)) {
+                        myNodeCont.markAsSplit(exp.node);
                         //  split the edge
-                        std::string nid = myCurrentID + "." +  toString(exp.nameid);
                         std::string pid = e->getID();
-                        myEdgeCont.splitAt(myDistrictCont, e, exp.pos - seen, rn,
-                                           pid, nid, e->getNumLanes(), (unsigned int) exp.lanes.size(), exp.speed);
+                        myEdgeCont.splitAt(myDistrictCont, e, exp.pos - seen, exp.node,
+                                           pid, exp.node->getID(), e->getNumLanes(), (unsigned int) exp.lanes.size(), exp.speed);
                         seen = exp.pos;
                         std::vector<int> newLanes = exp.lanes;
                         NBEdge* pe = myEdgeCont.retrieve(pid);
-                        NBEdge* ne = myEdgeCont.retrieve(nid);
+                        NBEdge* ne = myEdgeCont.retrieve(exp.node->getID());
                         // reconnect lanes
                         pe->invalidateConnections(true);
                         //  new on right
