@@ -5,12 +5,12 @@
 /// @author  Sascha Krieg
 /// @author  Michael Behrisch
 /// @date    Mon, 9 Jul 2001
-/// @version $Id: NLJunctionControlBuilder.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: NLJunctionControlBuilder.cpp 18213 2015-04-16 11:55:42Z behrisch $
 ///
 // Builder of microsim-junctions and tls
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -43,9 +43,9 @@
 #include <microsim/MSJunctionControl.h>
 #include <microsim/traffic_lights/MSTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSSimpleTrafficLightLogic.h>
+#include <microsim/traffic_lights/MSRailSignal.h>
 #include <microsim/MSEventControl.h>
 #include <microsim/MSGlobals.h>
-#include <microsim/traffic_lights/MSAgentbasedTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSOffTrafficLightLogic.h>
 #include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <utils/xml/SUMOXMLDefinitions.h>
@@ -135,6 +135,14 @@ NLJunctionControlBuilder::closeJunction() {
             }
 #endif
             break;
+        case NODETYPE_RAIL_SIGNAL:
+            myOffset = 0;
+            myActiveKey = myActiveID;
+            myActiveProgram = "0";
+            myLogicType = TLTYPE_RAIL;
+            closeTrafficLightLogic();
+            junction = buildLogicJunction();
+            break;
         default:
             throw InvalidArgument("False junction logic type.");
     }
@@ -214,29 +222,30 @@ NLJunctionControlBuilder::closeTrafficLightLogic() {
         }
         return;
     }
-    if (myAbsDuration == 0) {
-        throw InvalidArgument("TLS program '" + myActiveProgram + "' for TLS '" + myActiveKey + "' has a duration of 0.");
-    }
-    // compute the initial step and first switch time of the tls-logic
-    // a positive offset delays all phases by x (advance by absDuration - x) while a negative offset advances all phases by x seconds
-    // @note The implementation of % for negative values is implementation defined in ISO1998
-    SUMOTime offset; // the time to run the traffic light in advance
-    if (myOffset >= 0) {
-        offset = (myNet.getCurrentTimeStep() + myAbsDuration - (myOffset % myAbsDuration)) % myAbsDuration;
-    } else {
-        offset = (myNet.getCurrentTimeStep() + ((-myOffset) % myAbsDuration)) % myAbsDuration;
-    }
-    unsigned int step = 0;
     SUMOTime firstEventOffset = 0;
+    unsigned int step = 0;
     MSSimpleTrafficLightLogic::Phases::const_iterator i = myActivePhases.begin();
-    while (offset >= (*i)->duration) {
-        step++;
-        offset -= (*i)->duration;
-        ++i;
+    if (myLogicType != TLTYPE_RAIL) {
+        if (myAbsDuration == 0) {
+            throw InvalidArgument("TLS program '" + myActiveProgram + "' for TLS '" + myActiveKey + "' has a duration of 0.");
+        }
+        // compute the initial step and first switch time of the tls-logic
+        // a positive offset delays all phases by x (advance by absDuration - x) while a negative offset advances all phases by x seconds
+        // @note The implementation of % for negative values is implementation defined in ISO1998
+        SUMOTime offset; // the time to run the traffic light in advance
+        if (myOffset >= 0) {
+            offset = (myNet.getCurrentTimeStep() + myAbsDuration - (myOffset % myAbsDuration)) % myAbsDuration;
+        } else {
+            offset = (myNet.getCurrentTimeStep() + ((-myOffset) % myAbsDuration)) % myAbsDuration;
+        }
+        while (offset >= (*i)->duration) {
+            step++;
+            offset -= (*i)->duration;
+            ++i;
+        }
+        firstEventOffset = (*i)->duration - offset + myNet.getCurrentTimeStep();
     }
-    firstEventOffset = (*i)->duration - offset + myNet.getCurrentTimeStep();
 
-    //
     if (myActiveProgram == "") {
         myActiveProgram = "default";
     }
@@ -251,12 +260,6 @@ NLJunctionControlBuilder::closeTrafficLightLogic() {
                     myActivePhases, step, (*i)->minDuration + myNet.getCurrentTimeStep(),
                     myAdditionalParameter);
             break;
-        case TLTYPE_AGENT:
-            tlLogic = new MSAgentbasedTrafficLightLogic(getTLLogicControlToUse(),
-                    myActiveKey, myActiveProgram,
-                    myActivePhases, step, firstEventOffset,
-                    myAdditionalParameter);
-            break;
         case TLTYPE_STATIC:
             tlLogic =
                 new MSSimpleTrafficLightLogic(getTLLogicControlToUse(),
@@ -264,6 +267,11 @@ NLJunctionControlBuilder::closeTrafficLightLogic() {
                                               myActivePhases, step, firstEventOffset,
                                               myAdditionalParameter);
             break;
+        case TLTYPE_RAIL:
+            tlLogic = new MSRailSignal(getTLLogicControlToUse(),
+                                       myActiveKey, myActiveProgram,
+                                       myAdditionalParameter);
+            break;
     }
     myActivePhases.clear();
     if (tlLogic != 0) {
@@ -352,7 +360,7 @@ NLJunctionControlBuilder::initTrafficLightLogic(const std::string& id, const std
 
 void
 NLJunctionControlBuilder::addPhase(SUMOTime duration, const std::string& state,
-                                   int minDuration, int maxDuration) {
+                                   SUMOTime minDuration, SUMOTime maxDuration) {
     // build and add the phase definition to the list
     myActivePhases.push_back(new MSPhaseDefinition(duration, minDuration, maxDuration, state));
     // add phase duration to the absolute duration
