@@ -6,7 +6,7 @@
 /// @author  Michael Behrisch
 /// @author  Melanie Knocke
 /// @date    Mon, 05 Dec 2005
-/// @version $Id: polyconvert_main.cpp 18096 2015-03-17 09:50:59Z behrisch $
+/// @version $Id: polyconvert_main.cpp 18703 2015-08-24 07:29:42Z namdre $
 ///
 // Main for POLYCONVERT
 /****************************************************************************/
@@ -187,6 +187,8 @@ fillOptions() {
     oc.doRegister("ignore-errors", new Option_Bool(false));
     oc.addDescription("ignore-errors", "Processing", "Continue on broken input");
 
+    oc.doRegister("poi-layer-offset", new Option_Float(0));
+    oc.addDescription("poi-layer-offset", "Processing", "Adds FLOAT to the layer value for each poi (i.e. to raise it above polygons)");
 
     // building defaults options
     oc.doRegister("color", new Option_String("0.2,0.5,1."));
@@ -220,7 +222,8 @@ main(int argc, char** argv) {
         // initialise subsystems
         XMLSubSys::init();
         fillOptions();
-        OptionsIO::getOptions(true, argc, argv);
+        OptionsIO::setArgs(argc, argv);
+        OptionsIO::getOptions();
         if (oc.processMetaOptions(argc < 2)) {
             SystemFrame::close();
             return 0;
@@ -272,6 +275,9 @@ main(int argc, char** argv) {
             pruningBoundary = GeomConvHelper::parseBoundaryReporting(oc.getString("prune.boundary"), "--prune.boundary", 0, ok);
             prune = true;
         }
+        if (oc.isSet("osm-files") && oc.isDefault("poi-layer-offset")) {
+            oc.set("poi-layer-offset", "5"); // sufficient when using the default typemap
+        }
 
         PCPolyContainer toFill(prune, pruningBoundary, oc.getStringVector("remove"));
 
