@@ -7,12 +7,12 @@
 /// @author  Melanie Knocke
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: ROEdge.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: ROEdge.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // A basic edge for routing applications
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -40,6 +40,7 @@
 #include <iostream>
 #include "ROLane.h"
 #include "ROEdge.h"
+#include "RONet.h"
 #include "ROVehicle.h"
 #include <utils/vehicle/SUMOVTypeParameter.h>
 #include <utils/emissions/PollutantsInterface.h>
@@ -56,26 +57,34 @@
 bool ROEdge::myUseBoundariesOnOverrideTT = false;
 bool ROEdge::myUseBoundariesOnOverrideE = false;
 bool ROEdge::myInterpolate = false;
+bool ROEdge::myAmParallel = false;
 bool ROEdge::myHaveTTWarned = false;
 bool ROEdge::myHaveEWarned = false;
-std::vector<ROEdge*> ROEdge::myEdges;
+ROEdgeVector ROEdge::myEdges;
 
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-ROEdge::ROEdge(const std::string& id, RONode* from, RONode* to, unsigned int index, const int priority)
-    : Named(id), myFromNode(from), myToNode(to), myIndex(index), myPriority(priority),
-      mySpeed(-1), myLength(0),
-      myUsingTTTimeLine(false),
-      myUsingETimeLine(false),
-      myCombinedPermissions(0),
-      myFromJunction(0),
-      myToJunction(0) {
+ROEdge::ROEdge(const std::string& id, RONode* from, RONode* to, unsigned int index, const int priority) :
+    Named(id),
+    myFromJunction(from),
+    myToJunction(to),
+    myIndex(index),
+    myPriority(priority),
+    mySpeed(-1),
+    myLength(0),
+    myUsingTTTimeLine(false),
+    myUsingETimeLine(false),
+    myCombinedPermissions(0) {
     while (myEdges.size() <= index) {
         myEdges.push_back(0);
     }
     myEdges[index] = this;
+    if (from == 0 && to == 0) {
+        // TAZ edge, no lanes
+        myCombinedPermissions = SVCAll;
+    }
 }
 
 
@@ -134,8 +143,8 @@ ROEdge::getEffort(const ROVehicle* const veh, SUMOReal time) const {
 
 SUMOReal
 ROEdge::getDistanceTo(const ROEdge* other) const {
-    if (getToNode() != 0 && other->getFromNode() != 0) {
-        return getToNode()->getPosition().distanceTo2D(other->getFromNode()->getPosition());
+    if (getToJunction() != 0 && other->getFromJunction() != 0) {
+        return getToJunction()->getPosition().distanceTo2D(other->getFromJunction()->getPosition());
     } else {
         return 0; // optimism is just right for astar
     }
@@ -205,7 +214,7 @@ ROEdge::getStoredEffort(SUMOReal time, SUMOReal& ret) const {
 
 unsigned int
 ROEdge::getNumSuccessors() const {
-    if (getType() == ET_SINK) {
+    if (getFunc() == ET_SINK) {
         return 0;
     }
     return (unsigned int) myFollowingEdges.size();
@@ -214,19 +223,13 @@ ROEdge::getNumSuccessors() const {
 
 unsigned int
 ROEdge::getNumPredecessors() const {
-    if (getType() == ET_SOURCE) {
+    if (getFunc() == ET_SOURCE) {
         return 0;
     }
     return (unsigned int) myApproachingEdges.size();
 }
 
 
-void
-ROEdge::setType(ROEdge::EdgeType type) {
-    myType = type;
-}
-
-
 void
 ROEdge::buildTimeLines(const std::string& measure) {
     if (myUsingETimeLine) {
@@ -260,7 +263,7 @@ ROEdge::buildTimeLines(const std::string& measure) {
 
 bool
 ROEdge::allFollowersProhibit(const ROVehicle* const vehicle) const {
-    for (std::vector<ROEdge*>::const_iterator i = myFollowingEdges.begin(); i != myFollowingEdges.end(); ++i) {
+    for (ROEdgeVector::const_iterator i = myFollowingEdges.begin(); i != myFollowingEdges.end(); ++i) {
         if (!(*i)->prohibits(vehicle)) {
             return false;
         }
@@ -276,6 +279,65 @@ ROEdge::dictionary(size_t id) {
 }
 
 
+const ROEdgeVector&
+ROEdge::getSuccessors(SUMOVehicleClass vClass) const {
+    if (vClass == SVC_IGNORING || !RONet::getInstance()->hasPermissions() || myFunc == ET_DISTRICT) {
+        return myFollowingEdges;
+    }
+#ifdef HAVE_FOX
+    if (myAmParallel) {
+        RONet::getInstance()->lock();
+    }
+#endif
+    std::map<SUMOVehicleClass, ROEdgeVector>::const_iterator i = myClassesSuccessorMap.find(vClass);
+    if (i != myClassesSuccessorMap.end()) {
+        // can use cached value
+#ifdef HAVE_FOX
+        if (myAmParallel) {
+            RONet::getInstance()->unlock();
+        }
+#endif
+        return i->second;
+    } else {
+        // this vClass is requested for the first time. rebuild all successors
+        std::set<ROEdge*> followers;
+        for (std::vector<ROLane*>::const_iterator it = myLanes.begin(); it != myLanes.end(); ++it) {
+            ROLane* lane = *it;
+            if ((lane->getPermissions() & vClass) != 0) {
+                const std::vector<const ROLane*>& outgoing = lane->getOutgoingLanes();
+                for (std::vector<const ROLane*>::const_iterator it2 = outgoing.begin(); it2 != outgoing.end(); ++it2) {
+                    const ROLane* next = *it2;
+                    if ((next->getPermissions() & vClass) != 0) {
+                        followers.insert(&next->getEdge());
+                    }
+                }
+            }
+        }
+        // also add district edges (they are not connected at the lane level
+        for (ROEdgeVector::const_iterator it = myFollowingEdges.begin(); it != myFollowingEdges.end(); ++it) {
+            if ((*it)->getFunc() == ET_DISTRICT) {
+                followers.insert(*it);
+            }
+        }
+        myClassesSuccessorMap[vClass].insert(myClassesSuccessorMap[vClass].begin(),
+                                             followers.begin(), followers.end());
+#ifdef HAVE_FOX
+        if (myAmParallel) {
+            RONet::getInstance()->unlock();
+        }
+#endif
+        return myClassesSuccessorMap[vClass];
+    }
+
+}
+
+
+bool
+ROEdge::isConnectedTo(const ROEdge* const e, const ROVehicle* const vehicle) const {
+    const SUMOVehicleClass vClass = (vehicle == 0 ? SVC_IGNORING : vehicle->getVClass());
+    const ROEdgeVector& followers = getSuccessors(vClass);
+    return std::find(followers.begin(), followers.end(), e) != followers.end();
+}
 
 /****************************************************************************/
 
