@@ -7,7 +7,7 @@
 /// @author  Melanie Knocke
 /// @author  Yun-Pang Floetteroed
 /// @date    Sept 2002
-/// @version $Id: ROEdge.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: ROEdge.h 18627 2015-08-13 08:58:59Z namdre $
 ///
 // A basic edge for routing applications
 /****************************************************************************/
@@ -73,10 +73,10 @@ typedef std::vector<const ROEdge*> ConstROEdgeVector;
 class ROEdge : public Named {
 public:
     /**
-     * @enum EdgeType
-     * @brief Possible types of edges
+     * @enum EdgeFunc
+     * @brief Possible functions of edges
      */
-    enum EdgeType {
+    enum EdgeFunc {
         /// @brief A normal edge
         ET_NORMAL,
         /// @brief An edge representing a whole district
@@ -131,24 +131,35 @@ public:
     virtual void addSuccessor(ROEdge* s, std::string dir = "");
 
 
-    /** @brief Sets the type of te edge
-     * @param[in] type The new type for the edge
+    /** @brief Sets the function of the edge
+     * @param[in] func The new function for the edge
      */
-    void setType(EdgeType type);
+    inline void setFunc(EdgeFunc func) {
+        myFunc = func;
+    }
+
+
+    /** @brief Sets the vehicle class specific speed limits of the edge
+     * @param[in] restrictions The restrictions for the edge
+     */
+    inline void setRestrictions(const std::map<SUMOVehicleClass, SUMOReal>* restrictions) {
+        myRestrictions = restrictions;
+    }
+
 
     /// @brief return whether this edge is an internal edge
     inline bool isInternal() const {
-        return myType == ET_INTERNAL;
+        return myFunc == ET_INTERNAL;
     }
 
     /// @brief return whether this edge is a pedestrian crossing
     inline bool isCrossing() const {
-        return myType == ET_CROSSING;
+        return myFunc == ET_CROSSING;
     }
 
     /// @brief return whether this edge is walking area
     inline bool isWalkingArea() const {
-        return myType == ET_WALKINGAREA;
+        return myFunc == ET_WALKINGAREA;
     }
 
     /** @brief Builds the internal representation of the travel time/effort
@@ -168,12 +179,12 @@ public:
     /// @name Getter methods
     //@{
 
-    /** @brief Returns the type of the edge
-     * @return This edge's type
-     * @see EdgeType
+    /** @brief Returns the function of the edge
+     * @return This edge's basic function
+     * @see EdgeFunc
      */
-    EdgeType getType() const {
-        return myType;
+    EdgeFunc getFunc() const {
+        return myFunc;
     }
 
 
@@ -208,22 +219,6 @@ public:
     }
 
 
-    /** @brief Returns the node this edge starts at
-     * @return The node this edge starts at
-     */
-    RONode* getFromNode() const {
-        return myFromNode;
-    }
-
-
-    /** @brief Returns the node this edge ends at
-     * @return The node this edge ends at
-     */
-    RONode* getToNode() const {
-        return myToNode;
-    }
-
-
     /** @brief returns the information whether this edge is directly connected to the given
      *
      * @param[in] e The edge which may be connected
@@ -403,13 +398,15 @@ public:
         return myEdges.size();
     };
 
-    static void setTimeLineOptions(
+    static void setGlobalOptions(
         bool useBoundariesOnOverrideTT,
         bool useBoundariesOnOverrideE,
-        bool interpolate) {
+        bool interpolate,
+        bool isParallel) {
         myUseBoundariesOnOverrideTT = useBoundariesOnOverrideTT;
         myUseBoundariesOnOverrideE = useBoundariesOnOverrideE;
         myInterpolate = interpolate;
+        myAmParallel = isParallel;
     }
 
     /// @brief get edge priority (road class)
@@ -426,11 +423,6 @@ public:
     }
 
 
-    void setJunctions(RONode* from, RONode* to) {
-        myFromJunction = from;
-        myToJunction = to;
-    }
-
     /** @brief Returns this edge's lanes
      *
      * @return This edge's lanes
@@ -450,8 +442,9 @@ protected:
 
 
 protected:
-    /// @brief The nodes this edge is connecting
-    RONode* const myFromNode, * const myToNode;
+    /// @brief the junctions for this edge
+    RONode* myFromJunction;
+    RONode* myToJunction;
 
     /// @brief The index (numeric id) of the edge
     const unsigned int myIndex;
@@ -483,6 +476,9 @@ protected:
     /// @brief Information whether to interpolate at interval boundaries
     static bool myInterpolate;
 
+    /// @brief Information whether we are routing multi-threaded
+    static bool myAmParallel;
+
     /// @brief Information whether the edge has reported missing weights
     static bool myHaveEWarned;
     /// @brief Information whether the edge has reported missing weights
@@ -494,8 +490,11 @@ protected:
     /// @brief List of edges that approached this edge
     ROEdgeVector myApproachingEdges;
 
-    /// @brief The type of the edge
-    EdgeType myType;
+    /// @brief The function of the edge
+    EdgeFunc myFunc;
+
+    /// The vClass speed restrictions for this edge
+    const std::map<SUMOVehicleClass, SUMOReal>* myRestrictions;
 
     /// @brief This edge's lanes
     std::vector<ROLane*> myLanes;
@@ -505,13 +504,9 @@ protected:
 
     static ROEdgeVector myEdges;
 
-    /// @brief the junctions for this edge
-    RONode* myFromJunction;
-    RONode* myToJunction;
 
     /// @brief The successors available for a given vClass
-    typedef std::map<SUMOVehicleClass, ROEdgeVector> ClassesSuccesorMap;
-    mutable ClassesSuccesorMap myClassesSuccessorMap;
+    mutable std::map<SUMOVehicleClass, ROEdgeVector> myClassesSuccessorMap;
 
 private:
     /// @brief Invalidated copy constructor
