@@ -6,7 +6,7 @@
 /// @author  Michael Behrisch
 /// @author  Christian Roessel
 /// @date    Sept 2002
-/// @version $Id: ROLoader.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: ROLoader.cpp 18397 2015-05-18 14:04:14Z behrisch $
 ///
 // Loader for networks and route imports
 /****************************************************************************/
@@ -54,7 +54,6 @@
 #include "ROLoader.h"
 #include "ROEdge.h"
 #include "RORouteHandler.h"
-#include "RORouteAggregator.h"
 
 #ifdef CHECK_MEMORY_LEAKS
 #include <foreign/nvwa/debug_new.h>
@@ -182,7 +181,12 @@ ROLoader::openRoutes(RONet& net) {
             if (MsgHandler::getErrorInstance()->wasInformed()) {
                 throw ProcessError();
             } else {
-                throw ProcessError("No route input specified or all routes were invalid.");
+                const std::string error = "No route input specified or all routes were invalid.";
+                if (myOptions.getBool("ignore-errors")) {
+                    WRITE_WARNING(error);
+                } else {
+                    throw ProcessError(error);
+                }
             }
         }
         // skip routes prior to the begin time
@@ -225,15 +229,6 @@ ROLoader::processRoutes(const SUMOTime start, const SUMOTime end, const SUMOTime
 }
 
 
-void
-ROLoader::processAllRoutesWithBulkRouter(SUMOTime /* start */, SUMOTime end,
-        RONet& net, SUMOAbstractRouter<ROEdge, ROVehicle>& router) {
-    myLoaders.loadNext(SUMOTime_MAX);
-    RORouteAggregator::processAllRoutes(net, router);
-    net.saveAndRemoveRoutesUntil(myOptions, router, end);
-}
-
-
 bool
 ROLoader::openTypedRoutes(const std::string& optionName,
                           RONet& net) {
