@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: ROLoader.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: ROLoader.h 18397 2015-05-18 14:04:14Z behrisch $
 ///
 // Loader for networks and route imports
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2002-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2002-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -88,10 +88,6 @@ public:
     void processRoutes(const SUMOTime start, const SUMOTime end, const SUMOTime increment,
                        RONet& net, SUMOAbstractRouter<ROEdge, ROVehicle>& router);
 
-    /** @brief Loads all routes and processes them with BulkStarRouter */
-    void processAllRoutesWithBulkRouter(SUMOTime start, SUMOTime end,
-                                        RONet& net, SUMOAbstractRouter<ROEdge, ROVehicle>& router);
-
 protected:
     /** @brief Opens route handler of the given type
      *
