@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: RONet.cpp 18136 2015-03-24 15:10:45Z behrisch $
+/// @version $Id: RONet.cpp 18467 2015-05-29 03:50:41Z behrisch $
 ///
 // The router's network representation
 /****************************************************************************/
@@ -52,20 +52,39 @@
 #endif // CHECK_MEMORY_LEAKS
 
 
+// ===========================================================================
+// static member definitions
+// ===========================================================================
+RONet* RONet::myInstance = 0;
+
+
 // ===========================================================================
 // method definitions
 // ===========================================================================
+RONet*
+RONet::getInstance(void) {
+    if (myInstance != 0) {
+        return myInstance;
+    }
+    throw ProcessError("A network was not yet constructed.");
+}
+
+
 RONet::RONet()
     : myVehicleTypes(), myDefaultVTypeMayBeDeleted(true),
       myRoutesOutput(0), myRouteAlternativesOutput(0), myTypesOutput(0),
       myReadRouteNo(0), myDiscardedRouteNo(0), myWrittenRouteNo(0),
-      myHaveRestrictions(false),
+      myHavePermissions(false),
       myNumInternalEdges(0),
       myErrorHandler(OptionsCont::getOptions().exists("ignore-errors")
                      && OptionsCont::getOptions().getBool("ignore-errors") ? MsgHandler::getWarningInstance() : MsgHandler::getErrorInstance()) {
+    if (myInstance != 0) {
+        throw ProcessError("A network was already constructed.");
+    }
     SUMOVTypeParameter* type = new SUMOVTypeParameter(DEFAULT_VTYPE_ID, SVC_IGNORING);
     type->onlyReferenced = true;
     myVehicleTypes.add(type->id, type);
+    myInstance = this;
 }
 
 
@@ -78,6 +97,22 @@ RONet::~RONet() {
 }
 
 
+void
+RONet::addRestriction(const std::string& id, const SUMOVehicleClass svc, const SUMOReal speed) {
+    myRestrictions[id][svc] = speed;
+}
+
+
+const std::map<SUMOVehicleClass, SUMOReal>*
+RONet::getRestrictions(const std::string& id) const {
+    std::map<std::string, std::map<SUMOVehicleClass, SUMOReal> >::const_iterator i = myRestrictions.find(id);
+    if (i == myRestrictions.end()) {
+        return 0;
+    }
+    return &i->second;
+}
+
+
 bool
 RONet::addEdge(ROEdge* edge) {
     if (!myEdges.add(edge->getID(), edge)) {
@@ -85,7 +120,7 @@ RONet::addEdge(ROEdge* edge) {
         delete edge;
         return false;
     }
-    if (edge->getType() == ROEdge::ET_INTERNAL) {
+    if (edge->getFunc() == ROEdge::ET_INTERNAL) {
         myNumInternalEdges += 1;
     }
     return true;
@@ -100,9 +135,9 @@ RONet::addDistrict(const std::string id, ROEdge* source, ROEdge* sink) {
         delete sink;
         return false;
     }
-    sink->setType(ROEdge::ET_DISTRICT);
+    sink->setFunc(ROEdge::ET_DISTRICT);
     addEdge(sink);
-    source->setType(ROEdge::ET_DISTRICT);
+    source->setFunc(ROEdge::ET_DISTRICT);
     addEdge(source);
     myDistricts[id] = std::make_pair(std::vector<std::string>(), std::vector<std::string>());
     return true;
@@ -420,6 +455,50 @@ RONet::checkFlows(SUMOTime time) {
 }
 
 
+void
+RONet::createBulkRouteRequests(SUMOAbstractRouter<ROEdge, ROVehicle>& router, const SUMOTime time, const bool removeLoops, const std::map<std::string, ROVehicle*>& mmap) {
+    std::map<const ROEdge*, std::vector<ROVehicle*> > bulkVehs;
+    for (std::map<std::string, ROVehicle*>::const_iterator i = mmap.begin(); i != mmap.end(); ++i) {
+        ROVehicle* const vehicle = i->second;
+        if (vehicle->getDepart() < time) {
+            const RORoute* const stub = vehicle->getRouteDefinition()->getFirstRoute();
+            bulkVehs[stub->getFirst()].push_back(vehicle);
+            ROVehicle* const first = bulkVehs[stub->getFirst()].front();
+            if (first->getMaxSpeed() != vehicle->getMaxSpeed()) {
+                WRITE_WARNING("Bulking different maximum speeds ('" + first->getID() + "' and '" + vehicle->getID() + "') may lead to suboptimal routes.");
+            }
+            if (first->getVClass() != vehicle->getVClass()) {
+                WRITE_WARNING("Bulking different vehicle classes ('" + first->getID() + "' and '" + vehicle->getID() + "') may lead to invalid routes.");
+            }
+        }
+    }
+    int workerIndex = 0;
+    for (std::map<const ROEdge*, std::vector<ROVehicle*> >::const_iterator i = bulkVehs.begin(); i != bulkVehs.end(); ++i) {
+#ifdef HAVE_FOX
+        if (myThreadPool.size() > 0) {
+            ROVehicle* const first = i->second.front();
+            myThreadPool.add(new RoutingTask(first, removeLoops, myErrorHandler), workerIndex);
+            myThreadPool.add(new BulkmodeTask(true), workerIndex);
+            for (std::vector<ROVehicle*>::const_iterator j = i->second.begin() + 1; j != i->second.end(); ++j) {
+                myThreadPool.add(new RoutingTask(*j, removeLoops, myErrorHandler), workerIndex);
+            }
+            myThreadPool.add(new BulkmodeTask(false), workerIndex);
+            workerIndex++;
+            if (workerIndex == (int)myThreadPool.size()) {
+                workerIndex = 0;
+            }
+            continue;
+        }
+#endif
+        for (std::vector<ROVehicle*>::const_iterator j = i->second.begin(); j != i->second.end(); ++j) {
+            (*j)->setRoutingSuccess(computeRoute(router, *j, removeLoops, myErrorHandler));
+            router.setBulkMode(true);
+        }
+        router.setBulkMode(false);
+    }
+}
+
+
 SUMOTime
 RONet::saveAndRemoveRoutesUntil(OptionsCont& options, SUMOAbstractRouter<ROEdge, ROVehicle>& router,
                                 SUMOTime time) {
@@ -431,25 +510,35 @@ RONet::saveAndRemoveRoutesUntil(OptionsCont& options, SUMOAbstractRouter<ROEdge,
 #endif
     if (myVehicles.size() != 0) {
         const std::map<std::string, ROVehicle*>& mmap = myVehicles.getMyMap();
-        for (std::map<std::string, ROVehicle*>::const_iterator i = mmap.begin(); i != mmap.end(); ++i) {
-            if (i->second->getDepart() >= time) {
-                // we cannot go through a sorted list here, because the priority queue in the myVehicles container is not fully sorted
-                continue;
-            }
-            i->second->setRoutingSuccess(false);
+        if (options.getBool("bulk-routing")) {
 #ifdef HAVE_FOX
-            // add thread if necessary
-            const int numThreads = (int)myThreadPool.size();
-            if (numThreads < maxNumThreads && myThreadPool.isFull()) {
-                new WorkerThread(myThreadPool, numThreads == 0 ? &router : router.clone());
-            }
-            // add task
-            if (maxNumThreads > 0) {
-                myThreadPool.add(new RoutingTask(i->second, removeLoops, myErrorHandler));
-                continue;
+            while ((int)myThreadPool.size() < maxNumThreads) {
+                new WorkerThread(myThreadPool, myThreadPool.size() == 0 ? &router : router.clone());
             }
 #endif
-            i->second->setRoutingSuccess(computeRoute(router, i->second, removeLoops, myErrorHandler));
+            createBulkRouteRequests(router, time, removeLoops, mmap);
+        } else {
+            for (std::map<std::string, ROVehicle*>::const_iterator i = mmap.begin(); i != mmap.end(); ++i) {
+                ROVehicle* const vehicle = i->second;
+                if (vehicle->getDepart() >= time) {
+                    // we cannot go through a sorted list here, because the priority queue in the myVehicles container is not fully sorted
+                    continue;
+                }
+#ifdef HAVE_FOX
+                // add task
+                if (maxNumThreads > 0) {
+                    vehicle->setRoutingSuccess(false);
+                    // add thread if necessary
+                    const int numThreads = (int)myThreadPool.size();
+                    if (numThreads < maxNumThreads && myThreadPool.isFull()) {
+                        new WorkerThread(myThreadPool, numThreads == 0 ? &router : router.clone());
+                    }
+                    myThreadPool.add(new RoutingTask(vehicle, removeLoops, myErrorHandler));
+                    continue;
+                }
+#endif
+                vehicle->setRoutingSuccess(computeRoute(router, vehicle, removeLoops, myErrorHandler));
+            }
         }
 #ifdef HAVE_FOX
         myThreadPool.waitAll();
@@ -544,14 +633,14 @@ RONet::getEdgeMap() const {
 
 
 bool
-RONet::hasRestrictions() const {
-    return myHaveRestrictions;
+RONet::hasPermissions() const {
+    return myHavePermissions;
 }
 
 
 void
-RONet::setRestrictionFound() {
-    myHaveRestrictions = true;
+RONet::setPermissionsFound() {
+    myHavePermissions = true;
 }
 
 
