@@ -4,7 +4,7 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    Sept 2002
-/// @version $Id: RORouteDef.cpp 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: RORouteDef.cpp 18756 2015-08-31 19:16:33Z behrisch $
 ///
 // Base class for a vehicle's route definition
 /****************************************************************************/
@@ -105,15 +105,19 @@ void
 RORouteDef::preComputeCurrentRoute(SUMOAbstractRouter<ROEdge, ROVehicle>& router,
                                    SUMOTime begin, const ROVehicle& veh) const {
     myNewRoute = false;
+    const OptionsCont& oc = OptionsCont::getOptions();
     assert(myAlternatives[0]->getEdgeVector().size() > 0);
     MsgHandler* mh = (OptionsCont::getOptions().getBool("ignore-errors") ?
                       MsgHandler::getWarningInstance() : MsgHandler::getErrorInstance());
-    if (myAlternatives[0]->getFirst()->prohibits(&veh)) {
-        /// XXX check for specified arrivalLane / departLane
+    if (myAlternatives[0]->getFirst()->prohibits(&veh) && (!oc.getBool("repair.from")
+            // do not try to reassign starting edge for trip input
+            || myMayBeDisconnected || myAlternatives[0]->getEdgeVector().size() < 2)) {
         mh->inform("Vehicle '" + veh.getID() + "' is not allowed to depart on edge '" +
                    myAlternatives[0]->getFirst()->getID() + "'.");
         return;
-    } else if (myAlternatives[0]->getLast()->prohibits(&veh)) {
+    } else if (myAlternatives[0]->getLast()->prohibits(&veh) && (!oc.getBool("repair.to")
+               // do not try to reassign destination edge for trip input
+               || myMayBeDisconnected || myAlternatives[0]->getEdgeVector().size() < 2)) {
         // this check is not strictly necessary unless myTryRepair is set.
         // However, the error message is more helpful than "no connection found"
         mh->inform("Vehicle '" + veh.getID() + "' is not allowed to arrive on edge '" +
@@ -167,6 +171,23 @@ RORouteDef::repairCurrentRoute(SUMOAbstractRouter<ROEdge, ROVehicle>& router,
         }
     } else {
         // prepare mandatory edges
+        if (oldEdges.front()->prohibits(&veh)) {
+            // option repair.from is in effect
+            for (ConstROEdgeVector::iterator i = oldEdges.begin(); i != oldEdges.end();) {
+                if ((*i)->prohibits(&veh)) {
+                    i = oldEdges.erase(i);
+                } else {
+                    WRITE_MESSAGE("Changing invalid starting edge '"
+                                  + myAlternatives[0]->getEdgeVector().front()->getID()
+                                  + "' to '" + (*i)->getID() + "' for vehicle '" + veh.getID() + "'.");
+                    break;
+                }
+            }
+        }
+        if (oldEdges.size() == 0) {
+            mh->inform("Could not find new starting edge for vehicle '" + veh.getID() + "'.");
+            return;
+        }
         mandatory.push_back(oldEdges.front());
         ConstROEdgeVector stops = veh.getStopEdges();
         for (ConstROEdgeVector::const_iterator i = stops.begin(); i != stops.end(); ++i) {
@@ -174,6 +195,20 @@ RORouteDef::repairCurrentRoute(SUMOAbstractRouter<ROEdge, ROVehicle>& router,
                 mandatory.push_back(*i);
             }
         }
+        if (oldEdges.back()->prohibits(&veh)) {
+            // option repair.to is in effect
+            for (ConstROEdgeVector::reverse_iterator i = oldEdges.rbegin(); i != oldEdges.rend();) {
+                if ((*i)->prohibits(&veh)) {
+                    ++i;
+                    oldEdges.erase(i.base());
+                } else {
+                    WRITE_MESSAGE("Changing invalid destination edge '"
+                                  + myAlternatives[0]->getEdgeVector().back()->getID()
+                                  + "' to '" + (*i)->getID() + "' for vehicle '" + veh.getID() + "'.");
+                    break;
+                }
+            }
+        }
         if (mandatory.size() < 2 || oldEdges.back() != mandatory.back()) {
             mandatory.push_back(oldEdges.back());
         }
@@ -199,6 +234,10 @@ RORouteDef::repairCurrentRoute(SUMOAbstractRouter<ROEdge, ROVehicle>& router,
             if ((*(i - 1))->isConnectedTo(*i, &veh)) {
                 newEdges.push_back(*i);
             } else {
+                if (myAlternatives[0]->getEdgeVector().size() > 2) {
+                    // only inform if the input is (probably) not a trip
+                    WRITE_MESSAGE("Edge '" + (*(i - 1))->getID() + "' not connected to '" + (*i)->getID() + " for vehicle '" + veh.getID() + "'.");
+                }
                 ConstROEdgeVector edges;
                 router.compute(newEdges.back(), *i, &veh, begin, edges);
                 if (edges.size() == 0) {
@@ -247,8 +286,7 @@ RORouteDef::addAlternative(SUMOAbstractRouter<ROEdge, ROVehicle>& router,
     if (myTryRepair) {
         if (myNewRoute) {
             delete myAlternatives[0];
-            myAlternatives.pop_back();
-            myAlternatives.push_back(current);
+            myAlternatives[0] = current;
         }
         const SUMOReal costs = router.recomputeCosts(current->getEdgeVector(), veh, begin);
         if (costs < 0) {
