@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Jakob Erdmann
 /// @date    07.05.2009
-/// @version $Id: TraCIServerAPI_TLS.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: TraCIServerAPI_TLS.cpp 18213 2015-04-16 11:55:42Z behrisch $
 ///
 // APIs for getting/setting traffic light values via TraCI
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2009-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2009-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -36,6 +36,7 @@
 #include "TraCIConstants.h"
 #include <microsim/traffic_lights/MSTLLogicControl.h>
 #include <microsim/MSLane.h>
+#include <microsim/MSEdge.h>
 #include "TraCIServerAPI_TLS.h"
 
 #ifdef CHECK_MEMORY_LEAKS
@@ -56,7 +57,8 @@ TraCIServerAPI_TLS::processGet(TraCIServer& server, tcpip::Storage& inputStorage
     if (variable != ID_LIST && variable != TL_RED_YELLOW_GREEN_STATE && variable != TL_COMPLETE_DEFINITION_RYG
             && variable != TL_CONTROLLED_LANES && variable != TL_CONTROLLED_LINKS
             && variable != TL_CURRENT_PHASE && variable != TL_CURRENT_PROGRAM
-            && variable != TL_NEXT_SWITCH && variable != TL_PHASE_DURATION && variable != ID_COUNT) {
+            && variable != TL_NEXT_SWITCH && variable != TL_PHASE_DURATION && variable != ID_COUNT
+            && variable != VAR_PARAMETER && variable != TL_EXTERNAL_STATE) {
         return server.writeErrorStatusCmd(CMD_GET_TL_VARIABLE, "Get TLS Variable: unsupported variable specified", outputStorage);
     }
     // begin response building
@@ -120,13 +122,13 @@ TraCIServerAPI_TLS::processGet(TraCIServer& server, tcpip::Storage& inputStorage
                     for (unsigned int j = 0; j < phaseNo; ++j) {
                         MSPhaseDefinition phase = logic->getPhase(j);
                         tempContent.writeUnsignedByte(TYPE_INTEGER);
-                        tempContent.writeInt(phase.duration);
+                        tempContent.writeInt((int)phase.duration);
                         ++cnt;
                         tempContent.writeUnsignedByte(TYPE_INTEGER);
-                        tempContent.writeInt(phase.minDuration);
+                        tempContent.writeInt((int)phase.minDuration);
                         ++cnt; // not implemented
                         tempContent.writeUnsignedByte(TYPE_INTEGER);
-                        tempContent.writeInt(phase.maxDuration);
+                        tempContent.writeInt((int)phase.maxDuration);
                         ++cnt; // not implemented
                         const std::string& state = phase.getState();
                         //unsigned int linkNo = (unsigned int)(vars.getActive()->getLinks().size());
@@ -208,9 +210,75 @@ TraCIServerAPI_TLS::processGet(TraCIServer& server, tcpip::Storage& inputStorage
                 tempMsg.writeUnsignedByte(TYPE_INTEGER);
                 tempMsg.writeInt((int) vars.getActive()->getNextSwitchTime());
                 break;
+            case VAR_PARAMETER: {
+                std::string paramName = "";
+                if (!server.readTypeCheckingString(inputStorage, paramName)) {
+                    return server.writeErrorStatusCmd(CMD_GET_TL_VARIABLE, "Retrieval of a parameter requires its name.", outputStorage);
+                }
+                tempMsg.writeUnsignedByte(TYPE_STRING);
+                tempMsg.writeString(vars.getActive()->getParameter(paramName, ""));
+            }
+            break;
             case TL_CONTROLLED_JUNCTIONS: {
             }
             break;
+            case TL_EXTERNAL_STATE: {
+                MSTrafficLightLogic* tls = vars.getActive();
+                const std::string& state = tls->getCurrentPhaseDef().getState();
+                const std::map<std::string, std::string>& params = tls->getMap();
+                unsigned int num = 0;
+                for (std::map<std::string, std::string>::const_iterator i = params.begin(); i != params.end(); ++i) {
+                    if ("connection:" == (*i).first.substr(0, 11)) {
+                        ++num;
+                    }
+                }
+
+                tempMsg.writeUnsignedByte(TYPE_COMPOUND);
+                tempMsg.writeUnsignedByte(TYPE_INTEGER);
+                tempMsg.writeInt(num * 2);
+                for (std::map<std::string, std::string>::const_iterator i = params.begin(); i != params.end(); ++i) {
+                    if ("connection:" != (*i).first.substr(0, 11)) {
+                        continue;
+                    }
+                    tempMsg.writeUnsignedByte(TYPE_STRING);
+                    tempMsg.writeString((*i).second); // foreign id
+                    std::string connection = (*i).first.substr(11);
+                    std::string from, to;
+                    size_t b = connection.find("->");
+                    if (b == std::string::npos) {
+                        from = connection;
+                    } else {
+                        from = connection.substr(0, b);
+                        to = connection.substr(b + 2);
+                    }
+                    bool denotesEdge = from.find("_") == std::string::npos;
+                    MSLane* fromLane = 0;
+                    const MSTrafficLightLogic::LaneVectorVector& lanes = tls->getLaneVectors();
+                    MSTrafficLightLogic::LaneVectorVector::const_iterator j = lanes.begin();
+                    for (; j != lanes.end() && fromLane == 0;) {
+                        for (MSTrafficLightLogic::LaneVector::const_iterator k = (*j).begin(); k != (*j).end() && fromLane == 0;) {
+                            if (denotesEdge && (*k)->getEdge().getID() == from) {
+                                fromLane = *k;
+                            } else if (!denotesEdge && (*k)->getID() == from) {
+                                fromLane = *k;
+                            }
+                            if (fromLane == 0) {
+                                ++k;
+                            }
+                        }
+                        if (fromLane == 0) {
+                            ++j;
+                        }
+                    }
+                    if (fromLane == 0) {
+                        return server.writeErrorStatusCmd(CMD_GET_TL_VARIABLE, "Could not find edge or lane '" + from + "' in traffic light '" + id + "'.", outputStorage);
+                    }
+                    int pos = (int)std::distance(lanes.begin(), j);
+                    tempMsg.writeUnsignedByte(TYPE_UBYTE);
+                    tempMsg.writeUnsignedByte(state[pos]); // state
+                }
+            }
+            break;
             default:
                 break;
         }
@@ -227,8 +295,9 @@ TraCIServerAPI_TLS::processSet(TraCIServer& server, tcpip::Storage& inputStorage
     std::string warning = ""; // additional description for response
     // variable
     int variable = inputStorage.readUnsignedByte();
-    if (variable != TL_PHASE_INDEX && variable != TL_PROGRAM
-            && variable != TL_PHASE_DURATION && variable != TL_RED_YELLOW_GREEN_STATE && variable != TL_COMPLETE_PROGRAM_RYG) {
+    if (variable != TL_PHASE_INDEX && variable != TL_PROGRAM && variable != TL_PHASE_DURATION
+            && variable != TL_RED_YELLOW_GREEN_STATE && variable != TL_COMPLETE_PROGRAM_RYG
+            && variable != VAR_PARAMETER) {
         return server.writeErrorStatusCmd(CMD_SET_TL_VARIABLE, "Change TLS State: unsupported variable specified", outputStorage);
     }
     std::string id = inputStorage.readString();
@@ -247,7 +316,7 @@ TraCIServerAPI_TLS::processSet(TraCIServer& server, tcpip::Storage& inputStorage
             if (index < 0 || vars.getActive()->getPhaseNumber() <= (unsigned int)index) {
                 return server.writeErrorStatusCmd(CMD_SET_TL_VARIABLE, "The phase index is not in the allowed range.", outputStorage);
             }
-            int duration = vars.getActive()->getPhase(index).duration;
+            const SUMOTime duration = vars.getActive()->getPhase(index).duration;
             vars.getActive()->changeStepAndDuration(tlsControl, cTime, index, duration);
         }
         break;
@@ -336,6 +405,23 @@ TraCIServerAPI_TLS::processSet(TraCIServer& server, tcpip::Storage& inputStorage
             }
         }
         break;
+        case VAR_PARAMETER: {
+            if (inputStorage.readUnsignedByte() != TYPE_COMPOUND) {
+                return server.writeErrorStatusCmd(CMD_SET_TL_VARIABLE, "A compound object is needed for setting a parameter.", outputStorage);
+            }
+            //readt itemNo
+            inputStorage.readInt();
+            std::string name;
+            if (!server.readTypeCheckingString(inputStorage, name)) {
+                return server.writeErrorStatusCmd(CMD_SET_TL_VARIABLE, "The name of the parameter must be given as a string.", outputStorage);
+            }
+            std::string value;
+            if (!server.readTypeCheckingString(inputStorage, value)) {
+                return server.writeErrorStatusCmd(CMD_SET_TL_VARIABLE, "The value of the parameter must be given as a string.", outputStorage);
+            }
+            vars.getActive()->addParameter(name, value);
+        }
+        break;
         default:
             break;
     }
