@@ -5,12 +5,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: GeomHelper.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GeomHelper.h 18095 2015-03-17 09:39:00Z behrisch $
 ///
 // Some static methods performing geometrical operations
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -53,7 +53,11 @@
  * @brief Some static methods performing geometrical operations
  */
 class GeomHelper {
+
 public:
+    /// @brief a value to signify offsets outside the range of [0, Line.length()]
+    static const SUMOReal INVALID_OFFSET;
+
     /** @brief return whether given lines intersect
      * @param[in] p11 The begin position of the first line
      * @param[in] p12 The end position of the first line
@@ -118,7 +122,7 @@ public:
      * Return the distance from point to line as well as the intersection point.
      * If intersection does not lie within the line segment, the  start or end point of the segment is returned
      */
-    static SUMOReal closestDistancePointLine(const Position& point,
+    static SUMOReal closestDistancePointLine2D(const Position& point,
             const Position& lineStart, const Position& lineEnd,
             Position& outIntersection);
 
