@@ -4,7 +4,7 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Sept 2002
-/// @version $Id: PositionVector.h 18096 2015-03-17 09:50:59Z behrisch $
+/// @version $Id: PositionVector.h 18639 2015-08-17 13:59:19Z namdre $
 ///
 // A list of positions
 /****************************************************************************/
@@ -210,6 +210,9 @@ public:
 
     void add(SUMOReal xoff, SUMOReal yoff, SUMOReal zoff);
 
+    /// @brief mirror coordinates along the x-axis
+    void mirrorX();
+
     void reshiftRotate(SUMOReal xoff, SUMOReal yoff, SUMOReal rot);
 
     PositionVector convexHull() const;
@@ -317,6 +320,9 @@ public:
 
     void removeColinearPoints();
 
+    /// @brief return whether two positions differ in z-coordinate
+    bool hasElevation() const;
+
 };
 
 
