@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    June 2006
-/// @version $Id: GUIPointOfInterest.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIPointOfInterest.cpp 18421 2015-05-20 14:57:37Z namdre $
 ///
 // The GUI-version of a point of interest
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -121,8 +121,12 @@ GUIPointOfInterest::drawGL(const GUIVisualizationSettings& s) const {
         GLHelper::drawFilledCircle((SUMOReal) 1.3 * exaggeration, 16);
     }
     glPopMatrix();
-    drawName(Position(x() + 1.32 * exaggeration, y() + 1.32 * exaggeration),
-             s.scale, s.poiName);
+    const Position namePos = Position(x() + 1.32 * exaggeration, y() + 1.32 * exaggeration);
+    drawName(namePos, s.scale, s.poiName);
+    if (s.poiType.show) {
+        GLHelper::drawText(myType, namePos + Position(0, -0.6 * s.poiType.size / s.scale),
+                           GLO_MAX, s.poiType.size / s.scale, s.poiType.color);
+    }
     glPopName();
 }
 /****************************************************************************/
