@@ -5,12 +5,12 @@
 /// @author  Michael Behrisch
 /// @author  Laura Bieker
 /// @date    June 2006
-/// @version $Id: GUIPolygon.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: GUIPolygon.cpp 18722 2015-08-25 15:41:30Z namdre $
 ///
 // The GUI-version of a polygon
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -44,6 +44,7 @@
 #include <foreign/nvwa/debug_new.h>
 #endif // CHECK_MEMORY_LEAKS
 
+//#define GUIPolygon_DEBUG_DRAW_VERTICES
 
 // ===========================================================================
 // method definitions
@@ -137,6 +138,9 @@ GLfloat yPlane[] = {0.0, INV_POLY_TEX_DIM, 0.0, 0.0};
 
 void
 GUIPolygon::drawGL(const GUIVisualizationSettings& s) const {
+    if (s.polySize.getExaggeration(s) == 0) {
+        return;
+    }
     Boundary boundary = myShape.getBoxBoundary();
     if (s.scale * MAX2(boundary.getWidth(), boundary.getHeight()) < s.polySize.minSize) {
         return;
@@ -202,8 +206,16 @@ GUIPolygon::drawGL(const GUIVisualizationSettings& s) const {
         glDisable(GL_TEXTURE_GEN_S);
         glDisable(GL_TEXTURE_GEN_T);
     }
+#ifdef GUIPolygon_DEBUG_DRAW_VERTICES
+    GLHelper::debugVertices(myShape, 80 / s.scale);
+#endif
     glPopMatrix();
-    drawName(myShape.getPolygonCenter(), s.scale, s.polyName);
+    const Position namePos = myShape.getPolygonCenter();
+    drawName(namePos, s.scale, s.polyName);
+    if (s.polyType.show) {
+        GLHelper::drawText(myType, namePos + Position(0, -0.6 * s.polyType.size / s.scale),
+                           GLO_MAX, s.polyType.size / s.scale, s.polyType.color);
+    }
     glPopName();
 }
 
