@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Wed, 21. Dec 2005
-/// @version $Id: GUIDialog_ViewSettings.h 17262 2014-11-04 20:41:44Z behrisch $
+/// @version $Id: GUIDialog_ViewSettings.h 18421 2015-05-20 14:57:37Z namdre $
 ///
 // The dialog to change the view (gui) settings.
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -221,7 +221,7 @@ private:
     /** @brief Writes the currently used decals into a file
      * @param[in] file The name of the file to write the decals into
      */
-    void saveDecals(const std::string& file) const;
+    void saveDecals(OutputDevice& dev) const;
 
 
     /** @brief Loads decals from a file
@@ -293,6 +293,14 @@ private:
     std::vector<FXButton*> myPersonButtons;
     FXCheckButton* myPersonColorInterpolation;
 
+    FXComboBox* myContainerColorMode, *myContainerShapeDetail;
+    FXVerticalFrame* myContainerColorSettingFrame;
+    std::vector<FXColorWell*> myContainerColors;
+    std::vector<FXRealSpinDial*> myContainerThresholds;
+    std::vector<FXButton*> myContainerButtons;
+    FXCheckButton* myContainerColorInterpolation;
+    FXRealSpinDial* myContainerMinSizeDialer, *myContainerUpscaleDialer;
+
     FXComboBox* myJunctionColorMode;
     FXVerticalFrame* myJunctionColorSettingFrame;
     std::vector<FXColorWell*> myJunctionColors;
@@ -309,10 +317,14 @@ private:
 
     NamePanel* myEdgeNamePanel, *myInternalEdgeNamePanel, *myCwaEdgeNamePanel, *myStreetNamePanel,
                *myJunctionNamePanel, *myInternalJunctionNamePanel,
-               *myVehicleNamePanel, *myPersonNamePanel,
-               *myAddNamePanel, *myPOINamePanel, *myPolyNamePanel;
+               *myVehicleNamePanel, *myPersonNamePanel, *myContainerNamePanel,
+               *myAddNamePanel, *myPOINamePanel, *myPolyNamePanel, *myPOITypePanel, *myPolyTypePanel;
 
-    SizePanel* myVehicleSizePanel, *myPersonSizePanel, *myPOISizePanel, *myPolySizePanel, *myAddSizePanel;
+    SizePanel* myVehicleSizePanel, *myPersonSizePanel, *myContainerSizePanel, *myPOISizePanel, *myPolySizePanel, *myAddSizePanel;
+
+
+    // load/save-menu
+    FXCheckButton* mySaveViewPort, *mySaveDelay, *mySaveDecals;
     /// @}
 
 
