@@ -1,10 +1,10 @@
-/****************************************************************************/
+﻿/****************************************************************************/
 /// @file    SUMOVehicle.h
 /// @author  Michael Behrisch
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @date    Tue, 17 Feb 2009
-/// @version $Id: SUMOVehicle.h 18095 2015-03-17 09:39:00Z behrisch $
+/// @version $Id: SUMOVehicle.h 18730 2015-08-26 09:24:04Z namdre $
 ///
 // Abstract base class for vehicle representations
 /****************************************************************************/
@@ -49,7 +49,7 @@ class MSEdge;
 class MSLane;
 class MSDevice;
 class MSPerson;
-class MSContainer;
+class MSTransportable;
 class SUMOSAXAttributes;
 
 typedef std::vector<const MSEdge*> ConstMSEdgeVector;
@@ -101,6 +101,11 @@ public:
      */
     virtual SUMOReal getSpeed() const = 0;
 
+    /** @brief Returns the lane the vehicle is on
+    * @return The vehicle's current lane
+    */
+    virtual MSLane* getLane() const = 0;
+
     /** @brief Returns the vehicle's type
      * @return The vehicle's type
      */
@@ -180,6 +185,11 @@ public:
      */
     virtual bool isOnRoad() const = 0;
 
+    /** @brief Returns the information whether the vehicle is parked
+     * @return Whether the vehicle is parked
+     */
+    virtual bool isParking() const = 0;
+
     /** @brief Returns this vehicle's real departure time
      * @return This vehicle's real departure time
      */
@@ -191,6 +201,10 @@ public:
      */
     virtual SUMOReal getArrivalPos() const = 0;
 
+    /** @brief Sets this vehicle's desired arrivalPos for its current route
+     */
+    virtual void setArrivalPos(SUMOReal arrivalPos) = 0;
+
     /** @brief Returns whether this vehicle has departed
      */
     virtual bool hasDeparted() const = 0;
@@ -215,7 +229,7 @@ public:
      *
      * @param[in] person The person to add
      */
-    virtual void addPerson(MSPerson* person) = 0;
+    virtual void addPerson(MSTransportable* person) = 0;
 
     /** @brief Adds a container to this vehicle
      *
@@ -223,7 +237,7 @@ public:
      *
      * @param[in] container The container to add
      */
-    virtual void addContainer(MSContainer* container) = 0;
+    virtual void addContainer(MSTransportable* container) = 0;
 
     /** @brief Adds a stop
      *
