@@ -4,12 +4,12 @@
 /// @author  Jakob Erdmann
 /// @author  Michael Behrisch
 /// @date    Tue, 31.03.2009
-/// @version $Id: SUMOVehicleParameter.cpp 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: SUMOVehicleParameter.cpp 18186 2015-04-01 15:30:42Z behrisch $
 ///
 // Structure representing possible vehicle parameter
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -55,7 +55,7 @@ SUMOVehicleParameter::SUMOVehicleParameter()
       arrivalPos(0), arrivalPosProcedure(ARRIVAL_POS_DEFAULT),
       arrivalSpeed(-1), arrivalSpeedProcedure(ARRIVAL_SPEED_DEFAULT),
       repetitionNumber(-1), repetitionsDone(-1), repetitionOffset(-1), repetitionProbability(-1), repetitionEnd(-1),
-      line(), fromTaz(), toTaz(), personCapacity(0), personNumber(0), setParameter(0) {
+      line(), fromTaz(), toTaz(), personNumber(0), containerNumber(0), setParameter(0) {
 }
 
 
@@ -73,6 +73,8 @@ SUMOVehicleParameter::write(OutputDevice& dev, const OptionsCont& oc) const {
     }
     if (departProcedure == DEPART_TRIGGERED) {
         dev.writeAttr(SUMO_ATTR_DEPART, "triggered");
+    } else if (departProcedure == DEPART_CONTAINER_TRIGGERED) {
+        dev.writeAttr(SUMO_ATTR_DEPART, "containerTriggered");
     } else {
         dev.writeAttr(SUMO_ATTR_DEPART, time2string(depart));
     }
@@ -231,15 +233,18 @@ SUMOVehicleParameter::write(OutputDevice& dev, const OptionsCont& oc) const {
     if (wasSet(VEHPARS_LINE_SET)) {
         dev.writeAttr(SUMO_ATTR_LINE, line);
     }
-    if (wasSet(VEHPARS_TAZ_SET)) {
-        dev.writeAttr(SUMO_ATTR_FROM_TAZ, fromTaz).writeAttr(SUMO_ATTR_TO_TAZ, toTaz);
+    if (wasSet(VEHPARS_FROM_TAZ_SET)) {
+        dev.writeAttr(SUMO_ATTR_FROM_TAZ, fromTaz);
     }
-    if (wasSet(VEHPARS_PERSON_CAPACITY_SET)) {
-        dev.writeAttr(SUMO_ATTR_PERSON_CAPACITY, personCapacity);
+    if (wasSet(VEHPARS_TO_TAZ_SET)) {
+        dev.writeAttr(SUMO_ATTR_TO_TAZ, toTaz);
     }
     if (wasSet(VEHPARS_PERSON_NUMBER_SET)) {
         dev.writeAttr(SUMO_ATTR_PERSON_NUMBER, personNumber);
     }
+    if (wasSet(VEHPARS_CONTAINER_NUMBER_SET)) {
+        dev.writeAttr(SUMO_ATTR_CONTAINER_NUMBER, containerNumber);
+    }
 }
 
 
@@ -249,7 +254,11 @@ SUMOVehicleParameter::writeStops(OutputDevice& dev) const {
         dev.openTag(SUMO_TAG_STOP);
         if (stop->busstop != "") {
             dev.writeAttr(SUMO_ATTR_BUS_STOP, stop->busstop);
-        } else {
+        }
+        if (stop->containerstop != "") {
+            dev.writeAttr(SUMO_ATTR_CONTAINER_STOP, stop->containerstop);
+        }
+        if (stop->busstop == "" && stop->containerstop == "") {
             dev.writeAttr(SUMO_ATTR_LANE, stop->lane);
             if ((stop->setParameter & STOP_START_SET) != 0) {
                 dev.writeAttr(SUMO_ATTR_STARTPOS, stop->startPos);
@@ -267,6 +276,9 @@ SUMOVehicleParameter::writeStops(OutputDevice& dev) const {
         if ((stop->setParameter & STOP_TRIGGER_SET) != 0) {
             dev.writeAttr(SUMO_ATTR_TRIGGERED, stop->triggered);
         }
+        if ((stop->setParameter & STOP_CONTAINER_TRIGGER_SET) != 0) {
+            dev.writeAttr(SUMO_ATTR_CONTAINER_TRIGGERED, stop->containerTriggered);
+        }
         if ((stop->setParameter & STOP_PARKING_SET) != 0) {
             dev.writeAttr(SUMO_ATTR_PARKING, stop->parking);
         }
@@ -277,6 +289,9 @@ SUMOVehicleParameter::writeStops(OutputDevice& dev) const {
         if ((stop->setParameter & STOP_EXPECTED_SET) != 0) {
             dev.writeAttr(SUMO_ATTR_EXPECTED, stop->awaitedPersons);
         }
+        if ((stop->setParameter & STOP_EXPECTED_CONTAINERS_SET) != 0) {
+            dev.writeAttr(SUMO_ATTR_EXPECTED_CONTAINERS, stop->awaitedContainers);
+        }
         dev.closeTag();
     }
 }
@@ -287,6 +302,8 @@ SUMOVehicleParameter::parseDepart(const std::string& val, const std::string& ele
                                   SUMOTime& depart, DepartDefinition& dd, std::string& error) {
     if (val == "triggered") {
         dd = DEPART_TRIGGERED;
+    } else if (val == "containerTriggered") {
+        dd = DEPART_CONTAINER_TRIGGERED;
     } else if (val == "now") {
         dd = DEPART_NOW;
     } else {
@@ -298,7 +315,7 @@ SUMOVehicleParameter::parseDepart(const std::string& val, const std::string& ele
                 return false;
             }
         } catch (...) {
-            error = "Invalid departure time for " + element + " '" + id + "';\n must be one of (\"triggered\", \"now\", or a float >= 0)";
+            error = "Invalid departure time for " + element + " '" + id + "';\n must be one of (\"triggered\", \"containerTriggered\", \"now\", or a float >= 0)";
             return false;
         }
     }
