@@ -5,12 +5,12 @@
 /// @author  Axel Wegener
 /// @author  Michael Behrisch
 /// @date    2006-01-24
-/// @version $Id: SUMOVehicleParameter.h 17235 2014-11-03 10:53:02Z behrisch $
+/// @version $Id: SUMOVehicleParameter.h 18686 2015-08-21 14:52:03Z namdre $
 ///
 // Structure representing possible vehicle parameter
 /****************************************************************************/
 // SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2014 DLR (http://www.dlr.de/) and contributors
+// Copyright (C) 2001-2015 DLR (http://www.dlr.de/) and contributors
 /****************************************************************************/
 //
 //   This file is part of SUMO.
@@ -62,10 +62,12 @@ const int VEHPARS_ARRIVALLANE_SET = 2 << 7;
 const int VEHPARS_ARRIVALPOS_SET = 2 << 8;
 const int VEHPARS_ARRIVALSPEED_SET = 2 << 9;
 const int VEHPARS_LINE_SET = 2 << 10;
-const int VEHPARS_TAZ_SET = 2 << 11;
-const int VEHPARS_FORCE_REROUTE = 2 << 12;
-const int VEHPARS_PERSON_CAPACITY_SET = 2 << 13;
-const int VEHPARS_PERSON_NUMBER_SET = 2 << 14;
+const int VEHPARS_FROM_TAZ_SET = 2 << 11;
+const int VEHPARS_TO_TAZ_SET = 2 << 12;
+const int VEHPARS_FORCE_REROUTE = 2 << 13;
+const int VEHPARS_PERSON_CAPACITY_SET = 2 << 14;
+const int VEHPARS_PERSON_NUMBER_SET = 2 << 15;
+const int VEHPARS_CONTAINER_NUMBER_SET = 2 << 15;
 
 const int STOP_INDEX_END = -1;
 const int STOP_INDEX_FIT = -2;
@@ -75,6 +77,8 @@ const int STOP_START_SET = 2;
 const int STOP_TRIGGER_SET = 2 << 1;
 const int STOP_PARKING_SET = 2 << 2;
 const int STOP_EXPECTED_SET = 2 << 3;
+const int STOP_CONTAINER_TRIGGER_SET = 2 << 4;
+const int STOP_EXPECTED_CONTAINERS_SET = 2 << 5;
 
 
 // ===========================================================================
@@ -89,6 +93,8 @@ enum DepartDefinition {
     DEPART_GIVEN,
     /// @brief The departure is person triggered
     DEPART_TRIGGERED,
+    /// @brief The departure is container triggered
+    DEPART_CONTAINER_TRIGGERED,
     /// @brief The vehicle is discarded if emission fails (not fully implemented yet)
     DEPART_NOW,
     /// @brief Tag for the last element in the enum for safe int casting
@@ -464,6 +470,10 @@ public:
         std::string lane;
         /// @brief (Optional) bus stop if one is assigned to the stop
         std::string busstop;
+        /// @brief (Optional) container stop if one is assigned to the stop
+        std::string containerstop;
+        /// @brief (Optional) charging station if one is assigned to the stop
+        std::string chrgStn;
         /// @brief The stopping position start
         SUMOReal startPos;
         /// @brief The stopping position end
@@ -474,10 +484,14 @@ public:
         SUMOTime until;
         /// @brief whether an arriving person lets the vehicle continue
         bool triggered;
+        /// @brief whether an arriving container lets the vehicle continue
+        bool containerTriggered;
         /// @brief whether the vehicle is removed from the net while stopping
         bool parking;
         /// @brief IDs of persons the vehicle has to wait for until departing
         std::set<std::string> awaitedPersons;
+        /// @brief IDs of containers the vehicle has to wait for until departing
+        std::set<std::string> awaitedContainers;
         /// @brief at which position in the stops list
         int index;
         /// @brief Information for the output which parameter were set
@@ -487,12 +501,12 @@ public:
     /// @brief List of the stops the vehicle will make
     std::vector<Stop> stops;
 
-    /// @brief The vehicle's capacity (persons)
-    unsigned int personCapacity;
-
-    /// @brief The number of persons in the vehicle
+    /// @brief The static number of persons in the vehicle when it departs (not including boarding persons)
     unsigned int personNumber;
 
+    /// @brief The static number of containers in the vehicle when it departs
+    unsigned int containerNumber;
+
     /// @brief Information for the router which parameter were set
     mutable int setParameter;
 
