@@ -7,13 +7,13 @@
 @author  Jakob Erdmann
 @author  Yun-Pang Floetteroed
 @date    2008-02-13
-@version $Id: duaIterate.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: duaIterate.py 18378 2015-05-15 07:43:59Z behrisch $
 
 Run duarouter and sumo alternating to perform a dynamic user assignment.
 Based on the Perl script dua_iterate.pl.
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2008-2014 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2008-2015 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -22,7 +22,12 @@ the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 """
 from __future__ import print_function
-import os, sys, subprocess, types, shutil, glob
+import os
+import sys
+import subprocess
+import types
+import shutil
+import glob
 from datetime import datetime
 from optparse import OptionParser
 from costMemory import CostMemory
@@ -52,34 +57,39 @@ def addGenericOptions(optParser):
                          default=False, help="Whether mesosim shall be used")
     optParser.add_option("-p", "--path", help="Path to binaries")
     optParser.add_option("-y", "--absrand", action="store_true",
-                         default= False, help="use current time to generate random number")
+                         default=False, help="use current time to generate random number")
     optParser.add_option("-I", "--nointernal-link", action="store_true", dest="internallink",
-                         default = False, help="not to simulate internal link: true or false")
+                         default=False, help="not to simulate internal link: true or false")
     optParser.add_option("-L", "--lanechange-allowed", dest="lanechangeallowed", action="store_true",
-                         default = False, help="lane change allowed to swap")
+                         default=False, help="lane change allowed to swap")
     optParser.add_option("-j", "--meso-junctioncontrol", dest="mesojunctioncontrol", action="store_true",
-                         default = False, help="Enable mesoscopic traffic light and priority junciton handling")
+                         default=False, help="Enable mesoscopic traffic light and priority junciton handling")
     optParser.add_option("-q", "--meso-multiqueue", dest="mesomultiqueue", action="store_true",
-                         default = False, help="Enable multiple queues at edge ends")
-    optParser.add_option("--meso-recheck", dest="mesorecheck", type="int", default=0, 
+                         default=False, help="Enable multiple queues at edge ends")
+    optParser.add_option("--meso-recheck", dest="mesorecheck", type="int", default=0,
                          help="Delay before checking whether a jam is gone. (higher values can lead to a big speed increase)")
     optParser.add_option("-Q", "--eco-measure", dest="ecomeasure", type="choice",
-                         choices=('CO', 'CO2', 'PMx', 'HC', 'NOx', 'fuel', 'noise'),
+                         choices=(
+                             'CO', 'CO2', 'PMx', 'HC', 'NOx', 'fuel', 'noise'),
                          help="define the applied eco measure, e.g. fuel, CO2, noise")
     optParser.add_option("--eager-insert", action="store_true",
                          default=False, help="eager insertion tests (may slow down the sim considerably)")
-    optParser.add_option("--time-to-teleport", dest="timetoteleport", type="int", default=300,
-                         help="Delay before blocked vehicles are teleported where -1 means no teleporting")
-    optParser.add_option("--time-to-teleport.highways", dest="timetoteleport_highways", type="int", default=0,
+    optParser.add_option("--time-to-teleport", dest="timetoteleport", type="float", default=300,
+                         help="Delay before blocked vehicles are teleported (negative value disables teleporting)")
+    optParser.add_option("--time-to-teleport.highways", dest="timetoteleport_highways", type="float", default=0,
                          help="Delay before blocked vehicles are teleported on wrong highway lanes")
     optParser.add_option("--cost-modifier", dest="costmodifier", type="choice",
-                         choices=('grohnde', 'isar', 'None'), 
+                         choices=('grohnde', 'isar', 'None'),
                          default='None', help="Whether to modify link travel costs of the given routes")
     optParser.add_option("-7", "--zip", action="store_true",
                          default=False, help="zip old iterations using 7zip")
 
+
 def initOptions():
-    optParser = OptionParser()
+    optParser = OptionParser(usage="""usage: %prog [options] [sumo--options]
+Any options of the form sumo--long-option-name will be passed to sumo. 
+These must be given after all the other options of %prog
+example: sumo--step-length 0.5 will add the option --step-length 0.5 to sumo.""")
     addGenericOptions(optParser)
 
     optParser.add_option("-C", "--continue-on-unbuild", action="store_true", dest="continueOnUnbuild",
@@ -118,16 +128,19 @@ def initOptions():
                          type="int", default=10, help="Number of iterations to use for convergence calculation [default: %default]")
     optParser.add_option("--max-convergence-deviation", dest="convDev",
                          type="float", help="Maximum relative standard deviation in travel times [default: %default]")
-    optParser.add_option("-D", "--districts", help="use districts as sources and targets", metavar="FILE")
+    optParser.add_option(
+        "-D", "--districts", help="use districts as sources and targets", metavar="FILE")
     optParser.add_option("-x", "--vehroute-file",  dest="routefile", type="choice",
-                         choices=('None', 'routesonly', 'detailed'), 
-                         default = 'None', help="choose the format of the route file")
+                         choices=('None', 'routesonly', 'detailed'),
+                         default='None', help="choose the format of the route file")
     optParser.add_option("-z", "--output-lastRoute",  action="store_true", dest="lastroute",
-                         default = False, help="output the last routes")
+                         default=False, help="output the last routes")
     optParser.add_option("-K", "--keep-allroutes", action="store_true", dest="allroutes",
-                         default = False, help="save routes with near zero probability")
-    optParser.add_option("--routing-algorithm", default="dijkstra", help="select the routing algorithm")
-    optParser.add_option("--max-alternatives", default=5, help="prune the number of alternatives to INT")
+                         default=False, help="save routes with near zero probability")
+    optParser.add_option(
+        "--routing-algorithm", default="dijkstra", help="select the routing algorithm")
+    optParser.add_option(
+        "--max-alternatives", default=5, help="prune the number of alternatives to INT")
     optParser.add_option("--skip-first-routing", action="store_true", dest="skipFirstRouting",
                          default=False, help="run simulation with demands before first routing")
     optParser.add_option("--logit", action="store_true", dest="logit",
@@ -135,7 +148,7 @@ def initOptions():
     optParser.add_option("-g", "--logitbeta", type="float", dest="logitbeta",
                          default=0.15, help="use the c-logit model for route choice; logit model when beta = 0")
     optParser.add_option("-i", "--logitgamma", type="float", dest="logitgamma",
-                         default= 1., help="use the c-logit model for route choice")
+                         default=1., help="use the c-logit model for route choice")
     optParser.add_option("-G", "--logittheta", type="float", dest="logittheta",
                          help="parameter to adapt the cost unit")
     optParser.add_option("-J", "--addweights", dest="addweights",
@@ -145,16 +158,18 @@ def initOptions():
     optParser.add_option("-M", "--external-gawron", action="store_true", dest="externalgawron",
                          default=False, help="use the external gawron calculation")
     optParser.add_option("-N", "--calculate-oldprob", action="store_true", dest="caloldprob",
-                         default=False, help="calculate the old route probabilities with the free-flow travel times when using the external gawron calculation")   
+                         default=False, help="calculate the old route probabilities with the free-flow travel times when using the external gawron calculation")
     optParser.add_option("--weight-memory", action="store_true", default=False, dest="weightmemory",
-                         help="smooth edge weights across iterations")    
-    optParser.add_option("--pessimism", default=1, type="float", help="give traffic jams a higher weight")
+                         help="smooth edge weights across iterations")
+    optParser.add_option(
+        "--pessimism", default=1, type="float", help="give traffic jams a higher weight")
     optParser.add_option("--clean-alt", action="store_true", dest="clean_alt",
                          default=False, help="Whether old rou.alt.xml files shall be removed")
     optParser.add_option("--binary", action="store_true",
                          default=False, help="Use binary format for intermediate and resulting route files")
     return optParser
 
+
 def call(command, log):
     command = [str(c) for c in command]
     print("-" * 79, file=log)
@@ -162,8 +177,10 @@ def call(command, log):
     log.flush()
     retCode = subprocess.call(command, stdout=log, stderr=log)
     if retCode != 0:
-        print("Execution of %s failed. Look into %s for details." % (command, log.name), file=sys.stderr)
-        sys.exit(retCode) 
+        print("Execution of %s failed. Look into %s for details." %
+              (command, log.name), file=sys.stderr)
+        sys.exit(retCode)
+
 
 def writeRouteConf(step, options, file, output, routesInfo, initial_type):
     filename = os.path.basename(file)
@@ -178,13 +195,15 @@ def writeRouteConf(step, options, file, output, routesInfo, initial_type):
         <net-file value="%s"/>""" % options.net, file=fd)
     if options.districts:
         print('        <taz-files value="%s"/>' % options.districts, file=fd)
-    if step==0:
+    if step == 0:
         print('        <%s-files value="%s"/>' % (initial_type, file), file=fd)
     else:
         print('        <alternative-files value="%s"/>' % file, file=fd)
-        print('        <weights value="%s"/>' % get_weightfilename(options, step-1, "dump"), file=fd)
+        print('        <weights value="%s"/>' %
+              get_weightfilename(options, step - 1, "dump"), file=fd)
     if options.ecomeasure:
-        print('        <weight-attribute value="%s"/>' % options.ecomeasure, file=fd)
+        print('        <weight-attribute value="%s"/>' %
+              options.ecomeasure, file=fd)
     print("""    </input>
     <output>
         <output-file value="%s"/>
@@ -202,22 +221,25 @@ def writeRouteConf(step, options, file, output, routesInfo, initial_type):
         <logit value="%s"/>
         <logit.beta value="%s"/>
         <logit.gamma value="%s"/>""" % (
-                options.continueOnUnbuild, 
-                bool(options.districts), 
-                options.gBeta, 
-                options.gA, 
-                options.allroutes, 
-                options.routing_algorithm, 
-                ("" if 'CH' not in options.routing_algorithm else '\n<weight-period value="%s"/>\n' % options.aggregation),
-                options.max_alternatives, 
-                options.logit, 
-                options.logitbeta, 
-                options.logitgamma), file=fd)
+        options.continueOnUnbuild,
+        bool(options.districts),
+        options.gBeta,
+        options.gA,
+        options.allroutes,
+        options.routing_algorithm,
+        ("" if 'CH' not in options.routing_algorithm else '\n<weight-period value="%s"/>\n' %
+         options.aggregation),
+        options.max_alternatives,
+        options.logit,
+        options.logitbeta,
+        options.logitgamma), file=fd)
     if options.logittheta:
-        print('        <logit.theta value="%s"/>' % options.logittheta, file=fd)
+        print('        <logit.theta value="%s"/>' %
+              options.logittheta, file=fd)
     print('    </processing>', file=fd)
-                
-    print('    <random_number><random value="%s"/></random_number>' % options.absrand, file=fd)
+
+    print('    <random_number><random value="%s"/></random_number>' %
+          options.absrand, file=fd)
     print('    <time><begin value="%s"/>' % options.begin, end=' ', file=fd)
     if options.end:
         print('<end value="%s"/>' % options.end, end=' ', file=fd)
@@ -231,23 +253,27 @@ def writeRouteConf(step, options, file, output, routesInfo, initial_type):
     fd.close()
     return cfgname
 
+
 def get_scale(options, step):
     # compute scaling factor for simulation
-    # using incValue = 1 (default) and incBase = 10 would produce 
+    # using incValue = 1 (default) and incBase = 10 would produce
     # iterations with increasing scale 0.1, 0.2, ... 0.9, 1, 1, 1, ...
     if options.incBase > 0:
         return min(options.incStart + options.incValue * float(step + 1) / options.incBase, options.incMax)
     else:
         return options.incMax
 
+
 def get_dumpfilename(options, step, prefix):
     # the file to which edge costs (traveltimes) are written
-        return "%s_%03i_%s.xml" % (prefix, step, options.aggregation)
+    return "%s_%03i_%s.xml" % (prefix, step, options.aggregation)
+
 
 def get_weightfilename(options, step, prefix):
     # the file from which edge costs are loaded
     # this defaults to the dumpfile writen by the simulation but may be
-    # different if one of the options --addweights, --memory-weights or --cost-modifier is used
+    # different if one of the options --addweights, --memory-weights or
+    # --cost-modifier is used
     if options.addweights:
         prefix = "%s,%s" % (options.addweights, prefix)
     if options.weightmemory:
@@ -256,27 +282,29 @@ def get_weightfilename(options, step, prefix):
         prefix = options.costmodifier + "_" + prefix
     return get_dumpfilename(options, step, prefix)
 
+
 def writeSUMOConf(sumoBinary, step, options, additional_args, route_files):
     detectorfile = "dua_dump_%03i.add.xml" % step
     comma = (',' if options.additional != "" else '')
     sumoCmd = [sumoBinary,
-        '--save-configuration', "iteration_%03i.sumocfg" % step,
-        '--log', "iteration_%03i.sumo.log" % step,
-        '--net-file', options.net,
-        '--route-files', route_files,
-        '--additional-files', "%s%s%s" % (detectorfile, comma, options.additional),
-        '--no-step-log',
-        '--random', options.absrand,
-        '--begin', options.begin,
-        '--route-steps', options.routeSteps,
-        '--no-internal-links', options.internallink,
-        '--lanechange.allow-swap', options.lanechangeallowed,
-        '--eager-insert', options.eager_insert,
-        '--time-to-teleport', options.timetoteleport,
-        '--time-to-teleport.highways', options.timetoteleport_highways,
-        '--verbose',
-        '--no-warnings', options.noWarnings,
-        ] + additional_args
+               '--save-configuration', "iteration_%03i.sumocfg" % step,
+               '--log', "iteration_%03i.sumo.log" % step,
+               '--net-file', options.net,
+               '--route-files', route_files,
+               '--additional-files', "%s%s%s" % (
+                   detectorfile, comma, options.additional),
+               '--no-step-log',
+               '--random', options.absrand,
+               '--begin', options.begin,
+               '--route-steps', options.routeSteps,
+               '--no-internal-links', options.internallink,
+               '--lanechange.allow-swap', options.lanechangeallowed,
+               '--eager-insert', options.eager_insert,
+               '--time-to-teleport', options.timetoteleport,
+               '--time-to-teleport.highways', options.timetoteleport_highways,
+               '--verbose',
+               '--no-warnings', options.noWarnings,
+               ] + additional_args
 
     if hasattr(options, "noSummary") and not options.noSummary:
         sumoCmd += ['--summary-output', "summary_%03i.xml" % step]
@@ -286,10 +314,12 @@ def writeSUMOConf(sumoBinary, step, options, additional_args, route_files):
             sumoCmd += ['--device.hbefa.probability', '1']
     if hasattr(options, "routefile"):
         if options.routefile == "routesonly":
-            sumoCmd += ['--vehroute-output', "vehroute_%03i.xml" % step]
+            sumoCmd += ['--vehroute-output', "vehroute_%03i.xml" % step,
+                        '--vehroute-output.route-length']
         elif options.routefile == "detailed":
             sumoCmd += ['--vehroute-output', "vehroute_%03i.xml" % step,
-                    '--vehroute-output.exit-times']
+                        '--vehroute-output.route-length',
+                        '--vehroute-output.exit-times']
     if hasattr(options, "lastroute") and options.lastroute:
         sumoCmd += ['--vehroute-output.last-route', options.lastroute]
     if hasattr(options, "timeInc") and options.timeInc:
@@ -300,8 +330,8 @@ def writeSUMOConf(sumoBinary, step, options, additional_args, route_files):
     if hasattr(options, "incBase") and options.incBase > 0:
         sumoCmd += ['--scale', get_scale(options, step)]
     if options.mesosim:
-        sumoCmd += ['--mesosim', 
-                '--meso-recheck', options.mesorecheck]
+        sumoCmd += ['--mesosim',
+                    '--meso-recheck', options.mesorecheck]
         if options.mesomultiqueue:
             sumoCmd += ['--meso-multi-queue']
         if options.mesojunctioncontrol:
@@ -316,16 +346,18 @@ def writeSUMOConf(sumoBinary, step, options, additional_args, route_files):
     with open(detectorfile, 'w') as fd:
         suffix = "_%03i_%s" % (step, options.aggregation)
         print("<a>", file=fd)
-        if options.costmodifier !='None':
+        if options.costmodifier != 'None':
             print('    <edgeData id="dump%s" freq="%s" file="%s" excludeEmpty="defaults" minSamples="1"/>' % (
-                    suffix, options.aggregation, get_dumpfilename(options, step, "dump")), file=fd)
+                suffix, options.aggregation, get_dumpfilename(options, step, "dump")), file=fd)
         else:
             print('    <edgeData id="dump%s" freq="%s" file="%s" excludeEmpty="true" minSamples="1"/>' % (
-                    suffix, options.aggregation, get_dumpfilename(options, step, "dump")), file=fd)
+                suffix, options.aggregation, get_dumpfilename(options, step, "dump")), file=fd)
         if options.ecomeasure:
-            print('    <edgeData id="eco%s" type="hbefa" freq="%s" file="dump%s.xml" excludeEmpty="true" minSamples="1"/>' % (suffix, options.aggregation, suffix), file=fd)
+            print('    <edgeData id="eco%s" type="hbefa" freq="%s" file="dump%s.xml" excludeEmpty="true" minSamples="1"/>' %
+                  (suffix, options.aggregation, suffix), file=fd)
         print("</a>", file=fd)
 
+
 def filterTripinfo(step, attrs):
     attrs.add("id")
     inFile = "tripinfo_%03i.xml" % step
@@ -341,15 +373,17 @@ def filterTripinfo(step, attrs):
                 for a in attrs:
                     pos = line.find(a)
                     if pos >= 0:
-                        pos += len(a)+2
-                        print('%s="%s"' % (a, line[pos:line.find('"', pos)]), end=' ', file=out)
+                        pos += len(a) + 2
+                        print(
+                            '%s="%s"' % (a, line[pos:line.find('"', pos)]), end=' ', file=out)
                 hadOutput = True
             if "<emission" in line:
                 for a in attrs:
                     pos = line.find(a)
                     if pos >= 0:
-                        pos += len(a)+2
-                        print('%s="%s"' % (a, line[pos:line.find('"', pos)]), end=' ', file=out)
+                        pos += len(a) + 2
+                        print(
+                            '%s="%s"' % (a, line[pos:line.find('"', pos)]), end=' ', file=out)
         if hadOutput:
             print("/>", file=out)
         print("</tripinfos>", file=out)
@@ -357,11 +391,12 @@ def filterTripinfo(step, attrs):
         os.remove(inFile)
         os.rename(out.name, inFile)
 
+
 def assign_remaining_args(application, prefix, args):
     # assign remaining args [ prefix--o1 a1 prefix--o2 prefix--o3 a3  ...]
     # only handles long options!
     assigned = []
-    ## split into options and arguments
+    # split into options and arguments
     items = []
     item = None
     for arg in args:
@@ -371,12 +406,13 @@ def assign_remaining_args(application, prefix, args):
             item = [arg]
         else:
             if item == None:
-                sys.exit('Encounted argument "%s" without a preceeding option' % arg)
+                sys.exit(
+                    'Encounted argument "%s" without a preceeding option' % arg)
             item.append(arg)
     if item != None:
         items.append(item)
 
-    # assign to programs 
+    # assign to programs
     valid_options = set(get_long_option_names(application))
     for item in items:
         prefixed = item[0]
@@ -386,57 +422,66 @@ def assign_remaining_args(application, prefix, args):
                 assigned.append(option)
                 assigned += item[1:]
             else:
-                sys.exit('"%s" is not a valid option for "%s"' % (option, application))
+                sys.exit('"%s" is not a valid option for "%s"' %
+                         (option, application))
                 unassigned += item
 
     return assigned
 
+
 def get_basename(demand_file):
     basename = os.path.basename(demand_file)
     if 'alt' in basename:
-        return  basename[:-12]
+        return basename[:-12]
     elif 'trips' in basename:
         return basename[:-10]
     else:
         return basename[:basename.find(".")]
 
+
 def main(args=None):
     optParser = initOptions()
-    
+
     options, remaining_args = optParser.parse_args(args=args)
     if not options.net:
         optParser.error("Option --net-file is mandatory")
     if (not options.trips and not options.routes and not options.flows) or (options.trips and options.routes):
-        optParser.error("Either --trips, --flows, or --routes have to be given!")
+        optParser.error(
+            "Either --trips, --flows, or --routes have to be given!")
     duaBinary = sumolib.checkBinary("duarouter", options.path)
     if options.mesosim:
         sumoBinary = sumolib.checkBinary("meso", options.path)
     else:
         sumoBinary = sumolib.checkBinary("sumo", options.path)
     if options.addweights and options.weightmemory:
-        optParser.error("Options --addweights and --weight-memory are mutually exclusive.")
+        optParser.error(
+            "Options --addweights and --weight-memory are mutually exclusive.")
 
     # make sure BOTH binaries are callable before we start
     try:
         subprocess.call(duaBinary, stdout=subprocess.PIPE)
     except OSError:
-        sys.exit("Error: Could not locate duarouter (%s).\nMake sure its on the search path or set environment variable DUAROUTER_BINARY\n" % duaBinary)
+        sys.exit(
+            "Error: Could not locate duarouter (%s).\nMake sure its on the search path or set environment variable DUAROUTER_BINARY\n" % duaBinary)
     try:
         subprocess.call(sumoBinary, stdout=subprocess.PIPE)
     except OSError:
-        sys.exit("Error: Could not locate sumo (%s).\nMake sure its on the search path or set environment variable SUMO_BINARY\n" % sumoBinary)
+        sys.exit(
+            "Error: Could not locate sumo (%s).\nMake sure its on the search path or set environment variable SUMO_BINARY\n" % sumoBinary)
 
     sumo_args = assign_remaining_args(sumoBinary, 'sumo', remaining_args)
-    
+
     sys.stdout = sumolib.TeeFile(sys.stdout, open("stdout.log", "w+"))
     log = open("dua.log", "w+")
     if options.zip:
         if options.clean_alt:
-            sys.exit("Error: Please use either --zip or --clean-alt but not both.")
+            sys.exit(
+                "Error: Please use either --zip or --clean-alt but not both.")
         try:
             subprocess.call("7z", stdout=open(os.devnull, 'wb'))
         except:
-            sys.exit("Error: Could not locate 7z, please make sure its on the search path.")
+            sys.exit(
+                "Error: Could not locate 7z, please make sure its on the search path.")
         zipProcesses = {}
         zipLog = open("7zip.log", "w+")
     starttime = datetime.now()
@@ -455,16 +500,15 @@ def main(args=None):
         print('use externalgawron')
         edgesMap = {}
     if options.weightmemory:
-        costmemory = CostMemory('traveltime'
-                ,pessimism=options.pessimism
-                ,network_file=options.net
-                )
+        costmemory = CostMemory('traveltime', pessimism=options.pessimism, network_file=options.net
+                                )
     routesSuffix = ".xml"
     if options.binary:
         routesSuffix = ".sbx"
     if options.costmodifier != 'None':
         pyPath = os.path.abspath(os.path.dirname(sys.argv[0]))
-        sys.path.append(os.path.join(pyPath, "..", "..", "..", "..","..", "tools", "kkwSim"))
+        sys.path.append(
+            os.path.join(pyPath, "..", "..", "..", "..", "..", "tools", "kkwSim"))
         from kkwCostModifier import costModifier
         print('Use the cost modifier for KKW simulation')
 
@@ -472,7 +516,7 @@ def main(args=None):
         # load previous dump files when continuing a run
         print(">> Reassembling cost-memory from previous iteration steps")
         for step in range(0, options.firstStep):
-            dumpfile = get_dumpfilename(options, step,"dump")
+            dumpfile = get_dumpfilename(options, step, "dump")
             print(">>> Loading %s" % dumpfile)
             costmemory.load_costs(dumpfile, step, get_scale(options, step))
 
@@ -480,14 +524,16 @@ def main(args=None):
     for step in range(options.firstStep, options.lastStep):
         btimeA = datetime.now()
         print("> Executing step %s" % step)
-        
+
         router_demands = input_demands
         simulation_demands = input_demands
         # demand files have regular names based on the basename and the step
         if not (options.skipFirstRouting and step == 0):
-            simulation_demands = [get_basename(f) + "_%03i.rou%s" % (step, routesSuffix) for f in input_demands]
+            simulation_demands = [
+                get_basename(f) + "_%03i.rou%s" % (step, routesSuffix) for f in input_demands]
         if not ((options.skipFirstRouting and step == 1) or step == 0):
-            router_demands = [get_basename(f) + "_%03i.rou.alt%s" % (step-1, routesSuffix) for f in input_demands]
+            router_demands = [get_basename(
+                f) + "_%03i.rou.alt%s" % (step - 1, routesSuffix) for f in input_demands]
 
         if not (options.skipFirstRouting and step == options.firstStep):
             # call duarouter
@@ -495,14 +541,15 @@ def main(args=None):
                 print(">> Running router on %s" % router_input)
                 btime = datetime.now()
                 print(">>> Begin time: %s" % btime)
-                cfgname = writeRouteConf(step, options, router_input, output, options.routefile, initial_type)
+                cfgname = writeRouteConf(
+                    step, options, router_input, output, options.routefile, initial_type)
                 log.flush()
                 call([duaBinary, "-c", cfgname], log)
                 if options.clean_alt and not router_input in input_demands:
                     os.remove(router_input)
                 etime = datetime.now()
                 print(">>> End time: %s" % etime)
-                print(">>> Duration: %s" % (etime-btime))
+                print(">>> Duration: %s" % (etime - btime))
                 print("<<")
                 # use the external gawron
                 if options.externalgawron:
@@ -511,52 +558,63 @@ def main(args=None):
                         ecomeasure = options.ecomeasure
                     if step == options.firstStep + 1 and options.skipFirstRouting:
                         if options.caloldprob:
-                            calFirstRouteProbs("dump_000_%s.xml" % (options.aggregation), basename + "_001.rou.alt.xml",options.addweights,ecomeasure)
+                            calFirstRouteProbs("dump_000_%s.xml" % (
+                                options.aggregation), basename + "_001.rou.alt.xml", options.addweights, ecomeasure)
                         else:
-                            shutil.copy(basename + "_001.rou.alt.xml", basename + "_001.rou.galt.xml")
-                            shutil.copy(basename + "_001.rou.xml", basename + "_001.grou.xml")
+                            shutil.copy(
+                                basename + "_001.rou.alt.xml", basename + "_001.rou.galt.xml")
+                            shutil.copy(
+                                basename + "_001.rou.xml", basename + "_001.grou.xml")
                     if step == options.firstStep and not options.skipFirstRouting:
-                        shutil.copy(basename + "_000.rou.alt.xml", basename + "_000.rou.galt.xml")
-                        shutil.copy(basename + "_000.rou.xml", basename + "_000.grou.xml")
+                        shutil.copy(
+                            basename + "_000.rou.alt.xml", basename + "_000.rou.galt.xml")
+                        shutil.copy(
+                            basename + "_000.rou.xml", basename + "_000.grou.xml")
                     else:
                         print('step:', step)
                         print('get externalgawron')
-                        dumpfile = "dump_%03i_%s.xml" % (step-1, options.aggregation)
+                        dumpfile = "dump_%03i_%s.xml" % (
+                            step - 1, options.aggregation)
                         if (not options.skipFirstRouting) or (options.skipFirstRouting and step > 1):
-                            output, edgesMap = getRouteChoices(edgesMap,dumpfile,basename + "_%03i.rou.alt.xml" % step,options.net,options.addweights, options.gA, options.gBeta,step,ecomeasure)
+                            output, edgesMap = getRouteChoices(
+                                edgesMap, dumpfile, basename + "_%03i.rou.alt.xml" % step, options.net, options.addweights, options.gA, options.gBeta, step, ecomeasure)
 
         # simulation
         print(">> Running simulation")
         btime = datetime.now()
         print(">>> Begin time: %s" % btime)
         writeSUMOConf(sumoBinary, step, options, sumo_args,
-                ",".join(simulation_demands))   #  todo: change 'grou.xml'
+                      ",".join(simulation_demands))  # todo: change 'grou.xml'
         log.flush()
         call([sumoBinary, "-c", "iteration_%03i.sumocfg" % step], log)
         if options.tripinfoFilter:
             filterTripinfo(step, set(options.tripinfoFilter.split(",")))
         etime = datetime.now()
         print(">>> End time: %s" % etime)
-        print(">>> Duration: %s" % (etime-btime))
+        print(">>> Duration: %s" % (etime - btime))
         print("<<")
 
         if options.weightmemory:
             print(">> Smoothing edge weights")
             costmemory.load_costs(
-                    get_dumpfilename(options, step,"dump"), step, get_scale(options, step))
+                get_dumpfilename(options, step, "dump"), step, get_scale(options, step))
             costmemory.write_costs(get_weightfilename(options, step, "dump"))
             print(">>> Updated %s edges" % costmemory.loaded())
             print(">>> Decayed %s unseen edges" % costmemory.decayed())
-            print(">>> Error avg:%s mean:%s" % (costmemory.avg_error(), costmemory.mean_error()))
-            print(">>> Absolute Error avg:%s mean:%s" % (costmemory.avg_abs_error(), costmemory.mean_abs_error()))
+            print(">>> Error avg:%s mean:%s" %
+                  (costmemory.avg_error(), costmemory.mean_error()))
+            print(">>> Absolute Error avg:%s mean:%s" %
+                  (costmemory.avg_abs_error(), costmemory.mean_abs_error()))
 
         if options.costmodifier != 'None':
             currentDir = os.getcwd()
-            costModifier(get_weightfilename(options, step, "dump"), step, "dump", options.aggregation, currentDir, options.costmodifier, 'dua-iterate')
+            costModifier(get_weightfilename(options, step, "dump"), step, "dump",
+                         options.aggregation, currentDir, options.costmodifier, 'dua-iterate')
 
         if options.zip and step - options.firstStep > 1:
             # this is a little hackish since we zip and remove all files by glob, which may have undesired side effects
-            # also note that the 7z file does not have an "_" before the iteration number in order to be not picked up by the remove
+            # also note that the 7z file does not have an "_" before the
+            # iteration number in order to be not picked up by the remove
             for s in zipProcesses.keys():
                 if zipProcesses[s].poll() is not None:
                     for f in glob.glob("*_%03i*" % s):
@@ -566,7 +624,8 @@ def main(args=None):
                             print("Could not remove %s" % f, file=zipLog)
                     del zipProcesses[s]
             zipStep = step - 2
-            zipProcesses[zipStep] = subprocess.Popen(["7z", "a", "iteration%03i.7z" % zipStep] + glob.glob("*_%03i*" % zipStep), stdout=zipLog, stderr=zipLog)
+            zipProcesses[zipStep] = subprocess.Popen(
+                ["7z", "a", "iteration%03i.7z" % zipStep] + glob.glob("*_%03i*" % zipStep), stdout=zipLog, stderr=zipLog)
 
         converged = False
         if options.convDev:
@@ -577,11 +636,13 @@ def main(args=None):
                 count += 1
             avgTT.add(sum / count)
             relStdDev = avgTT.relStdDev(options.convIt)
-            print("< relative travel time deviation in the last %s steps: %.05f" % (min(avgTT.count(), options.convIt), relStdDev))
+            print("< relative travel time deviation in the last %s steps: %.05f" % (
+                min(avgTT.count(), options.convIt), relStdDev))
             if avgTT.count() >= options.convIt and relStdDev < options.convDev:
                 converged = True
-    
-        print("< Step %s ended (duration: %s)" % (step, datetime.now() - btimeA))
+
+        print("< Step %s ended (duration: %s)" %
+              (step, datetime.now() - btimeA))
         print("------------------\n")
 
         log.flush()
@@ -597,7 +658,7 @@ def main(args=None):
                     print("Could not remove %s" % f, file=zipLog)
         zipLog.close()
     print("dua-iterate ended (duration: %s)" % (datetime.now() - starttime))
-    
+
     log.close()
 
 if __name__ == "__main__":
