@@ -26,6 +26,8 @@ import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.io.DataInputStream;
 import java.io.DataOutputStream;
+import java.io.BufferedInputStream;
+import java.io.BufferedOutputStream;
 import java.io.IOException;
 import java.net.ConnectException;
 import java.net.InetAddress;
@@ -92,6 +94,7 @@ public class SumoTraciConnection {
 	private int randomSeed;
 	private Socket socket;
 	
+	/** The current simulation step, in seconds. */
 	private int currentSimStep;
 	private Process sumoProcess;
 	
@@ -180,6 +183,10 @@ public class SumoTraciConnection {
 	 * Creates an instance of this class and connects to a running instance of
 	 * SUMO.
 	 * 
+	 * <br/><br/> Note that TCP_NODELAY on the socket connection to SUMO will be
+	 * <b>enabled</b> by default. You can modify this behaviour later before the
+	 * call to <code>runServer()</code>.
+	 * 
 	 * @param addr
 	 *            the IP address of the machine where SUMO runs
 	 * @param port
@@ -191,6 +198,9 @@ public class SumoTraciConnection {
 	public SumoTraciConnection(InetAddress addr, int port) throws IOException,
 			InterruptedException {
 
+	   // Set TCP_NODELAY as enabled by default.
+	   enableTcpNoDelay();
+	   
 		tryConnect(addr, port, null);
 		postConnect();
 	}
@@ -274,20 +284,61 @@ public class SumoTraciConnection {
 		}
 	}
 
-	private boolean tryConnectOnce(InetAddress addr, int port) throws UnknownHostException, IOException {
-		socket = new Socket();
-        socket.setTcpNoDelay(Boolean.getBoolean(TCP_NODELAY_PROPERTY));
-		try {
-			socket.connect(new InetSocketAddress(addr, port));
-			return true;
-		} catch (ConnectException ce) {
-			return false;
-		}
+	/**
+	 * Is TCP_NODELAY activated for the socket connection to SUMO?
+	 * 
+	 * @return true if active and false otherwise.
+	 */
+	public boolean isTcpNoDelayActive()
+   {
+      return Boolean.getBoolean(TCP_NODELAY_PROPERTY);
+   }
+   
+	/**
+    * Enables TCP_NODELAY in the socket connection to SUMO
+    */
+   public void enableTcpNoDelay()
+   {
+      setTcpNoDelay(true);      
+   }
+   
+   /**
+    * Disables TCP_NODELAY in the socket connection to SUMO
+    */
+   public void disableTcpNoDelay()
+   {
+      setTcpNoDelay(false);
+   }
+   
+   /**
+    * Forcibly set TCP_NODELAY_PROPERTY
+    *  
+    * @param on If true TCP_NODELAY will be turned on. Else turned off.
+    */
+   public void setTcpNoDelay(boolean on)
+   {
+      System.setProperty(TCP_NODELAY_PROPERTY, String.valueOf(on));
+   }
+   
+	private boolean tryConnectOnce(InetAddress addr, int port)
+	      throws UnknownHostException, IOException
+   {
+	  boolean tcpNoDelay = Boolean.getBoolean(TCP_NODELAY_PROPERTY);
+
+	  socket = new Socket();
+	  socket.setTcpNoDelay(tcpNoDelay);
+	  
+	  try {
+	    socket.connect(new InetSocketAddress(addr, port));
+	    return true;
+	  } catch (ConnectException ce) {
+	    return false;
+	  }
 	}
 	
 	private void postConnect() throws IOException {
-		dis = new DataInputStream(socket.getInputStream());
-		dos = new DataOutputStream(socket.getOutputStream());
+		dis = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
+		dos = new DataOutputStream(new BufferedOutputStream(socket.getOutputStream()));
 
 		closeQuery = new CloseQuery(dis, dos);
 		simData = new SimulationData(dis, dos);
@@ -495,7 +546,7 @@ public class SumoTraciConnection {
 		/*
 		 * forces querying of vehicle IDs when requested
 		 */
-		simData.nextStep(currentSimStep);
+		simData.nextStep(currentSimStep * 1000);
 		
 		/*
 		 * constructs a multi-query that advances one step, reads the list of
@@ -506,7 +557,7 @@ public class SumoTraciConnection {
 		MultiQuery multi = new MultiQuery(dos, dis);
 		{ // begin multi-query
 			SimStepQuery ssq = new SimStepQuery(dis, dos);
-			ssq.setTargetTime(currentSimStep);
+			ssq.setTargetTime(currentSimStep * 1000);
 			multi.add(ssq);
 
 			multi.add(vehicleListQuery);
