@@ -4,7 +4,7 @@
 @author  Daniel Krajzewicz
 @author  Michael Behrisch
 @date    2009-05-27
-@version $Id: vissim_parseRoutes.py 17235 2014-11-03 10:53:02Z behrisch $
+@version $Id: vissim_parseRoutes.py 18596 2015-08-04 11:33:00Z ait-mob-dts $
 
 
 Parses routes given in the Vissim file (first parameter) as (in-)flows and 
@@ -21,7 +21,7 @@ A third parameter may be given to change the random number seed (default=42)
  variable (see below).
 
 SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-Copyright (C) 2009-2014 DLR (http://www.dlr.de/) and contributors
+Copyright (C) 2009-2015 DLR (http://www.dlr.de/) and contributors
 
 This file is part of SUMO.
 SUMO is free software; you can redistribute it and/or modify
@@ -31,7 +31,7 @@ the Free Software Foundation; either version 3 of the License, or
 """
 
 edgemap = {}
-edgemap["203"] = "203[0]"
+#edgemap["203"] = "203[0]"
 
 
 SEED = 42
@@ -40,60 +40,64 @@ SEED = 42
 import sys
 from random import *
 
+
 def getName(vals, beg):
     name = vals[beg]
-    while name.count('"')!=2:
+    while name.count('"') != 2:
         beg = beg + 1
         name = name + " " + vals[beg]
     return name.replace('"', '')
-        
+
 
 def parseInFlow(inflow, suggested_name):
     vals = inflow.split()
     i = vals.index("NAME")
-    name = getName(vals, i+1)
-    if (name==""): name=suggested_name #SUMO can't cope with empty IDs, unlike VISSIM
+    name = getName(vals, i + 1)
+    if (name == ""):
+        name = suggested_name  # SUMO can't cope with empty IDs, unlike VISSIM
     i = vals.index("STRECKE", i)
-    strecke = vals[i+1]
-    i = vals.index("Q", i+1)
-    if vals[i+1]=="EXAKT":
-        q = float(vals[i+2])
+    strecke = vals[i + 1]
+    i = vals.index("Q", i + 1)
+    if vals[i + 1] == "EXAKT":
+        q = float(vals[i + 2])
     else:
-        q = float(vals[i+1])
-    i = vals.index("ZUSAMMENSETZUNG", i+1)
-    zusammensetzung = vals[i+1]
+        q = float(vals[i + 1])
+    i = vals.index("ZUSAMMENSETZUNG", i + 1)
+    zusammensetzung = vals[i + 1]
     i = vals.index("ZEIT", i)
     i = vals.index("VON", i)
-    von = float(vals[i+1])
+    von = float(vals[i + 1])
     i = vals.index("BIS")
-    bis = float(vals[i+1])
+    bis = float(vals[i + 1])
     return (name, strecke, q, von, bis)
 
+
 def parseRouteDecision(rd):
     vals = rd.split()
     i = vals.index("STRECKE")
-    strecke = vals[i+1]
-    i = vals.index("ZEIT", i) # probably, we would normally already here to iterate over time spans...
-    i = vals.index("VON", i+1)
-    von = float(vals[i+1])
-    i = vals.index("BIS", i+1)
-    bis = float(vals[i+1])
+    strecke = vals[i + 1]
+    # probably, we would normally already here to iterate over time spans...
+    i = vals.index("ZEIT", i)
+    i = vals.index("VON", i + 1)
+    von = float(vals[i + 1])
+    i = vals.index("BIS", i + 1)
+    bis = float(vals[i + 1])
     if "ROUTE" in vals:
-        i = vals.index("ROUTE", i+1)
+        i = vals.index("ROUTE", i + 1)
     else:
         i = 0
     sumAnteil = 0
     routes = []
-    while i>0:
-        r_id = vals[i+1]
-        i = vals.index("STRECKE", i+1)
-        r_ziel = vals[i+1]
-        i = vals.index("ANTEIL", i+1)
-        r_anteil = float(vals[i+1])
+    while i > 0:
+        r_id = vals[i + 1]
+        i = vals.index("STRECKE", i + 1)
+        r_ziel = vals[i + 1]
+        i = vals.index("ANTEIL", i + 1)
+        r_anteil = float(vals[i + 1])
         r_edges = []
-        if vals[i+2]=="UEBER":
+        if vals[i + 2] == "UEBER":
             i = i + 3
-            while i<len(vals) and vals[i]!="ROUTE":
+            while i < len(vals) and vals[i] != "ROUTE":
                 if vals[i] in edgemap:
                     r_edges.append(edgemap[vals[i]])
                 else:
@@ -108,13 +112,12 @@ def parseRouteDecision(rd):
             r_edges.append(edgemap[r_ziel])
         else:
             r_edges.append(r_ziel)
-        routes.append( (r_id, r_anteil, r_edges) )
+        routes.append((r_id, r_anteil, r_edges))
         sumAnteil = sumAnteil + r_anteil
-        if i>=len(vals):
+        if i >= len(vals):
             i = -1
-    return ( strecke, sumAnteil, von, bis, routes )
+    return (strecke, sumAnteil, von, bis, routes)
 
-        
 
 def sorter(idx):
     def t(i, j):
@@ -143,24 +146,24 @@ haveInFlow = False
 currentInFlow = ""
 for line in fd:
     # put all route decision ("ROUTENENTSCHEIDUNG") to a list (routeDecisions)
-    if line.find("ROUTENENTSCHEIDUNG")==0:
+    if line.find("ROUTENENTSCHEIDUNG") == 0:
         if haveRouteDecision:
             routeDecisions.append(" ".join(currentRouteDecision.split()))
         haveRouteDecision = True
         currentRouteDecision = ""
-    elif line[0]!=' ':
+    elif line[0] != ' ':
         if haveRouteDecision:
             routeDecisions.append(" ".join(currentRouteDecision.split()))
         haveRouteDecision = False
     if haveRouteDecision:
         currentRouteDecision = currentRouteDecision + line
     # put all inflows ("ZUFLUSS") to a list (inflows)
-    if line.find("ZUFLUSS")==0:
+    if line.find("ZUFLUSS") == 0:
         if haveInFlow:
             inflows.append(" ".join(currentInFlow.split()))
         haveInFlow = True
         currentInFlow = ""
-    elif line[0]!=' ':
+    elif line[0] != ' ':
         if haveInFlow:
             inflows.append(" ".join(currentInFlow.split()))
         haveInFlow = False
@@ -174,9 +177,10 @@ fdo.write("<flowdefs>\n")
 flow_sn = 0
 for inflow in inflows:
     (name, strecke, q, von, bis) = parseInFlow(inflow, str(flow_sn))
-    fdo.write('    <flow id="' + name + '" from="' + strecke + '" begin="' + str(von) + '" end="' + str(bis) + '" no="' + str(int(q)) + '"/>\n')
+    fdo.write('    <flow id="' + name + '" from="' + strecke + '" begin="' +
+              str(von) + '" end="' + str(bis) + '" no="' + str(int(q)) + '"/>\n')
     flow_sn = flow_sn + 1
-fdo.write("</flowdefs>\n")    
+fdo.write("</flowdefs>\n")
 fdo.close()
 
 # process combinations
@@ -185,10 +189,10 @@ print "Building routes..."
 edges2check = {}
 edgesSumFlows = {}
 for rd in routeDecisions:
-    ( strecke, sumAnteil, von, bis, routes ) = parseRouteDecision(rd)
+    (strecke, sumAnteil, von, bis, routes) = parseRouteDecision(rd)
     if strecke not in edges2check:
         edgesSumFlows[strecke] = sumAnteil
-        edges2check[strecke] = ( von, bis, routes )
+        edges2check[strecke] = (von, bis, routes)
 # compute emissions with routes
 emissions = []
 flow_sn = 0
@@ -198,14 +202,14 @@ for inflow in inflows:
     if strecke in edges2check:
         routes = edges2check[strecke]
         for vi in range(0, int(q)):
-            t = von + float(bis-von) / float(q) * float(vi)
-            fi = random() * edgesSumFlows[strecke] 
+            t = von + float(bis - von) / float(q) * float(vi)
+            fi = random() * edgesSumFlows[strecke]
             edges = []
             ri = 0
             rid = ""
-            while len(edges)==0 and ri<len(routes[2]) and fi>=0:
+            while len(edges) == 0 and ri < len(routes[2]) and fi >= 0:
                 fi = fi - routes[2][ri][1]
-                if fi<0:
+                if fi < 0:
                     edges = routes[2][ri][2]
                     rid = routes[2][ri][0]
                 ri = ri + 1
@@ -216,7 +220,7 @@ for inflow in inflows:
             else:
                 if strecke not in edges:
                     edges.insert(0, strecke)
-            emissions.append( ( int(t), id, edges ) )
+            emissions.append((int(t), id, edges))
 # sort emissions
 print "Sorting routes..."
 emissions.sort(sorter(0))
@@ -226,12 +230,9 @@ print "Writing routes..."
 fdo = open(sys.argv[2] + ".rou.xml", "w")
 fdo.write("<routes>\n")
 for emission in emissions:
-    if len(emission[2])<2:
-        continue;
-    fdo.write('    <vehicle id="' + emission[1] + '" depart="' + str(emission[0]) + '"><route edges="' + " ".join(emission[2]) + '"/></vehicle>\n');
+    if len(emission[2]) < 2:
+        continue
+    fdo.write('    <vehicle id="' + emission[1] + '" depart="' + str(
+        emission[0]) + '"><route edges="' + " ".join(emission[2]) + '"/></vehicle>\n')
 fdo.write("</routes>\n")
 fdo.close()
-
-
-
-
